<?php

namespace comment\event;

use comment\model\comment;
use comment\model\comment_like_data;

class handler
{
	public const DEPEND_KEY_COMMENT_LIST = 'depend_key_comment_list_(:module_namespace)_(:resource_id)';

	/**
	 * event_member_delete 事件处理器
	 * @param int|array $id
	 * @return void
	 * @throws \Exception
	 */
	public function event_member_delete_handler(int|array $id): void
	{
		if (empty($id)) {
			return;
		}

		if (!is_array($id)) {
			$id = [$id];
		}

		/**
		 * @var comment $comment_model
		 * @var comment_like_data $comment_like_data_model
		 */
		$comment_model = model(comment::class);
		$comment_like_data_model = model(comment_like_data::class);

		$rows = $comment_model->select('id, parent_id, status')->where_in('uid', $id)->query()->fetch_all_array();
		foreach ($rows as $row) {
			if ((int)$row['parent_id'] > 0) { //reply
				if ((int)$row['status'] === 1) {
					$comment_model->child_count_increment($row['parent_id'], -1);
					$comment_model->remove_child_id($row['parent_id'], $row['id']);
				}
			} else { //comment
				//<xxx> delete all reply
				$comment_model->where(['parent_id' => $row['id']])->delete();
			}
			$comment_model->delete($row['id']);
		}

		$comment_like_data_model->where_in('uid', $id)->delete();
	}

	/**
	 * event_module_uninstall 事件处理器
	 * @param string $module_namespace
	 * @return void
	 * @throws \Exception
	 */
	public function event_module_uninstall_handler(string $module_namespace): void
	{
		if ($module_namespace == 'comment') {
			return;
		}

		/**
		 * @var comment $comment_model
		 * @var comment_like_data $comment_like_data_model
		 */
		$comment_model = model(comment::class);
		$comment_like_data_model = model(comment_like_data::class);

		$rows = $comment_model->select('id')->where(['module_namespace' => $module_namespace])->query()->fetch_all_array();
		if (!empty($rows)) {
			$ids = array_column($rows, 'id');
			if (!empty($ids)) {
				$comment_like_data_model->where_in('cid', $ids)->delete();
			}
			$comment_model->where(['module_namespace' => $module_namespace])->delete();
		}
	}

	/**
	 * insert_from_module 事件处理器
	 * @param string $module_namespace
	 * @param int $resource_id
	 * @return void
	 * @throws \Exception
	 */
	public function event_insert_from_module_handler(string $module_namespace, int $resource_id): void
	{
		$this->update_comment_list_depend($module_namespace, $resource_id);
	}

	/**
	 * update_from_module 事件处理器
	 * @param string $module_namespace
	 * @param int $resource_id
	 * @return void
	 * @throws \Exception
	 */
	public function event_update_from_module_handler(string $module_namespace, int $resource_id): void
	{
		$this->update_comment_list_depend($module_namespace, $resource_id);
	}

	/**
	 * delete_from_module 事件处理器
	 * @param string $module_namespace
	 * @param int|array $resource_id
	 * @return void
	 * @throws \Exception
	 */
	public function event_delete_from_module_handler(string $module_namespace, int|array $resource_id): void
	{
		$this->update_comment_list_depend($module_namespace, $resource_id);
		$this->delete_resource_comment($module_namespace, $resource_id);
	}

	/**
	 * 更新评论列表依赖
	 * @param string $module_namespace
	 * @param int|array $resource_id
	 * @return void
	 * @throws \Exception
	 */
	private function update_comment_list_depend(string $module_namespace, int|array $resource_id): void
	{
		if ($module_namespace != 'comment' || empty($resource_id)) {
			return;
		}

		/**
		 * @var comment $comment_model
		 */
		$comment_model = model(comment::class);

		$ids = is_array($resource_id) ? $resource_id : [$resource_id];
		$rows = $comment_model->select('module_namespace, resource_id')->where_in('id', $ids)->query()->fetch_all_array();
		if (!empty($rows)) {
			foreach ($rows as $row) {
				$depend_key = strtr(self::DEPEND_KEY_COMMENT_LIST, [
					'(:module_namespace)' => $row['module_namespace'],
					'(:resource_id)' => $row['resource_id']
				]);
				\phpvod::$app->cache->get_handler()->set($depend_key, TIMESTAMP); //更新依赖
			}
		}
	}

	/**
	 * 删除资源评论
	 * @param string $module_namespace
	 * @param int|array $resource_id
	 * @return void
	 * @throws \Exception
	 */
	private function delete_resource_comment(string $module_namespace, int|array $resource_id): void
	{
		if ($module_namespace == 'comment' || empty($resource_id)) {
			return;
		}

		/**
		 * @var comment $comment_model
		 * @var comment_like_data $comment_like_data_model
		 */
		$comment_model = model(comment::class);
		$comment_like_data_model = model(comment_like_data::class);

		if (!is_array($resource_id)) {
			$resource_id = [$resource_id];
		}

		$rows = $comment_model->select('id')->where(['module_namespace' => $module_namespace])->where_in('resource_id', $resource_id)->query()->fetch_all_array();
		if (!empty($rows)) {
			$ids = array_column($rows, 'id');
			if (!empty($ids)) {
				$comment_like_data_model->where_in('cid', $ids)->delete();
			}
			$comment_model->where(['module_namespace' => $module_namespace])->where_in('resource_id', $resource_id)->delete();
		}
	}
}