<?php

namespace comment\model;

use phpvod\widget\cravatar;
use phpvod\widget\form\admin_form;
use phpvod\widget\pagination\pagination;

class comment extends \phpvod\base\model
{
	protected string $table = 'pv_comment';
	protected ?string $primary_key = 'id';
	protected array $allowed_fields = ['uid', 'module_namespace', 'resource_id', 'content', 'parent_id', 'child_id', 'child_count', 'at_user', '`like`', 'dislike', 'created_at', 'ip', 'status'];
	protected ?string $created_field = 'created_at';

	public function get_data(array $params): array
	{
		//username
		if (isset($params['username'])) {
			if ($params['username'] == lang('app.guest')) {
				$this->where(['c.uid' => 0]);
			} else {
				$this->where(['m.username' => $params['username']]);
			}
		}
		//module_namespace
		if (isset($params['module_namespace'])) {
			$this->where(['c.module_namespace' => $params['module_namespace']]);
		}
		//type
		if (isset($params['type'])) {
			if ($params['type'] == 'comment') {
				$this->where(['c.parent_id' => 0]);
			} elseif ($params['type'] == 'reply') {
				$this->where(['c.parent_id!=' => 0]);
			}
		}
		//content
		if (isset($params['content'])) {
			$this->where('c.content LIKE :content', ['content' => '%' . $params['content'] . '%']);
		}
		//status
		if (isset($params['status']) && is_numeric($params['status'])) {
			$this->where(['c.status' => $params['status']]);
		}

		$total = (int)$this->select('count(c.id)')
			->from('pv_comment AS c LEFT JOIN pv_member AS m ON c.uid=m.id')->query(false)->fetch_column();

		$this->order_by($params['order_by']);
		$this->limit($params['offset_limit']);

		$rows = $this->select('c.*,m.username')
			->from('pv_comment AS c LEFT JOIN pv_member AS m ON c.uid=m.id')
			->query()->fetch_all_by_callback(function (array $row) {
				$row['content'] = htmlspecialchars($row['content']);
				$row['status'] = admin_form::get_instance()->switch((boolean)$row['status'], ['class' => 'form-check-input', 'role' => 'switch', 'value' => '1']);
				$row['created_at'] = timetostr($row['created_at'], 'Y-m-d H:i:s');
				return $row;
			});

		return ['total' => $total, 'rows' => $rows];
	}

	/**
	 * 更新状态
	 * @param int $id ID
	 * @return int|false 成功返回更新后的状态，失败返回false
	 * @throws \Exception
	 */
	public function change_status(int $id): int|false
	{
		$this->update(['status' => 'CASE status WHEN 0 THEN 1 ELSE 0 END'], $id, ['status']);
		$row = $this->select("id, parent_id, status")->find($id)->fetch_array();

		//回复审核状态
		if (is_array($row) && (int)$row['parent_id'] > 0) {
			if ((int)$row['status'] === 1) {
				$this->child_count_increment($row['parent_id']);
			} else {
				$this->child_count_increment($row['parent_id'], -1);
				$this->remove_child_id($row['parent_id'], $row['id']);
			}
		}

		return is_array($row) ? (int)$row['status'] : false;
	}

	/**
	 * 增加或减少子评论数量
	 * @param int $id
	 * @param int $increment
	 * @return void
	 */
	public function child_count_increment(int $id, int $increment = 1): void
	{
		$this->update(['child_count' => 'child_count+' . $increment], $id, ['child_count']);
	}

	/**
	 * 移除推荐回复
	 * @param int $id
	 * @param int $child_id
	 * @return void
	 */
	public function remove_child_id(int $id, int $child_id): void
	{
		$this->update(['child_id' => "TRIM(BOTH ',' FROM REPLACE(CONCAT(',', child_id, ','), CONCAT(',', '$child_id', ','), ','))"], $id, ['child_id']);
	}

	/**
	 * 增加或减少喜欢、不喜欢数量
	 * @param int $id
	 * @param int $like_increment
	 * @param int $dislike_increment
	 * @return void
	 */
	public function like_increment(int $id, int $like_increment, int $dislike_increment): void
	{
		$this->update(['`like`' => '`like`+' . $like_increment, 'dislike' => 'dislike+' . $dislike_increment], $id, ['`like`', 'dislike']);
	}

	/**
	 * 获取资源评论
	 * @param string $module_namespace
	 * @param int $resource_id
	 * @param string $order_by
	 * @param int $page_number
	 * @param int $items_per_page
	 * @return array
	 */
	public function get_resource_comment(string $module_namespace, int $resource_id, string $order_by = 'id DESC', int $page_number = 1, int $items_per_page = 10): array
	{
		$where = [
			'c.module_namespace' => $module_namespace,
			'c.resource_id' => $resource_id,
			'c.parent_id' => 0,
			'c.status' => 1,
		];

		$page_number < 1 && $page_number = 1;
		$items_per_page < 1 && $items_per_page = 10;
		$limit = ($page_number - 1) * $items_per_page . ',' . $items_per_page;

		$data = $this->get_comment($where, $order_by, $limit);

		// 子评论
		$child_id = array_reduce($data['rows'], function (string $carry, array $row) {
			if (!empty($row['child_id'])) {
				$carry .= $carry === '' ? $row['child_id'] : ',' . $row['child_id'];
			}
			return $carry;
		}, '');

		if ($child_id != '') {
			$cravatar = cravatar::get_instance();
			$child_id_array = explode(',', $child_id);
			$child_data = $this->select('c.*,m.username,m.email')
				->from('pv_comment AS c LEFT JOIN pv_member AS m ON c.uid=m.id')
				->where_in('c.id', $child_id_array)->query()->fetch_all_array();
			foreach ($child_data as $row) {
				$row['avatar'] = $cravatar->avatar($row['email']);
				isset($data['rows'][$row['parent_id']]) && $data['rows'][$row['parent_id']]['child_comment'][] = $row;
			}
		}

		// 分页
		$pagination = new pagination($data['total'], $items_per_page, $page_number, "comment.get_resource_comment((:num))");
		$pagination->set_template('default_js');
		$data['pagination'] = $pagination;

		return $data;
	}

	/**
	 * 获取子评论
	 * @param int $parent_id
	 * @param string $order_by
	 * @param int $page_number
	 * @param int $items_per_page
	 * @return array
	 */
	public function get_child_comment(int $parent_id, string $order_by = 'id DESC', int $page_number = 1, int $items_per_page = 10): array
	{
		$where = [
			'c.parent_id' => $parent_id,
			'c.status' => 1,
		];

		$page_number < 1 && $page_number = 1;
		$items_per_page < 1 && $items_per_page = 10;
		$limit = ($page_number - 1) * $items_per_page . ',' . $items_per_page;

		$data = $this->get_comment($where, $order_by, $limit);

		// 分页
		$pagination = new pagination($data['total'], $items_per_page, $page_number, "comment.get_child_comment(this,$parent_id,(:num),$items_per_page)");
		$pagination->set_template('default_js');
		$data['pagination'] = $pagination;

		return $data;
	}

	private function get_comment(array $where = [], ?string $order_by = null, ?string $limit = null): array
	{
		!empty($where) && $this->where($where);
		$total = (int)$this->select('count(c.id)')->from('pv_comment AS c')->query(false)->fetch_column();

		$this->order_by($order_by);
		$this->limit($limit);

		$cravatar = cravatar::get_instance();
		$rows = $this->select('c.*,m.username,m.email')
			->from('pv_comment AS c LEFT JOIN pv_member AS m ON c.uid=m.id')
			->query()->fetch_all_by_callback(function (array $row) use ($cravatar) {
				$row['content'] = htmlspecialchars($row['content']);
				$row['avatar'] = $cravatar->avatar($row['email']);
				return ['key' => $row['id'], 'value' => $row];
			});

		return ['total' => $total, 'rows' => $rows];
	}
}