<?php

namespace comment\service;

use comment\library\config;
use phpvod\component\user_auth\user_auth;
use phpvod\event\event;
use venus\cache\depend\cache_depend;
use venus\cache\handler;
use venus\http\request;

class comment
{
	private handler $cache_handler;
	private request $request;
	private user_auth $user_auth;
	private \comment\model\comment $comment_model;
	private \comment\model\comment_like_data $comment_like_data_model;

	/** @noinspection PhpFieldAssignmentTypeMismatchInspection */
	public function __construct()
	{
		$this->cache_handler = \phpvod::$app->cache->get_handler();
		$this->request = \phpvod::$app->request;
		$this->user_auth = \phpvod::$app->user_auth;
		$this->comment_model = model(\comment\model\comment::class);
		$this->comment_like_data_model = model(\comment\model\comment_like_data::class);
	}

	/**
	 * 获取缓存键
	 * @param array $params
	 * @return string
	 */
	private function get_cache_key(array $params): string
	{
		return 'comment_list_' . md5(serialize($params));
	}

	/**
	 * 读缓存
	 * @param string $key
	 * @return array|null
	 */
	private function get_cache_data(string $key): array|null
	{
		return $this->cache_handler->get($key);
	}

	/**
	 * 写缓存
	 * @param string $key
	 * @param array $value
	 * @param string|null $depend_key
	 * @return void
	 * @throws \Exception
	 */
	private function set_cache_data(string $key, array $value, ?string $depend_key): void
	{
		$depend = !is_null($depend_key) ? new cache_depend($depend_key) : null;
		$this->cache_handler->set($key, $value, 0, $depend);
	}

	/**
	 * 获取资源评论
	 * @param string $module_namespace
	 * @param int $resource_id
	 * @param string $order_by
	 * @param int $page_number
	 * @param int $items_per_page
	 * @return array
	 * @throws \Exception
	 */
	public function get_resource_comment(string $module_namespace, int $resource_id, string $order_by = 'id DESC', int $page_number = 1, int $items_per_page = 10): array {
		//get_cache_data
		$key = $this->get_cache_key(func_get_args());
		$use_cache = config::get_instance()->get('cache', 'use_cache') == '1'; //是否使用缓存(bool)
		if($use_cache) {
			$cache_data = $this->get_cache_data($key);
			if(!is_null($cache_data)) {
				return $cache_data;
			}
		}

		$value = $this->comment_model->get_resource_comment($module_namespace, $resource_id, $order_by, $page_number, $items_per_page);

		//set_cache_data
		if($use_cache) {
			$depend_key = strtr(\comment\event\handler::DEPEND_KEY_COMMENT_LIST, [
				'(:module_namespace)' => $module_namespace,
				'(:resource_id)' => $resource_id
			]);
			$this->set_cache_data($key, $value, $depend_key);
		}

		return $value;
	}

	/**
	 * 获取子评论
	 * @param int $parent_id
	 * @param string $order_by
	 * @param int $page_number
	 * @param int $items_per_page
	 * @return array
	 * @throws \Exception
	 */
	public function get_child_comment(int $parent_id, string $order_by = 'id DESC', int $page_number = 1, int $items_per_page = 10): array {
		//get_cache_data
		$key = $this->get_cache_key(func_get_args());
		$use_cache = config::get_instance()->get('cache', 'use_cache') == '1'; //是否使用缓存(bool)
		if($use_cache) {
			$cache_data = $this->get_cache_data($key);
			if(!is_null($cache_data)) {
				return $cache_data;
			}
		}

		$value = $this->comment_model->get_child_comment($parent_id, 'id ASC', $page_number, $items_per_page);

		//set_cache_data
		if($use_cache) {
			$row = $this->comment_model->select('module_namespace, resource_id')->where(['id' => $parent_id])->query()->fetch_array();
			if(is_array($row)) {
				$depend_key = strtr(\comment\event\handler::DEPEND_KEY_COMMENT_LIST, [
					'(:module_namespace)' => $row['module_namespace'],
					'(:resource_id)' => $row['resource_id']
				]);
				$this->set_cache_data($key, $value, $depend_key);
			}
		}

		return $value;
	}

	/**
	 * 新增评论
	 * @return array
	 */
	public function add(): array
	{
		$data = $this->request->post('data');
		$data['content'] = trim($data['content']);

		//检测用户是否有发布评论的权限
		$allow_post_comment = $this->user_auth->get_frontend_permission('comment:allow_post_comment');
		if ($allow_post_comment != '1') {
			return ['id' => 0, 'msg' => lang('app.permission_denied')];
		}

		//用户发布评论是否需要审核
		$need_review = $this->user_auth->get_frontend_permission('comment:need_review');
		$data['status'] = $need_review == '1' ? 0 : 1;

		$data['uid'] = $this->user_auth->frontend_is_logged_in() ? $this->user_auth->get_frontend_identity('id') : 0; //0=guest
		$data['ip'] = $this->request->get_ip();
		if ($this->comment_model->insert($data) > 0) {
			$comment_id = $this->comment_model->insert_id();
			//event
			event()->trigger(event::EVENT_INSERT_FROM_MODULE, 'comment', $comment_id);
			if ($data['status'] == 1) {
				$data['parent_id'] > 0 && $this->comment_model->child_count_increment($data['parent_id']);
				return ['id' => 1, 'msg' => lang('home.post_success'), 'data' => [
					'comment_id' => $comment_id,
					'content' => htmlspecialchars($data['content']),
					'created_at' => timetostr(TIMESTAMP, 'Y-m-d H:i', true)
				]];
			} else {
				return ['id' => 2, 'msg' => lang('home.post_success_wait_for_review')];
			}
		}

		return ['id' => 0, 'msg' => lang('home.post_fail')];
	}

	/**
	 * 删除评论
	 * @param int $id
	 * @return void
	 * @throws \Exception
	 */
	public function delete(int $id): void
	{
		$row = $this->comment_model->select('parent_id, status')->find($id)->fetch_array();

		if ((int)$row['parent_id'] > 0 && (int)$row['status'] === 1) {
			$this->comment_model->child_count_increment($row['parent_id'], -1);
			$this->comment_model->remove_child_id($row['parent_id'], $id);
		}

		$this->comment_model->delete($id);
		$this->comment_like_data_model->where(['cid' => $id])->delete();

		//event
		event()->trigger(event::EVENT_DELETE_FROM_MODULE, 'comment', $id);
	}

	/**
	 * 喜欢、不喜欢
	 * @return array
	 * @throws \Exception
	 */
	public function like(): array
	{
		//检测用户是否登录
		if (!$this->user_auth->frontend_is_logged_in()) {
			return ['id' => 0, 'msg' => lang('app.please_login_first')];
		}

		$uid = $this->user_auth->get_frontend_identity('id'); //会员ID
		$cid = (int)$this->request->post('cid'); //评论ID
		$value = (int)$this->request->post('value'); //喜欢1、不喜欢2
		$parent_child_id = $this->request->post('parent_child_id'); //父评论child_id

		//查询评论数据
		$comment = $this->comment_model->select('parent_id, `like`,dislike')->find($cid)->fetch_array();
		if (!$comment) {
			return ['id' => 0, 'msg' => lang('app.invalid_params')];
		}
		$parent_id = (int)$comment['parent_id'];
		$like = (int)$comment['like'];
		$dislike = (int)$comment['dislike'];

		//查询喜欢、不喜欢数据
		$data = $this->comment_like_data_model->select('id,value')->where(['cid' => $cid, 'uid' => $uid])->query()->fetch_array();
		if (!$data) {
			if ($value === 1) {
				$this->comment_model->like_increment($cid, 1, 0);
				$like++;
			} else {
				$this->comment_model->like_increment($cid, 0, 1);
				$dislike++;
			}
			$this->comment_like_data_model->insert(['cid' => $cid, 'uid' => $uid, 'value' => $value]);
		} else {
			$data['value'] = (int)$data['value']; //0 or 1 or 2
			if ($data['value'] === 0) {
				if ($value === 1) {
					$this->comment_model->like_increment($cid, 1, 0);
					$like++;
				} else {
					$this->comment_model->like_increment($cid, 0, 1);
					$dislike++;
				}
				$this->comment_like_data_model->update(['value' => $value], $data['id']);
			} elseif ($data['value'] === 1) {
				if ($value === 1) {
					$this->comment_model->like_increment($cid, -1, 0);
					$this->comment_like_data_model->update(['value' => 0], $data['id']);
					$like--;
				} else {
					$this->comment_model->like_increment($cid, -1, 1);
					$this->comment_like_data_model->update(['value' => 2], $data['id']);
					$like--;
					$dislike++;
				}
			} elseif ($data['value'] === 2) {
				if ($value === 1) {
					$this->comment_model->like_increment($cid, 1, -1);
					$this->comment_like_data_model->update(['value' => 1], $data['id']);
					$like++;
					$dislike--;
				} else {
					$this->comment_model->like_increment($cid, 0, -1);
					$this->comment_like_data_model->update(['value' => 0], $data['id']);
					$dislike--;
				}
			}
		}

		//设置父评论child_id
		$min_like = 1; //回复喜欢数量到达该值即为优质回复
		$max_best = 3; //每条评论最多显示的优质回复数量
		$parent_child_id_list = empty($parent_child_id) ? [] : explode(',', $parent_child_id);
		if ($parent_id > 0 && $like >= $min_like && count($parent_child_id_list) < $max_best && !in_array($cid, $parent_child_id_list)) {
			$child_id = $this->comment_model->select('child_id')->find($parent_id)->fetch_column();
			if ($child_id !== false && (string)$child_id === $parent_child_id) { //check parent_child_id
				$parent_child_id .= empty($parent_child_id) ? $cid : ',' . $cid;
				$this->comment_model->update(['child_id' => $parent_child_id], $parent_id);
			}
		}

		//event
		event()->trigger(event::EVENT_UPDATE_FROM_MODULE, 'comment', $cid);
		return ['id' => 1, 'msg' => lang('app.operation_succeeded'), 'data' => ['like' => $like, 'dislike' => $dislike, 'parent_child_id' => $parent_child_id]];
	}

}