<?php

namespace phpvod\base;

use config\validation\validator;
use phpvod\component\user_auth\user_auth;
use phpvod\model\admin;
use config\validation\admin\bootstrap_table;
use venus\application\exception\controller_exception;

class backend_controller extends controller
{
	/**
	 * init
	 * @throws \Exception
	 */
	public function init(): void
	{
		parent::init();

		//user_auth component
		\phpvod::$app->register('user_auth', user_auth::class);
		/* @var admin $admin_model */
		$admin_model = model(admin::class);
		\phpvod::$app->user_auth->init($admin_model);

		//check route
		$this->check_route();
	}

	/**
	 * 获取bootstrap_table查询参数
	 * @return array|false 成功返回查询参数数组(包含search/order_by/offset_limit元素)，失败返回false
	 * @throws \Exception
	 */
	protected function get_bootstrap_table_query_params(): false|array
	{
		if ($this->validation->load_config(bootstrap_table::class)->with_get()->run()) {
			$data = $this->request->get();
			//search
			if(!isset($data['search']) || trim($data['search']) == '') {
				$data['search'] = null;
			}
			//order_by
			$data['order_by'] = isset($data['sort']) ? $data['sort'] . ' ' . $data['order'] : null;
			$data['offset_limit'] = isset($data['offset']) && isset($data['limit']) ? $data['offset'] . ',' . $data['limit'] : null;
			return $data;
		}
		return false;
	}

	/**
	 * 后台跳转
	 * @param string $uri
	 * @param bool $with_input
	 * @param $message
	 * @return void
	 * @throws \Exception
	 */
	protected function backend_redirect(string $uri, bool $with_input = false, $message = null): void
	{
		parent::redirect(admin_uri($uri), $with_input, $message);
	}

	/**
	 * 模块后台跳转
	 * @param string $uri
	 * @param bool $with_input
	 * @param $message
	 * @return void
	 * @throws \Exception
	 */
	protected function module_redirect(string $uri, bool $with_input = false, $message = null): void
	{
		parent::redirect(admin_module_uri($uri), $with_input, $message);
	}

	/**
	 * 载入视图
	 * @param string $view
	 * @param array $data
	 * @return string
	 * @throws \Exception
	 */
	protected function view(string $view, array $data = []): string
	{
		return parent::view('admin/' . $view, $data);
	}

	/**
	 * 更新后台菜单
	 * @return void
	 * @throws \Exception
	 */
	protected function update_backend_menu(): void
	{
		session()->set_flash_data('update_backend_menu', true);
	}

	/**
	 * 更改状态
	 * @param callable $callback 更新数据库的回调函数
	 * @param int $id 回调函数参数
	 * @param callable|null $success_callback $callback执行成功后的回调，一般用于更新缓存
	 * @return void
	 * @throws \Exception
	 */
	protected function change_status_by_id(callable $callback, int $id, callable $success_callback = null): void
	{
		if ($this->validation->check($id, 'id', validator::class)) {
			$status = call_user_func($callback, $id);
			if ($status === false) {
				$this->response->set_json(['id' => 0, 'msg' => lang('app.operation_failed')]);
			} else {
				$this->response->set_json(['id' => 1, 'msg' => lang('app.operation_succeeded'), 'data' => (int)$status]);
				!is_null($success_callback) && call_user_func($success_callback);
			}
		} else {
			$this->response->set_json(['id' => 0, 'msg' => lang('app.invalid_id')]);
		}
	}

	/**
	 * 检测路由
	 * @return void
	 * @throws \Exception
	 */
	protected function check_route(): void
	{
		$module_namespace = \phpvod::$app->module->get_current_module('namespace');
		$controller_name = basename(str_replace('\\', '/', \phpvod::$app->router->get_controller()));
		$method_name = \phpvod::$app->router->get_method();
		//route
		$route = (!is_null($module_namespace) ? $module_namespace . '$' : '') . $controller_name . '::' . $method_name;

		$backend_identity = \phpvod::$app->user_auth->get_backend_identity();
		if (is_null($backend_identity)) {
			//guest
			$guest_allowed_routes = $this->get_guest_allowed_routes();
			if (!in_array($route, $guest_allowed_routes)) {
				$this->backend_redirect('login');
			}
		} else {
			//check permission
			$ignore_permission_routes = $this->get_ignore_permission_routes();
			if (!in_array($route, $ignore_permission_routes)) {
				$request_method = $this->request->get_method(true);
				$route = $route . '/' . $request_method;
				if (!\phpvod::$app->user_auth->check_backend_permission($route)) {
					if ($this->request->is_ajax()) {
						$this->response->set_json(['id' => 0, 'msg' => lang('admin.access_denied')]);
						throw controller_exception::for_interrupt();
					} else {
						$http_referer = previous_url();
						$this->redirect($http_referer, false, lang('admin.access_denied'));
					}
				}
			}
		}
	}

	/**
	 * 获取允许游客访问的路由
	 * @return array
	 */
	private function get_guest_allowed_routes(): array
	{
		return ['home::login', 'home::logout'];
	}

	/**
	 * 获取不需要检测权限的路由
	 * @return array
	 */
	private function get_ignore_permission_routes(): array
	{
		return ['home::login', 'home::logout', 'home::index', 'home::get_backend_menu', 'main::index', 'main::get_notice', 'update::check_version'];
	}
}