<?php

namespace phpvod\component\user_auth;

use phpvod\library\config\config;
use phpvod\model\admin;
use phpvod\model\member;
use venus\base\component;

/**
 * 会员身份认证组件
 */
class user_auth extends component
{
	/**
	 * @var array|null 后台会员信息
	 */
	protected ?array $backend_identity = null;

	/**
	 * @var array|null 前台会员信息
	 */
	protected ?array $frontend_identity = null;

	/**
	 * @var array|null 前台会员权限
	 */
	protected ?array $frontend_permission = null;

	/**
	 * @var admin|member 会员模型
	 */
	protected member|admin $user_model;

	//EVENTS
	public const EVENT_FRONTEND_LOGIN = 'event_frontend_login';
	public const EVENT_FRONTEND_LOGOUT = 'event_frontend_logout';
	public const EVENT_MEMBER_GROUP_EXPIRY = 'event_member_group_expiry';
	public const EVENT_BACKEND_LOGIN = 'event_backend_login';
	public const EVENT_BACKEND_LOGOUT = 'event_backend_logout';

	/**
	 * 组件初始化
	 * @param admin|member $user_model
	 * @throws \Exception
	 */
	public function init(admin|member $user_model): void
	{
		$this->user_model = $user_model;

		if ($this->user_model instanceof admin) {
			$id = session()->get('backend_identity_id');
			if (!is_null($id)) {
				$this->backend_identity = $this->user_model->get_identity_by_id($id);
			}
		} elseif ($this->user_model instanceof member) {
			$id = session()->get('frontend_identity_id');
			if (!is_null($id)) {
				$this->frontend_identity = $this->user_model->get_identity_by_id($id);
				$this->check_member_group_expiry();
			}
		}
	}

	/**
	 * 是否已经登录后台
	 * @return bool
	 */
	public function backend_is_logged_in(): bool
	{
		return is_array($this->backend_identity);
	}

	/**
	 * 是否已经登录前台
	 * @return bool
	 */
	public function frontend_is_logged_in(): bool
	{
		return is_array($this->frontend_identity);
	}

	/**
	 * 获取后台登录用户数据
	 * @param string|null $field
	 * @return mixed
	 */
	public function get_backend_identity(?string $field = null): mixed
	{
		if (is_null($field))
			return $this->backend_identity;

		return is_array($this->backend_identity) && array_key_exists($field, $this->backend_identity) ? $this->backend_identity[$field] : null;
	}

	/**
	 * 获取前台登录用户数据
	 * @param string|null $field
	 * @return mixed
	 */
	public function get_frontend_identity(?string $field = null): mixed
	{
		if (is_null($field))
			return $this->frontend_identity;

		return is_array($this->frontend_identity) && array_key_exists($field, $this->frontend_identity) ? $this->frontend_identity[$field] : null;
	}

	/**
	 * 检测前台用户会员组是否过期
	 * @return void
	 * @throws \Exception
	 */
	private function check_member_group_expiry(): void
	{
		if (!$this->frontend_is_logged_in())
			return;

		if (is_null($this->frontend_identity['group_expiry']))
			return;

		if (!($this->user_model instanceof member)) {
			throw new \Exception('Invalid model type');
		}

		if (TIMESTAMP > $this->frontend_identity['group_expiry']) {
			$identity = $this->frontend_identity;

			//update database
			$this->user_model->change_group_id($this->frontend_identity['id']);
			//update frontend_identity
			$this->frontend_identity['group_id'] = $this->frontend_identity['change_to'];
			$this->frontend_identity['group_expiry'] = null;
			$group = config::get_instance()->get('member_group', $this->frontend_identity['group_id']);
			$this->frontend_identity['group_name'] = $group['name'] ?? null;

			//event
			\phpvod::$app->event->trigger(self::EVENT_MEMBER_GROUP_EXPIRY, $identity);
		}
	}

	/**
	 * 检测后台用户是否具备访问当前路由地址的权限
	 * @param string $route 路由地址
	 * @return bool
	 */
	public function check_backend_permission(string $route): bool
	{
		if (is_array($this->backend_identity) && array_key_exists('permissions', $this->backend_identity)) {
			$permissions = $this->backend_identity['permissions'];

			if ($permissions == '-1') {
				return true;
			}

			$data = explode('/', $route);
			helper('string');
			return in_string($route, $permissions) || in_string($data[0], $permissions);
		}
		return false;
	}

	/**
	 * 获取前台会员权限
	 * @param string $name 权限名称
	 * @return mixed 返回权限值，权限不存在时返回null
	 */
	public function get_frontend_permission(string $name): mixed
	{
		if (is_null($this->frontend_permission)) {
			$member_group = config::get_instance()->get('member_group');

			//member_group_permissions
			$group_id = (int)is_array($this->frontend_identity) ? $this->frontend_identity['group_id'] : 1; // 1=guest
			$member_group_permissions = $member_group[$group_id]['permissions'];

			//system & module permission file
			$list = [CONFIG_PATH . 'permission/member_group.php'];
			$modules = \phpvod::$app->module->get_modules();
			foreach ($modules as $module) {
				$list[] = MODULE_PATH . $module['directory'] . '/config/permission/member_group.php';
			}

			//default_permissions
			$default_permissions = [];
			foreach ($list as $file) {
				if (!is_file($file)) continue;
				$config = include $file;
				if (is_array($config)) {
					$default_permissions = array_merge($default_permissions, array_map(function (array $row) {
						return $row['default'];
					}, $config));
				}
			}

			$this->frontend_permission = array_merge($default_permissions, $member_group_permissions);
		}

		return array_key_exists($name, $this->frontend_permission) ? $this->frontend_permission[$name] : null;
	}

	/**
	 * 登录
	 * @param array $auth_field 认证字段，键为字段名，值为字段值
	 * @return bool 成功返回true，失败返回false
	 * @throws \Exception
	 */
	public function login(array $auth_field): bool
	{
		$identity = $this->user_model->auth($auth_field);
		if (is_array($identity)) {
			if ($this->user_model instanceof admin) { //backend
				session()->set('backend_identity_id', $identity['id']);
				$this->backend_identity = $identity;

				//update login_at & login_ip
				$this->user_model->update([
					'login_at' => TIMESTAMP,
					'login_ip' => \phpvod::$app->request->get_ip()
				], $identity['id']);

				//event
				\phpvod::$app->event->trigger(self::EVENT_BACKEND_LOGIN, $identity);
				return true;
			} elseif ($this->user_model instanceof member) { //frontend
				session()->set('frontend_identity_id', $identity['id']);
				$this->frontend_identity = $identity;

				//update login_at & login_ip
				$this->user_model->update([
					'login_at' => TIMESTAMP,
					'login_ip' => \phpvod::$app->request->get_ip()
				], $identity['id']);

				//event
				\phpvod::$app->event->trigger(self::EVENT_FRONTEND_LOGIN, $identity);
				return true;
			}
		}
		return false;
	}

	/**
	 * 登出
	 * @throws \Exception
	 */
	public function logout(): void
	{
		if ($this->user_model instanceof admin) {
			$backend_identity = $this->backend_identity;
			session()->remove('backend_identity_id');
			$this->backend_identity = null;
			//event
			\phpvod::$app->event->trigger(self::EVENT_BACKEND_LOGOUT, $backend_identity);
		} elseif ($this->user_model instanceof member) {
			$frontend_identity = $this->frontend_identity;
			session()->remove('frontend_identity_id');
			$this->frontend_identity = null;
			//event
			\phpvod::$app->event->trigger(self::EVENT_FRONTEND_LOGOUT, $frontend_identity);
		}
	}

}
