<?php

namespace config;

use venus\application\exception\controller_exception;
use venus\exception\handler;
use venus\http\status;

class exception extends handler
{
	/**
	 * 将异常类映射至方法，针对不同的异常定义不同的处理方法
	 * 定义格式：[exceptionClassName => methodName]
	 * @var array $exception_map
	 */
	public array $exception_map = [
		\PDOException::class => 'handle_pdo_exception',
	];

	/**
	 * handle pdo exception
	 * @param \PDOException $exception ;
	 */
	public function handle_pdo_exception(\PDOException $exception): void
	{
		parent::exception_handler($exception);

		//write log
		$log_file = WRITABLE_PATH . 'log/database_exception.log';
		$this->write_exception_log($exception, $log_file);
	}

	/**
	 * 默认异常处理器
	 * @param \Throwable $exception
	 */
	public function exception_handler(\Throwable $exception): void
	{
		parent::exception_handler($exception);

		//生产模式下的 controller_exception::for_not_exists 异常不写入日志
		if (ENV == 'prod'
			&& $exception instanceof controller_exception
			&& $exception->getCode() == controller_exception::NOT_EXISTS_CODE
		) {
			\venus::$app->response->set_status_code(status::HTTP_NOT_FOUND)->send();
			return;
		}

		//write log
		$log_file = WRITABLE_PATH . 'log/exception.log';
		$this->write_exception_log($exception, $log_file);
	}

	/**
	 * 错误处理
	 * @param int $err_no
	 * @param string $err_str
	 * @param string $err_file
	 * @param int $err_line
	 * @return bool
	 */
	public function error_handler(int $err_no, string $err_str, string $err_file, int $err_line): bool
	{
		//write log
		$log_file = WRITABLE_PATH . 'log/error.log';
		$log_message = sprintf("Time: %s, IP: %s\r\nUri: %s, Method: %s\r\nError: %s (%s)\r\nFile: %s (%s)\r\n\r\n",
			date('Y-m-d H:i:s'), $_SERVER['REMOTE_ADDR'],
			$_SERVER['REQUEST_URI'], $_SERVER['REQUEST_METHOD'],
			$err_str, $err_no,
			$err_file, $err_line
		);
		error_log($log_message, 3, $log_file);
		return false;
	}

	/**
	 * 写入异常日志
	 * @param \Exception $exception
	 * @param string $log_file
	 * @return void
	 */
	private function write_exception_log(\Throwable $exception, string $log_file): void
	{
		$log_message = sprintf("Time: %s, IP: %s\r\nUri: %s, Method: %s\r\nException: %s\r\nMessage: %s, Code: %s\r\nFile: %s, Line: %s\r\nTrace:\r\n%s\r\n\r\n",
			date('Y-m-d H:i:s'), $_SERVER['REMOTE_ADDR'],
			$_SERVER['REQUEST_URI'], $_SERVER['REQUEST_METHOD'],
			get_class($exception),
			$this->get_message($exception), $exception->getCode(),
			$exception->getFile(), $exception->getLine(),
			$exception->getTraceAsString()
		);
		error_log($log_message, 3, $log_file);
	}
}