<?php

namespace phpvod\controller\api;

use config\validation\home\buy_member_group;
use config\validation\home\change_password;
use config\validation\home\login;
use config\validation\home\register;
use config\validation\home\reset_password;
use config\validation\home\send_email_code;
use phpvod\base\frontend_controller;
use phpvod\library\config\config;
use phpvod\library\verify_manager;
use phpvod\model\transaction_history;
use phpvod\service\transaction;
use phpvod\widget\toastr;

class member extends frontend_controller
{
	private \phpvod\model\member $model;
	public const EVENT_MEMBER_REGISTER = 'event_member_register';
	public const ACTION_REGISTER = 'register';
	public const ACTION_RESET_PASSWORD = 'reset_password';

	public function init(): void
	{
		parent::init();
		/** @noinspection PhpFieldAssignmentTypeMismatchInspection */
		$this->model = model(\phpvod\model\member::class);
	}

	/**
	 * 会员登录(AJAX)
	 * @return void
	 * @throws \Exception
	 */
	public function login(): void
	{
		if (\phpvod::$app->user_auth->frontend_is_logged_in()) {
			$this->response->set_json(['id' => -1, 'msg' => lang('home.user_has_logged')]);
			return;
		}

		if ($this->validation->load_config(login::class)->with_post()->run()) {
			$login_result = \phpvod::$app->user_auth->login([
				'username' => $this->request->post('username'),
				'password' => $this->request->post('password'),
				'status' => 1
			]);
			if ($login_result) {
				$this->set_message(lang('home.login_success'), toastr::TYPE_SUCCESS);
				$this->response->set_json(['id' => 1, 'msg' => lang('home.login_success')]);
			} else {
				$this->response->set_json(['id' => 0, 'msg' => lang('app.invalid_username_or_password')]);
			}
		} else {
			$errors = $this->validation->get_errors();
			$this->response->set_json(['id' => 0, 'msg' => array_shift($errors)]);
		}
	}

	/**
	 * 会员登出(AJAX)
	 * @return void
	 * @throws \Exception
	 */
	public function logout(): void
	{
		\phpvod::$app->user_auth->logout();
		$this->set_message(lang('home.logout_success'), toastr::TYPE_SUCCESS);
		$this->response->set_json(['id' => 1, 'msg' => lang('app.operation_succeeded')]);
	}

	/**
	 * 会员注册(AJAX)
	 * @return void
	 * @throws \Exception
	 */
	public function register(): void
	{
		if (\phpvod::$app->user_auth->frontend_is_logged_in()) {
			$this->response->set_json(['id' => -1, 'msg' => lang('home.user_has_logged')]);
			return;
		}

		$register_config = config::get_instance()->get('config_register');
		if ($register_config['allow_register'] != '1') {
			$this->response->set_json(['id' => 0, 'msg' => lang('home.register_disable')]);
			return;
		}

		if ($this->validation->load_config(register::class)->with_post()->run()) {
			$member = $this->request->post('member');
			$member['group_id'] = $register_config['register_check'] == '1' ? 2 : 3; //2待审核会员，3普通会员
			$member['password_hash'] = password_hash($member['password'], PASSWORD_BCRYPT);
			$member['status'] = 1; //1启用，0停用
			$member['created_at'] = TIMESTAMP;
			$this->model->save($member);
			$id = $this->model->insert_id();
			//delete verify code cache
			verify_manager::get_instance(\phpvod::$app->cache->get_handler())
				->remove_code(self::ACTION_REGISTER, $member['email']);
			//event
			event()->trigger(self::EVENT_MEMBER_REGISTER, (int)$id);
			$this->response->set_json(['id' => 1, 'msg' => lang('home.register_success')]);
		} else {
			$errors = $this->validation->get_errors();
			$this->response->set_json(['id' => 0, 'msg' => array_shift($errors)]);
		}
	}

	/**
	 * 修改密码(AJAX)
	 * @return void
	 * @throws \Exception
	 */
	public function change_password(): void
	{
		$identity = \phpvod::$app->user_auth->get_frontend_identity();

		if (!is_array($identity)) {
			$this->response->set_json(['id' => 0, 'msg' => lang('app.please_login_first')]);
			return;
		}
		if ($this->validation->load_config(change_password::class)->with_post()->run()) {
			$original_password = $this->request->post('original_password');
			$new_password = $this->request->post('new_password');
			if (password_verify($original_password, $identity['password_hash'])) {
				$this->model->update(['password_hash' => password_hash($new_password, PASSWORD_BCRYPT)], $identity['id']);
				$this->response->set_json(['id' => 1, 'msg' => lang('home.password_change_success')]);
			} else {
				$this->response->set_json(['id' => 0, 'msg' => lang('home.original_password_error')]);
			}
		} else {
			$errors = $this->validation->get_errors();
			$this->response->set_json(['id' => 0, 'msg' => array_shift($errors)]);
		}
	}

	/**
	 * 找回密码(AJAX)
	 * @return void
	 * @throws \Exception
	 */
	public function reset_password(): void
	{
		$action = $this->request->post('action');
		if (!in_array($action, ['profile', 'auth', 'reset'])) {
			$this->response->set_json(['id' => 0, 'msg' => lang('app.invalid_params')]);
			return;
		}

		if ($action === 'profile') { //STEP1: 获取会员资料（id/username/email/phone）
			if ($this->validation->load_config(reset_password::class, 'profile')->with_post()->run()) {
				$account = $this->request->post('account');
				$profile = $this->model->select('id, username, email, phone')->where(['username' => $account])->or_where(['email' => $account])->or_where(['phone' => $account])->query()->fetch_array();
				if (is_array($profile)) {
					//session
					$_SESSION['reset_password'] = [
						'profile' => $profile,
						'auth' => false
					];;
					helper('string');
					!is_null($profile['email']) && $profile['email'] = mark_string($profile['email'], 2, 0, 1);
					!is_null($profile['phone']) && $profile['phone'] = mark_string($profile['phone'], 3, 4);
					$this->response->set_json(['id' => 1, 'data' => $profile]);
				} else {
					$this->response->set_json(['id' => 0, 'msg' => lang('home.account_not_exists')]);
				}
			}
		} elseif ($action === 'auth') { //STEP2: 验证身份
			$email_code = $this->request->post('email_code');
			$email = $_SESSION['reset_password']['profile']['email'] ?? null;
			if ($this->validation->load_config(reset_password::class, 'auth')->set_data(['email_code' => $email_code, 'email' => $email])->run()) {
				$_SESSION['reset_password']['auth'] = true;
				//delete verify code cache
				verify_manager::get_instance(\phpvod::$app->cache->get_handler())
					->remove_code(self::ACTION_RESET_PASSWORD, $email);
				$this->response->set_json(['id' => 1]);
			}
		} elseif ($action === 'reset') { //STEP3: 重置密码
			if ($this->validation->load_config(reset_password::class, 'reset')->with_post()->run()) {
				if ($_SESSION['reset_password']['auth'] !== true) {
					$this->response->set_json(['id' => 0, 'msg' => lang('app.permission_denied')]);
					return;
				}
				$password_hash = password_hash($this->request->post('new_password'), PASSWORD_BCRYPT);
				$this->model->update(['password_hash' => $password_hash], $_SESSION['reset_password']['profile']['id']);
				//unset session
				unset($_SESSION['reset_password']);
				$this->response->set_json(['id' => 1, 'msg' => lang('app.operation_succeeded')]);
			}
		}

		if ($this->validation->has_errors()) {
			$errors = $this->validation->get_errors();
			$this->response->set_json(['id' => 0, 'msg' => array_shift($errors)]);
		}
	}

	/**
	 * 发送邮件验证码(AJAX)
	 * @param string $action 验证码所应用的操作(register/reset_password)
	 * @return void
	 * @throws \Exception
	 */
	public function send_email_code(string $action): void
	{
		if (!in_array($action, [self::ACTION_REGISTER, self::ACTION_RESET_PASSWORD])) {
			$this->response->set_json(['id' => 0, 'msg' => lang('app.invalid_params')]);
			return;
		}

		$email = match ($action) {
			'register' => $this->request->post('email'),
			'reset_password' => $_SESSION['reset_password']['profile']['email'] ?? null
		};

		if ($this->validation->load_config(send_email_code::class, $action)->set_data(['email' => $email])->run()) {

			$verify_manager = verify_manager::get_instance(\phpvod::$app->cache->get_handler());

			try {
				$sent = $verify_manager->send_email_code($action, $email);
				if ($sent) {
					$this->response->set_json(['id' => 1, 'msg' => lang('app.mail_sent_successfully')]);
				} else {
					$this->response->set_json(['id' => 0, 'msg' => lang('app.mail_sent_failed')]);
				}
			} catch (\Exception $exception) {
				$this->response->set_json(['id' => 0, 'msg' => $exception->getMessage()]);
			}

		} else {
			$errors = $this->validation->get_errors();
			$this->response->set_json(['id' => 0, 'msg' => array_shift($errors)]);
		}
	}

	/**
	 * 购买会员组(AJAX)
	 * @return void
	 * @throws \Exception
	 */
	public function buy_member_group(): void
	{
		if ($this->validation->load_config(buy_member_group::class)->with_post()->run()) {
			$id = $this->request->post('id');
			$index = $this->request->post('index');
			/* @var transaction $transaction_service */
			$transaction_service = service(transaction::class);
			$this->response->set_json($transaction_service->buy_member_group($id, $index));
		} else {
			$errors = $this->validation->get_errors();
			$this->response->set_json(['id' => 0, 'msg' => array_shift($errors)]);
		}
	}

	/**
	 * 我的账单(AJAX)
	 * @return void
	 * @throws \Exception
	 */
	public function my_transaction_history(): void
	{
		$identity = \phpvod::$app->user_auth->get_frontend_identity();

		if (!is_array($identity)) {
			$this->response->set_json(['id' => 0, 'msg' => lang('app.please_login_first')]);
			return;
		}

		\phpvod::$app->language->load_file('transaction_action', \venus\language\language::ADDITIONAL_ALL_MODULE_LANGUAGE);

		$page = $this->request->get('page');
		$page = is_numeric($page) && intval($page) >= 1 ? intval($page) : 1;

		$limit = $this->request->get('limit');
		$limit = is_numeric($limit) && intval($limit) >= 1 ? intval($limit) : 10;

		/* @var transaction_history $transaction_history_model */
		$transaction_history_model = model(transaction_history::class);
		$total = (int)$transaction_history_model->select('count(id)')->where(['member_id' => $identity['id']])->query()->fetch_column();
		$max_page = $total > 0 ? ceil($total / $limit) : 1;
		$page > $max_page && $page = $max_page;
		$offset = ($page - 1) * $limit;
		$data = $transaction_history_model->select('id,type,action,related,amount,created_at')->where(['member_id' => $identity['id']])->order_by('id DESC')->limit($offset . ',' . $limit)->query()->fetch_all_by_callback(function (array $row) {
			$row['action'] = lang('transaction_action.' . $row['action']);
			$row['amount'] = number_format(floatval($row['amount']) / 100, 2, '.', '');
			$row['created_at'] = timetostr($row['created_at']);
			return $row;
		});

		$this->response->set_json(['id' => 1, 'data' => [
			'total' => $total,
			'limit' => $limit,
			'page' => $page,
			'data' => $data,
		]]);
	}
}