<?php

namespace phpvod\controller\install;

use phpvod\library\config\config;
use phpvod\model\admin;
use phpvod\model\module;
use venus\application\exception\controller_exception;

class home extends \phpvod\base\controller
{
	protected array $helper = ['file_system'];
	public const INSTALL_LOCK = WRITABLE_PATH . 'install.lock';
	private string $module_directory = APP_PATH . 'view/install/module';

	/**
	 * init
	 * @return void
	 * @throws \Exception | controller_exception
	 */
	public function init(): void
	{
		parent::init();

		if(!function_exists('exec') || !function_exists('symlink')) {
			$this->response->set_content_type('text/plain')->set_body('exec or symlink function does not exist.');
			throw controller_exception::for_interrupt();
		}

		if (is_file(self::INSTALL_LOCK)) {
			echo $this->view('install/home/already_installed');
			throw controller_exception::for_interrupt();
		}
	}

	/**
	 * index 显示软件使用协议
	 * @return string
	 * @throws \Exception
	 */
	public function index(): string
	{
		$licence = file_get_contents(ROOT_PATH . 'licence.txt');
		$licence = str_replace('  ', '&nbsp; ', nl2br($licence));
		return $this->view('install/home/index', ['licence' => $licence]);
	}

	/**
	 * 环境检测
	 * @return string
	 * @throws \Exception
	 */
	public function check_env(): string
	{
		return $this->view('install/home/check_env');
	}

	/**
	 * 写权限检测
	 * @return string
	 * @throws \Exception
	 */
	public function check_writable(): string
	{
		return $this->view('install/home/check_writable');
	}

	/**
	 * 数据库配置信息
	 * @return string
	 * @throws \Exception
	 */
	public function configure_database(): string
	{
		if ($this->request->is_post()) {
			$config = [
				'dsn' => "mysql:host={$_POST['host']};port={$_POST['port']};dbname={$_POST['dbname']};charset=utf8",
				'username' => $_POST['username'],
				'password' => $_POST['password'],
				'charset' => 'utf8',
				'prefix' => $_POST['prefix'],
				'persistent' => false,
			];
			write_file(WRITABLE_PATH . 'config/database.php', "<?php\nreturn " . var_export($config, true) . ";");
			try {
				$db = \phpvod::$app->database;
			} catch (\PDOException $pdo_exception) {
				$message = $pdo_exception->getCode() == 1049 ? "未知数据库`{$_POST['dbname']}`" : '数据库连接失败';
				$this->redirect('/install/configure-database', true, '[' . $pdo_exception->getCode() . '] ' . $message);
			}
			$this->redirect('/install/install');
		}
		return $this->view('install/home/configure_database');
	}

	/**
	 * 安装
	 * @throws \Exception
	 */
	public function install(): void
	{
		if ($this->request->is_post()) {
			$sql_file = APP_PATH . 'view/install/mysql/phpvod.sql';

			$db = \phpvod::$app->database;
			try {
				$db->import_file($sql_file);
			}catch (\PDOException $pdo_exception) {
				$this->response->set_json(['id' => 0, 'msg' => '安装失败: ' . $pdo_exception->getMessage()]);
				return;
			}
			//生成缓存
			$config = config::get_instance();
			$config->update('config_site');
			$config->update('config_register');
			$config->update('config_template');
			$config->update('config_cache');
			$config->update('config_background');
			$config->update('config_mail');
			$config->update('admin_role');
			$config->update('language');
			$config->update('member_group');
			$this->response->set_json(['id' => 1, 'msg' => '安装成功']);
		} else {
			$config = include WRITABLE_PATH . 'config/database.php';
			$already_installed = \phpvod::$app->database->query("SHOW TABLES LIKE '" . $config['prefix'] . "admin'")->fetch_column() !== false;
			echo $this->view('/install/home/install', ['already_installed' => $already_installed]);
		}
	}

	public function set_admin(): string
	{
		if ($this->request->is_post()) {
			/* @var admin $admin_model */
			$admin_model = model(admin::class);
			$affected_rows = $admin_model->insert([
				'role_id' => 1,
				'username' => $_POST['username'],
				'password_hash' => password_hash($_POST['password'], PASSWORD_BCRYPT),
				'email' => $_POST['email'],
				'status' => 1,
				'created_at' => TIMESTAMP
			]);
			if ($affected_rows > 0) {
				$this->redirect('/install/install-module');
			} else {
				$this->redirect('/install/set-admin', true, '管理员账号添加失败');
			}
		}
		return $this->view('/install/home/set_admin');
	}

	public function install_module(): string
	{
		if ($this->request->is_post()) {
			$module = $_POST['module'] ?? [];
			/* @var module $module_model */
			$module_model = model(module::class);
			foreach ($module as $file) {
				$zip_file = $this->module_directory . '/' . $file;
				if (!is_file($zip_file)) {
					continue;
				}
				$module_model->install($zip_file);
				config::get_instance()->update('module'); //更新模块缓存
			}
			$this->redirect('/install/complete');
		}

		$module_list = [];
		$handle = opendir($this->module_directory);
		if ($handle !== false) {
			while (($file = readdir($handle)) !== false) {
				if ($file === '.' || $file === '..' || !str_ends_with(strtolower($file), '.zip')) {
					continue;
				}
				$config = get_zip_config($this->module_directory . '/' . $file);
				if (!is_null($config)) {
					$module_list[$file] = $config['name'];
				}
			}
			closedir($handle);
		}

		if (empty($module_list)) {
			$this->redirect('/install/complete');
		}

		return $this->view('/install/home/install_module', ['module_list' => $module_list]);
	}

	/**
	 * 安装完成
	 * @return string
	 * @throws \Exception
	 */
	public function complete(): string
	{
		//映射目录
		symlink(WRITABLE_PATH . 'upload', PUBLIC_PATH . 'upload');
		write_file(self::INSTALL_LOCK, timetostr());
		return $this->view('/install/home/complete');
	}
}