<?php

namespace phpvod\library\config;

abstract class abstract_config
{
	/**
	 * @var array $data
	 * <p>格式：['file' => ['key' => 'value', ...], ...]</p>
	 */
	protected array $data = [];

	/**
	 * 根据缓存文件名映射缓存生成方法
	 * @var array $method_map
	 * <p>格式：['file' => ['缓存生成方法', '参数1', '参数2', ... ], ... ]
	 */
	protected array $method_map = [];

	protected string $config_path = WRITABLE_PATH . 'cache/config';

	/**
	 * 获取配置数据
	 * @param string $file 配置文件名，不含扩展名
	 * @param string|null $key 配置项名称
	 * @param boolean $get_data_from_database 是否从数据库中获取配置
	 * @return mixed 成功时返回配置数据，失败返回null
	 */
	public function get(string $file, ?string $key = null, bool $get_data_from_database = true): mixed
	{
		if (empty($file)) {
			return null;
		}

		if (!isset($this->data[$file])) {
			$config_file = $this->config_path . DIRECTORY_SEPARATOR . $file . '.php';
			if (!is_file($config_file) && $get_data_from_database) {
				//查询数据库并生成缓存
				$this->create_cache($file);
			}
			if (is_file($config_file)) {
				$this->data[$file] = include $config_file;
			}
		}

		if (is_null($key)) {
			return $this->data[$file] ?? null;
		} else {
			return $this->data[$file][$key] ?? null;
		}
	}

	/**
	 * 删除配置文件
	 * @param string $file 配置文件名
	 */
	public function delete(string $file): void
	{
		if (empty($file)) {
			return;
		}

		if (isset($this->data[$file])) {
			unset($this->data[$file]);
		}
		$config_file = $this->get_file_path($file);
		if (is_file($config_file)) {
			@unlink($config_file);
		}
	}

	/**
	 * 更新配置文件
	 * @param string $file 配置文件名
	 */
	public function update(string $file): void
	{
		$this->delete($file);
		$this->create_cache($file);
		$config_file = $this->get_file_path($file);
		if (is_file($config_file)) {
			$this->data[$file] = include $config_file;
		}
	}

	/**
	 * 生成配置文件
	 * @param string $file 配置文件名
	 */
	protected function create_cache(string $file): void
	{
		helper('file_system');
		if (isset($this->method_map[$file])) {

			$data = $this->method_map[$file];
			$method_name = array_shift($data);

			if (is_callable([$this, $method_name])) {
				call_user_func_array([$this, $method_name], $data);
			}
		}
	}

	/**
	 * 获取文件路径
	 * @param string $filename 文件名，不含扩展名
	 * @return string
	 */
	protected function get_file_path(string $filename): string
	{
		return $this->config_path . DIRECTORY_SEPARATOR . $filename . '.php';
	}
}
