<?php

namespace phpvod\library;

use PHPMailer\PHPMailer\PHPMailer;
use PHPMailer\PHPMailer\SMTP;
use PHPMailer\PHPMailer\Exception;
use phpvod\library\config\config;

class mail extends PHPMailer
{
	public const EVENT_MAIL_BEFORE_SEND = 'event_mail_before_send';
	public const EVENT_MAIL_SENT_SUCCESSFULLY = 'event_mail_sent_successfully';
	public const EVENT_MAIL_SENT_FAILED = 'event_mail_sent_failed';

	private string $template_path = CONFIG_PATH . 'mail' . DIRECTORY_SEPARATOR . 'template';

	/**
	 * __construct
	 * @throws Exception
	 */
	public function __construct(?bool $exceptions = null)
	{
		parent::__construct($exceptions);

		//charset
		$this->CharSet = self::CHARSET_UTF8;

		//config
		$config_mail = config::get_instance()->get('config_mail');
		switch ($config_mail['mail_protocol']) {
			case 'mail':
				$this->isMail();
				break;
			case 'sendmail':
				$this->isSendmail();
				!empty($config_mail['mail_sendmail_path']) && $this->Sendmail = $config_mail['mail_sendmail_path'];
				break;
			case 'qmail':
				$this->isQmail();
				!empty($config_mail['mail_qmail_path']) && $this->Sendmail = $config_mail['mail_qmail_path'];
				break;
			case 'smtp':
				$this->isSMTP();
				$this->SMTPDebug = SMTP::DEBUG_OFF;
				$this->Host = $config_mail['mail_smtp_host'];
				$this->Port = $config_mail['mail_smtp_port'];
				$this->SMTPAuth = true;
				$this->SMTPSecure = $config_mail['mail_smtp_secure'];
				$this->Username = $config_mail['mail_smtp_user'];
				$this->Password = $config_mail['mail_smtp_pass'];
				$this->setFrom($config_mail['mail_smtp_user'], $config_mail['mail_smtp_name']);
				break;
		}

		//language
		$langcode = 'en';
		foreach (['{lang}_{tag}', '{lang}'] as $format) {
			$lang = \phpvod::$app->language->get_format_language($format);
			if (is_null($lang)) continue;
			$file = ROOT_PATH . 'vendor' . DIRECTORY_SEPARATOR . 'phpmailer' . DIRECTORY_SEPARATOR . 'phpmailer' .
				DIRECTORY_SEPARATOR . 'language' . DIRECTORY_SEPARATOR . 'phpmailer.lang-' . $lang . '.php';
			if (is_file($file)) {
				$langcode = $lang;
				break;
			}
		}
		$this->setLanguage($langcode);
	}

	/**
	 * set_template_path
	 * @param string $template_path
	 * @return mail
	 */
	public function set_template_path(string $template_path): mail
	{
		$this->template_path = $template_path;
		return $this;
	}

	/**
	 * 设置邮件标题
	 * @param string $subject
	 * @param array $vars
	 * @return $this
	 */
	public function set_subject(string $subject, array $vars = []): mail
	{
		!empty($vars) && $subject = strtr($subject, $vars);
		$this->Subject = $subject;
		return $this;
	}

	/**
	 * 设置邮件内容模板
	 * @param string $file 模板文件名
	 * @param array $vars 替换变量
	 * <p>格式: ['{username}' => $username]</p>
	 * @return $this
	 * @throws Exception
	 */
	public function set_body_from_template(string $file, array $vars = []): mail
	{
		$file = $this->template_path . DIRECTORY_SEPARATOR . $file;
		if (is_file($file)) {
			$body = file_get_contents($file);
			if ($body !== false) {
				$this->set_body($body, $vars);
			}
		}
		return $this;
	}

	/**
	 * 从HTML字符串创建消息体。
	 * 自动内联图像并通过转换HTML创建纯文本版本，并对Body和AltBody赋值。
	 * @param string $body html消息体
	 * @param array $vars 替换变量
	 * <p>格式: ['{username}' => $username]</p>
	 * @return $this
	 * @throws Exception
	 */
	public function set_body(string $body, array $vars = []): mail
	{
		!empty($vars) && $body = strtr($body, $vars);
		$this->msgHTML($body, $this->template_path);
		return $this;
	}

	/**
	 * 添加收件人地址
	 * @param string|array $address
	 * <p>格式一：xxx@xxx.xxx</p>
	 * <p>格式二：[xxx@xxx.xxx, xxx@xxx.xxx, ...]</p>
	 * <p>格式三：[[xxx@xxx.xxx, xxx], [xxx@xxx.xxx, xxx], ...]
	 * @return $this
	 * @throws Exception
	 */
	public function add_address($address): mail
	{
		if (is_string($address)) {
			$address = [$address];
		}

		foreach ($address as $value) {
			if (is_string($value)) {
				$this->addAddress($value);
			} elseif (is_array($value) && count($value) == 2) {
				$this->addAddress($value[0], $value[1]);
			}
		}

		return $this;
	}

	/**
	 * 发送邮件
	 * @return bool
	 * @throws Exception
	 */
	public function send(): bool
	{
		$event = \phpvod::$app->event;
		$event->trigger(self::EVENT_MAIL_BEFORE_SEND, $this);

		$send = parent::send();

		if ($send)
			$event->trigger(self::EVENT_MAIL_SENT_SUCCESSFULLY, $this);
		else
			$event->trigger(self::EVENT_MAIL_SENT_FAILED, $this);

		return $send;
	}
}
