<?php

namespace phpvod\library\plupload;

use venus\http\request;
use venus\http\response;
use venus\language\language;

class plupload
{
	/**
	 * @var string 上传文件保存路径
	 */
	private string $save_path = WRITABLE_PATH . 'upload';

	/**
	 * @var string 上传文件临时保存路径 (.part)
	 */
	private string $temp_save_path = WRITABLE_PATH . 'temp';

	/**
	 * @var string url访问路径
	 */
	private string $url_path = 'upload';

	/**
	 * @var int 执行清除操作的概率(取值范围：0-100)
	 */
	private int $cleanup_probability = 10;

	/**
	 * @var int 临时文件超时时间
	 */
	private int $max_file_age = 6 * 3600;

	/**
	 * @var string 上传文件名(不重复)
	 */
	private string $name;

	/**
	 * @var ?string 客户端文件名
	 */
	private ?string $truename;

	/**
	 * @var ?int 客户端文件尺寸
	 */
	private ?int $size;

	/**
	 * @var int 当前分块序号
	 */
	private int $chunk;

	/**
	 * @var int 分块总数
	 */
	private int $chunks;

	/**
	 * @var string 允许上传的文件类型
	 */
	private string $format = '*';

	/**
	 * @var request 请求组件
	 */
	private request $request;

	/**
	 * @var response 响应组件
	 */
	private response $response;

	public function __construct()
	{
		helper('file_system');
		\phpvod::$app->language->load_file('plupload', language::NON_ADDITIONAL_LANGUAGE, [__DIR__ . '/language']);
		$this->request = \phpvod::$app->request;
		$this->response = \phpvod::$app->response;
	}

	/**
	 * 设置文件存储目录
	 * @param string $save_path
	 * @return plupload
	 */
	public function set_save_path(string $save_path): plupload
	{
		$this->save_path = $save_path;
		return $this;
	}

	/**
	 * 设置url访问路径
	 * @param string $url_path
	 * @return plupload
	 */
	public function set_url_path(string $url_path): plupload
	{
		$this->url_path = $url_path;
		return $this;
	}

	/**
	 * 设置执行清除操作的概率
	 * @param int $cleanup_probability 概率(取值范围:0-100)，设置为0时永不执行清除操作
	 */
	public function set_cleanup_probability(int $cleanup_probability): plupload
	{
		$this->cleanup_probability = $cleanup_probability;
		return $this;
	}

	/**
	 * 设置允许上传的文件类型
	 * @param string $format 文件类型，多个文件类型之间用半角逗号分开
	 * @return plupload
	 */
	public function set_format(string $format): plupload
	{
		$this->format = $format;
		return $this;
	}

	/**
	 * 接收上传数据
	 * @param callable|null $success_callback 文件上传成功后的回调方法，格式：
	 * <code>function (array $data)</code>
	 * <p>$data参数格式：</p>
	 * <code>
	 * [
	 *   'truename' => 'abc.mp4',
	 *   'size' => 100000,
	 *   'file' => 'c:\\pv4\\upload\\video\\video\\202306\\o_1h3m7jfu812081nii14aueljevnd.mp4',
	 *   'url' => 'upload/video/video/202306/o_1h3m7jfu812081nii14aueljevnd.mp4'
	 * ]
	 * </code>
	 * @return ?array
	 * @throws \Exception
	 */
	public function receive(callable $success_callback = null): ?array
	{
		//60 minutes execution time
		@set_time_limit(60 * 60);
		//response
		$this->response->set_header('Expires', 'Mon, 26 Jul 1997 05:00:00 GMT')
			->set_date_header('Last-Modified')
			->no_cache();
		//name & truename & size
		$this->name = $this->request->post('name');
		$this->name = preg_replace('/[^\w\._]+/', '_', $this->name);    // Clean the fileName for security reasons
		$this->truename = $this->request->post('truename') ?? null;
		$this->size = $this->request->post('size') ?? null;
		//chunk &chunks
		$this->chunk = $this->request->post('chunk', ['filter' => FILTER_VALIDATE_INT, 'flags' => FILTER_REQUIRE_SCALAR, 'options' => ['default' => 0]]);
		$this->chunks = $this->request->post('chunks', ['filter' => FILTER_VALIDATE_INT, 'flags' => FILTER_REQUIRE_SCALAR, 'options' => ['default' => 0]]);

		//cleanup dir
		$this->cleanup_dir();

		//create directory
		if(!is_dir($this->save_path)) {
			create_directory($this->save_path);
		}
		if(!is_dir($this->temp_save_path)) {
			create_directory($this->temp_save_path);
		}

		//file & url & temp_file & $part_file
		$file = $this->save_path . '/' . $this->name;
		$url = $this->url_path . '/' . $this->name;
		$temp_file = $this->temp_save_path . '/' . $this->name;
		$part_file = $temp_file . '.part';

		// Look for the content type header
		$content_type = $this->request->server('CONTENT_TYPE') ?? $this->request->server('HTTP_CONTENT_TYPE');

		// multipart in HTML5
		if (str_contains($content_type, "multipart")) {
			if (isset($_FILES['file']['tmp_name']) && is_uploaded_file($_FILES['file']['tmp_name'])) {
				// Open temp file
				$out = @fopen($part_file, $this->chunk == 0 ? "wb" : "ab");
				if ($out) {
					// Read binary input stream and append it to temp file
					$in = @fopen($_FILES['file']['tmp_name'], "rb");
					if ($in) {
						while ($buff = fread($in, 4096))
							fwrite($out, $buff);
					} else {
						return ['id' => -1, 'msg' => lang('plupload.failed_to_open_input_stream'), 'data' => ['file' => $_FILES['file']['tmp_name']]];
					}
					@fclose($in);
					@fclose($out);
					remove_file($_FILES['file']['tmp_name']);
				} else {
					return ['id' => -2, 'msg' => lang('plupload.failed_to_open_output_stream'), 'data' => ['file' => $part_file]];
				}
			} else {
				return ['id' => -3, 'msg' => lang('plupload.failed_to_move_uploaded_file')];
			}
		} else {
			return ['id' => -4, 'msg' => lang('plupload.not_multipart')];
		}

		// Check if file has been uploaded
		if (!$this->chunks || $this->chunk == $this->chunks - 1) {
			// Strip the temp .part suffix off
			rename($part_file, $temp_file);
			if ($this->check_extension($temp_file)) {
				if ($temp_file != $file) {
					rename($temp_file, $file);
				}
				// callback
				$data = null;
				if(is_callable($success_callback)) {
					$data = $success_callback(['truename' => $this->truename, 'size' => $this->size, 'file' => $file, 'url' => $url]);
				}
				return ['id' => 1, 'msg' => lang('plupload.file_upload_succeeded'), 'data' => $data];
			} else {
				remove_file($temp_file);
				return ['id' => -5, 'msg' => lang('plupload.invalid_file_extension'), 'data' => ['file' => $temp_file]];
			}
		}

		return null;
	}

	/**
	 * 检测扩展名
	 * @param string $filepath
	 * @return bool
	 */
	private function check_extension(string $filepath): bool
	{
		if ($this->format == '*')
			return true;

		$file_extension = strtolower(substr(strrchr(basename($filepath), '.'), 1));
		$valid_extension = explode(',', $this->format);
		return in_array($file_extension, $valid_extension);
	}

	/**
	 * 清理临时文件
	 * @return void
	 */
	private function cleanup_dir(): void
	{
		if ($this->cleanup_probability === 0) {
			return;
		}
		$rand = mt_rand(1, 100);
		if ($rand <= $this->cleanup_probability) {
			$current_part_file = $this->temp_save_path . '/' . $this->name . '.part';
			if (is_dir($this->temp_save_path) && ($dir = opendir($this->temp_save_path))) {
				while (($filename = readdir($dir)) !== false) {
					$file = $this->temp_save_path . '/' . $filename;
					// Remove temp file if it is older than the max age and is not the current file
					if (preg_match('/\.part$/', $filename) && (filemtime($file) < time() - $this->max_file_age) && ($file != $current_part_file)) {
						remove_file($file);
					}
				}
				closedir($dir);
			}
		}
	}
}