<?php

namespace phpvod\library;

use venus\cache\handler;

class verify_manager
{
	private static ?verify_manager $instance = null;

	private handler $handler; //缓存处理器
	private int $interval = 90; //重复发送验证码的最小时间间隔(秒)
	private int $ttl = 30 * 60; //验证码有效期(秒)
	private string $mail_template = 'code.html'; //邮件模板文件

	private function __construct(handler $handler)
	{
		$this->handler = $handler;
	}

	private function __clone()
	{
	}

	public static function get_instance(handler $handler): ?verify_manager
	{
		if (is_null(self::$instance)) {
			self::$instance = new verify_manager($handler);
		}
		return self::$instance;
	}

	/**
	 * 发送邮件验证码
	 * @throws \Exception
	 */
	public function send_email_code(string $action, string $email): bool
	{
		if (!$this->is_interval_enough($action, $email)) {
			throw new \Exception(lang('app.please_try_again_later'));
		}

		$code = $this->generate_verify_code();
		$mail = new mail();
		$send = $mail->set_subject(lang('home.code_email_subject'))
			->set_body_from_template($this->mail_template, [
				'{code}' => $code,
				'{ttl}' => floor($this->ttl / 60),
				'{action}' => lang('home.' . $action)
			])
			->add_address($email)
			->send();
		if ($send) {
			$this->set_code_data($action, $email, ['code' => $code, 'post_time' => TIMESTAMP]);
			return true;
		}
		return false;
	}

	/**
	 * 比较验证码
	 * @param string $action
	 * @param string $code
	 * @param string $key
	 * @return bool
	 */
	public function compare_code(string $code, string $action, string $key): bool
	{
		$code_data = $this->get_code_data($action, $key);
		return is_array($code_data) && $code_data['code'] === $code;
	}

	/**
	 * 获取重复发送验证码的最小时间间隔(秒)
	 * @return int
	 */
	public function get_interval(): int
	{
		return $this->interval;
	}

	/**
	 * 移除验证码
	 * @param string $action
	 * @param string $key
	 * @return void
	 */
	public function remove_code(string $action, string $key): void
	{
		$cache_key = $this->get_cache_key($action, $key);
		$this->handler->delete($cache_key);
	}

	/**
	 * 获取缓存键
	 * @param string $action
	 * @param string $key
	 * @return string
	 */
	private function get_cache_key(string $action, string $key): string
	{
		return md5(strtolower($action) . '_' . strtolower($key));
	}

	/**
	 * 生成验证码
	 * @throws \Exception
	 */
	private function generate_verify_code(): string
	{
		return (string)random_int(100000, 999999);
	}

	/**
	 * 从缓存中获取验证码数据
	 * @param string $action
	 * @param string $key
	 * @return array|null
	 */
	private function get_code_data(string $action, string $key): ?array
	{
		$cache_key = $this->get_cache_key($action, $key);
		$cache_value = $this->handler->get($cache_key);
		return is_array($cache_value) && isset($cache_value['code'], $cache_value['post_time']) ? $cache_value : null;
	}

	/**
	 * 将验证码数据写入缓存
	 * @param string $action
	 * @param string $key
	 * @param array $value
	 * @return void
	 */
	private function set_code_data(string $action, string $key, array $value): void
	{
		if (isset($value['code'], $value['post_time'])) {
			$cache_key = $this->get_cache_key($action, $key);
			$this->handler->set($cache_key, $value, $this->ttl);
		}
	}

	/**
	 * 重复发送验证码的时间间隔是否有效
	 * @param string $action
	 * @param string $key
	 * @return bool
	 */
	private function is_interval_enough(string $action, string $key): bool
	{
		$code_data = $this->get_code_data($action, $key);
		return !(is_array($code_data) && $code_data['post_time'] + $this->interval >= TIMESTAMP);
	}

}