<?php

namespace phpvod\model;

class language extends \phpvod\base\model
{
	protected string $table = 'pv_language';
	protected ?string $primary_key = 'id';
	protected array $allowed_fields = ['name', 'directory', 'default_language'];

	/**
	 * 获取数据
	 * @param array $params 查询参数，包含search/order_by/offset_limit元素(非null即可用)
	 * @return array
	 */
	public function get_data(array $params): array
	{
		//phpvod language
		//where
		if (!is_null($params['search'])) {
			$this->where('name LIKE :name OR directory LIKE :directory', [
				'name' => '%' . $params['search'] . '%',
				'directory' => '%' . $params['search'] . '%'
			]);
		}
		//order by
		if (!is_null($params['order_by'])) {
			$this->order_by($params['order_by']);
		}
		$rows = $this->select('*')->query()->fetch_all_array();

		//module language
		$modules = \phpvod::$app->module->get_modules();
		foreach ($modules as $module) {
			$module_language = $this->get_module_language(MODULE_PATH . $module['directory'] . '/language');
			foreach ($module_language as $directory => $name) {
				if (!is_null($params['search']) &&
					stripos($directory, $params['search']) === false &&
					stripos($name, $params['search']) === false
				) continue;
				$rows[] = [
					'name' => $name,
					'directory' => $directory,
					'module_namespace' => $module['namespace'],
					'module_name' => $module['name']
				];
			}
		}

		//total
		$total = count($rows);

		//offset/limit
		if (isset($params['offset']) && isset($params['limit'])) {
			$rows = array_slice($rows, $params['offset'], $params['limit']);
		}

		return ['total' => $total, 'rows' => $rows];
	}

	public function set_default_language(int $id): void
	{
		$this->where('id!=:id', ['id' => $id])->update(['default_language' => 0]);
		$this->update(['default_language' => 1], $id);
	}

	/**
	 * 安装语言
	 * @param string $zip_file 语言压缩包路径
	 * @return void
	 * @throws \Exception
	 */
	public function install(string $zip_file): void
	{
		// check data
		$data = get_zip_config($zip_file);
		if (!$this->check_config($data)) {
			throw new \Exception(lang('admin.config_format_error'));
		}

		if (isset($data['module_namespace'])) {
			//language_directory
			$module_directory = \phpvod::$app->module->get_module($data['module_namespace'], 'directory');
			if (is_null($module_directory)) {
				throw new \Exception(lang('admin.module_not_exists'));
			}
			$language_directory = MODULE_PATH . $module_directory . '/language';

			//Check if language exists
			$module_language = $this->get_module_language($language_directory);
			if (array_key_exists($data['directory'], $module_language) || in_array($data['name'], $module_language)) {
				throw new \Exception(lang('admin.language_exists'));
			}
		} else {
			//language_directory
			$language_directory = APP_PATH . 'language';

			//Check if language exists
			$count = (int)$this->select('count(id)')->where(['name' => $data['name'], 'directory' => $data['directory']], 'OR')->query()->fetch_column();
			if ($count > 0) {
				throw new \Exception(lang('admin.language_exists'));
			}
		}

		// is_writable
		if (!is_writable($language_directory)) {
			throw new \Exception(lang('admin.language_path_is_not_writable'));
		}

		// unzip
		if (!unzip($zip_file, $language_directory)) {
			throw new \Exception(lang('admin.language_unzip_failed'));
		}

		// install
		if (!array_key_exists('module_namespace', $data)) {
			if ($this->insert($data) <= 0) {
				throw new \Exception(lang('admin.language_install_failed'));
			}
		}
	}

	/**
	 * 卸载语言
	 * @param int $id 语言ID
	 * @return void
	 * @throws \Exception
	 */
	public function uninstall(int $id): void
	{
		$language = $this->select('*')->find($id)->fetch_array();

		if (!is_array($language)) {
			throw new \Exception(lang('app.invalid_id'));
		}

		if ($language['default_language'] == '1') {
			throw new \Exception(lang('admin.default_language_cannot_be_delete'));
		}

		if (\phpvod::$app->language->get_language() == $language['directory']) {
			throw new \Exception(lang('admin.current_language_cannot_be_delete'));
		}

		$this->delete($id);

		//delete files
		$language_directory = APP_PATH . 'language' . DIRECTORY_SEPARATOR . $language['directory'];
		if (!is_blank($language['directory']) &&
			is_dir($language_directory) &&
			@remove_directory($language_directory) !== true
		) {
			throw new \Exception(lang('admin.language_files_delete_failed', [$language_directory]), 2); //code:2
		}
	}

	/**
	 * 卸载模块语言
	 * @param string $module_namespace 模块名字空间
	 * @param string $directory 语言目录
	 * @return void
	 * @throws \Exception
	 */
	public function uninstall_module_language(string $module_namespace, string $directory): void
	{
		$module_directory = \phpvod::$app->module->get_module($module_namespace, 'directory');
		if (is_null($module_directory)) {
			throw new \Exception(lang('admin.module_not_exists'));
		}
		$language_directory = MODULE_PATH . $module_directory . '/language/' . $directory;

		if (!is_blank($directory) &&
			is_dir($language_directory) &&
			@remove_directory($language_directory) !== true
		) {
			throw new \Exception(lang('admin.language_files_delete_failed', [$language_directory]), 2); //code:2
		}
	}

	/**
	 * 检测配置数据格式是否正确
	 * @param array|null $data
	 * @return bool
	 */
	private function check_config(?array $data): bool
	{
		if (is_array($data) &&
			isset($data['name']) && !is_blank($data['name']) &&
			isset($data['directory']) && !is_blank($data['directory']) &&
			(!isset($data['module_namespace']) || !is_blank($data['module_namespace']))
		) {
			return true;
		}
		return false;
	}

	/**
	 * 获取模块语言列表
	 * @param string $language_directory 语言目录
	 * @param string $json_file 配置文件名称
	 * @return array 返回模块已安装的语言，格式：['en-US'=>'English', 'zh-CN'=>'简体中文', ...]
	 */
	private function get_module_language(string $language_directory, string $json_file = 'info.json'): array
	{
		$module_language = [];

		if (is_dir($language_directory) && $handle = opendir($language_directory)) {
			while (false !== ($entry = readdir($handle))) {
				if ($entry == '.' || $entry == '..' || !is_file($language_directory . '/' . $entry . '/' . $json_file)) {
					continue;
				}
				$json = file_get_contents($language_directory . '/' . $entry . '/' . $json_file);
				$config = json_decode($json, true);
				if (is_array($config) && array_key_exists('name', $config)) {
					$module_language[$entry] = trim($config['name']);
				}
			}
			closedir($handle);
		}

		return $module_language;
	}
}