<?php

namespace phpvod\model;

use phpvod\widget\form\admin_form;
use venus\module\action;

class module extends \phpvod\base\model implements action
{
	public const EVENT_MODULE_INSTALL = 'event_module_install';
	public const EVENT_MODULE_UNINSTALL = 'event_module_uninstall';
	public const EVENT_MODULE_STATUS_CHANGED = 'event_module_status_changed';

	protected string $table = 'pv_module';
	protected ?string $primary_key = 'id';
	protected array $allowed_fields = ['name', 'namespace', 'segment', 'directory', 'main', 'version', 'author', 'description', 'website', 'status', 'domain'];

	/**
	 * 获取数据
	 * @param array $params
	 * @return array
	 */
	public function get_data(array $params): array
	{
		//where
		if (!is_null($params['search'])) {
			$this->where('name LIKE :name OR namespace LIKE :namespace OR directory LIKE :directory', [
				'name' => '%' . $params['search'] . '%',
				'namespace' => '%' . $params['search'] . '%',
				'directory' => '%' . $params['search'] . '%'
			]);
		}

		$total = (int)$this->select('count(id)')->query(false)->fetch_column();

		//order by
		if (!is_null($params['order_by'])) {
			$this->order_by($params['order_by']);
		}

		//offset_limit
		if (!is_null($params['offset_limit'])) {
			$this->limit($params['offset_limit']);
		}

		$rows = $this->select('*')->query()->fetch_all_by_callback(function (array $row) {
			$row['status'] = admin_form::get_instance()->switch((boolean)$row['status'], ['class' => 'form-check-input', 'role' => 'switch']);
			return $row;
		});

		return ['total' => $total, 'rows' => $rows];
	}

	/**
	 * 更新模块状态
	 * @param int $id 用户ID
	 * @return int|false 成功返回更新后的状态，失败返回false
	 * @throws \Exception
	 */
	public function change_status(int $id): int|false
	{
		$affected_rows = $this->update(['status' => 'CASE status WHEN 0 THEN 1 ELSE 0 END'], $id, ['status']);
		$row = $this->select("namespace, status")->find($id)->fetch_array();

		if ($affected_rows > 0 && is_array($row)) {
			//event
			event()->trigger(self::EVENT_MODULE_STATUS_CHANGED, $row['namespace'], (int)$row['status']);
		}

		return is_array($row) ? (int)$row['status'] : false;
	}

	/**
	 * 安装模块
	 * @param string $zip_file 模块压缩包路径
	 * @return void
	 * @throws \Exception
	 */
	public function install(string $zip_file): void
	{
		// check data
		$data = get_zip_config($zip_file);
		if(!$this->check_config($data)) {
			throw new \Exception(lang('admin.config_format_error'));
		}

		// is_writable
		if (!is_writable(MODULE_PATH)) {
			throw new \Exception(lang('admin.module_path_is_not_writable'));
		}

		//Check if module exists
		$conflict = $this->select('name')->where([
			'name' => $data['name'],
			'namespace' => $data['namespace'],
			'segment' => $data['segment'],
			'directory' => $data['directory']
		], 'OR')->query()->fetch_column();
		if ($conflict !== false) {
			throw new \Exception(lang('admin.module_conflict', [$conflict]));
		}

		// unzip
		if (!unzip($zip_file, MODULE_PATH)) {
			throw new \Exception(lang('admin.module_unzip_failed'));
		}

		// run install.php
		$install_file = MODULE_PATH . $data['directory'] . DIRECTORY_SEPARATOR . self::INSTALL_FILE;
		if (is_file($install_file)) {
			$result = include $install_file;
			if ($result !== true) {
				throw new \Exception(lang('admin.module_script_exec_failed', [self::INSTALL_FILE]));
			}
		}

		// install
		if ($this->insert($data) <= 0) {
			throw new \Exception(lang('admin.module_install_failed'));
		}

		//event
		event()->trigger(self::EVENT_MODULE_INSTALL, $data['namespace']);
	}

	/**
	 * 卸载模块
	 * @param string $namespace 模块命名空间
	 * @return void
	 * @throws \Exception
	 */
	public function uninstall(string $namespace): void
	{
		$module = \phpvod::$app->module->get_module($namespace);
		if (is_null($module)) {
			throw new \Exception(lang('admin.module_not_exists'));
		}

		$main_module = \phpvod::$app->get_main_module();
		if($main_module == $namespace) {
			throw new \Exception(lang('admin.main_module_cannot_uninstall'));
		}

		// uninstall.php
		$module_directory = MODULE_PATH . $module['directory'];
		$uninstall_file = $module_directory . DIRECTORY_SEPARATOR . self::UNINSTALL_FILE;
		if (is_file($uninstall_file)) {
			$result = include $uninstall_file;
			if ($result !== true) {
				throw new \Exception(lang('admin.module_script_exec_failed', [self::UNINSTALL_FILE]));
			}
		}

		// uninstall
		$this->where('namespace=:namespace', ['namespace' => $namespace])->delete();

		//delete files
		if (!is_blank($module['directory']) &&
			is_dir($module_directory) &&
			@remove_directory($module_directory) !== true
		) {
			throw new \Exception(lang('admin.module_files_delete_failed', [$module_directory]), 2); //code:2
		}

		//event
		event()->trigger(self::EVENT_MODULE_UNINSTALL, $namespace);
	}

	/**
	 * 检测配置数据格式是否正确
	 * @param array|null $data 模块配置数据
	 * @return bool
	 */
	private function check_config(?array $data): bool
	{
		if (is_array($data) &&
			isset($data['name'], $data['namespace'], $data['segment'], $data['directory'], $data['main'], $data['version']) &&
			!is_blank($data['name']) && !is_blank($data['namespace']) && !is_blank($data['segment']) && !is_blank($data['directory']) && !is_blank($data['main']) && !is_blank($data['version'])
		) {
			return true;
		}
		return false;
	}
}