<?php

namespace phpvod\model;

use phpvod\library\template_manager;

class template extends \phpvod\base\model
{
	/**
	 * 获取数据
	 * @param array $params 查询参数，包含search/order_by/offset_limit元素(非null即可用)
	 * @return array
	 */
	public function get_data(array $params): array
	{
		$template_manager = template_manager::get_instance();

		//phpvod template
		$rows = $template_manager->get_template_list(APP_PATH . 'view');

		//module template
		$modules = \phpvod::$app->module->get_modules();
		foreach ($modules as $module) {
			$template_list = $template_manager->get_template_list(MODULE_PATH . $module['directory'] . '/view');
			foreach ($template_list as $template) {
				$template['module_name'] = $module['name'];
				$rows[] = $template;
			}
		}

		//search
		if (!is_null($params['search'])) {
			$rows = array_filter($rows, function (array $row) use ($params) {
				return !(stripos($row['name'], $params['search']) === false && stripos($row['directory'], $params['search']) === false);
			});
		}

		//total
		$total = count($rows);

		//offset/limit
		if (isset($params['offset']) && isset($params['limit'])) {
			$rows = array_slice($rows, $params['offset'], $params['limit']);
		}

		return ['total' => $total, 'rows' => $rows];
	}

	/**
	 * 安装模板
	 * @param string $zip_file 模板压缩包路径
	 * @return void
	 * @throws \Exception
	 */
	public function install(string $zip_file): void
	{
		//check config
		$data = get_zip_config($zip_file);
		if (!$this->check_config($data)) {
			throw new \Exception(lang('admin.config_format_error'));
		}

		//view directory
		if (isset($data['module_namespace'])) {
			$module_directory = \phpvod::$app->module->get_module($data['module_namespace'], 'directory');
			if (is_null($module_directory)) {
				throw new \Exception(lang('admin.module_not_exists'));
			}
			$view_directory = MODULE_PATH . $module_directory . '/view';
		} else {
			$view_directory = APP_PATH . 'view';
		}

		// is_writable
		if (!is_writable($view_directory)) {
			throw new \Exception(lang('admin.template_path_is_not_writable'));
		}

		$template_manager = template_manager::get_instance();

		//Check if template exists
		$template_list = $template_manager->get_template_list($view_directory);
		foreach ($template_list as $template) {
			if ($template['directory'] == $data['directory'] || $template['name'] == $data['name']) {
				throw new \Exception(lang('admin.template_exists'));
			}
		}

		// unzip
		if (!unzip($zip_file, $view_directory)) {
			throw new \Exception(lang('admin.template_unzip_failed'));
		}
	}

	/**
	 * 卸载模板
	 * @param string|null $module_namespace
	 * @param string $directory
	 * @return void
	 * @throws \Exception
	 */
	public function uninstall(?string $module_namespace, string $directory): void
	{
		//Default template are not allowed to be deleted
		if ($directory == 'home') {
			throw new \Exception(lang('admin.default_template_cannot_be_delete'));
		}

		//The template in use cannot be deleted
		$config_template = \phpvod\library\config\config::get_instance()->get('config_template');
		if (!is_null($module_namespace) && !is_blank($module_namespace)) {
			if (isset($config_template['module'][$module_namespace]) && $directory == $config_template['module'][$module_namespace]) {
				throw new \Exception(lang('admin.template_in_use_cannot_be_delete'));
			}
			//view_directory
			$module_directory = \phpvod::$app->module->get_module($module_namespace, 'directory');
			if (is_null($module_directory)) {
				throw new \Exception(lang('admin.module_not_exists'));
			}
			$view_directory = MODULE_PATH . $module_directory . '/view';
		} else {
			if (isset($config_template['phpvod']) && $directory == $config_template['phpvod']) {
				throw new \Exception(lang('admin.template_in_use_cannot_be_delete'));
			}
			//view_directory
			$view_directory = APP_PATH . 'view';
		}

		//delete files
		$template_directory = $view_directory . '/' . $directory;
		if (!is_blank($directory) &&
			is_dir($template_directory) &&
			@remove_directory($template_directory) !== true
		) {
			throw new \Exception(lang('admin.template_files_delete_failed', [$template_directory]), 2); //code:2
		}
	}

	/**
	 * 检测配置数据格式是否正确
	 * @param array|null $data
	 * @return bool
	 */
	private function check_config(?array $data): bool
	{
		if (is_array($data) &&
			(isset($data['name']) && !is_blank($data['name'])) &&
			(isset($data['directory']) && !is_blank($data['directory'])) &&
			(!isset($data['module_namespace']) || !is_blank($data['module_namespace']))
		) {
			return true;
		}
		return false;
	}
}