<?php

namespace phpvod\service;

use phpvod\model\config;
use phpvod\model\module;

class update_system
{
	public const EVENT_UPDATE = 'event_update';

	/**
	 * check_config
	 * @param array|null $data
	 * @return bool
	 */
	private function check_config(?array $data): bool
	{
		if (is_array($data) &&
			array_key_exists('module_namespace', $data) &&
			array_key_exists('applicable_version', $data) && !is_null($data['applicable_version']) && !is_blank($data['applicable_version']) &&
			array_key_exists('updated_version', $data) && !is_null($data['updated_version']) && !is_blank($data['updated_version'])
		) {
			return true;
		}
		return false;
	}

	/**
	 * get_module
	 * @param string|null $module_namespace
	 * @return array|null
	 * @throws \Exception
	 */
	private function get_module(?string $module_namespace): ?array
	{
		if (is_null($module_namespace) || is_blank($module_namespace)) {
			return null;
		}

		$module = \phpvod::$app->module->get_module($module_namespace);
		if (is_null($module)) {
			throw new \Exception(lang('admin.module_not_exists'), 0);
		}
		return $module;
	}

	/**
	 * 改变站点状态
	 * @param int $status 0关闭/1开启/-1恢复原状
	 * @return void
	 * @throws \Exception
	 */
	private function change_site_status(int $status): void
	{
		static $original_status = null;

		/* @var config $config_model */
		$config_model = model(config::class);
		$now_status = (int)$config_model->select('value')->where(['type' => 'site', 'name' => 'site_status'])->query()->fetch_column();
		if (is_null($original_status)) {
			$original_status = $now_status;
		}
		if ($status === -1 && !is_null($original_status)) {
			$status = $original_status;
		}
		if ($status === $now_status) {
			return;
		}
		$config_model->update_config(['site_status' => $status], 'site');
		\phpvod\library\config\config::get_instance()->update('config_site');
	}

	/**
	 * run_script
	 * @param string $script_file
	 * @return void
	 * @throws \Exception
	 */
	private function run_script(string $script_file): void
	{
		if (!is_file($script_file)) {
			return;
		}
		$result = include $script_file;
		if ($result !== true) {
			throw new \Exception(lang('admin.update_script_exec_failed', [basename($script_file)]));
		}
	}

	/**
	 * update_version
	 * @param array $data
	 * @return void
	 * @throws \Exception
	 */
	private function update_version(array $data): void
	{
		if (is_null($data['module_namespace']) || is_blank($data['module_namespace'])) { //phpvod
			$phpvod_content = file_get_contents(APP_PATH . 'phpvod.php');
			$phpvod_content = preg_replace('/public\s+const\s+VERSION\s*\=\s*[\'"]?(.+?)[\'"]?;/m', 'public const VERSION = \'' . $data['updated_version'] . '\';', $phpvod_content);
			file_put_contents(APP_PATH . 'phpvod.php', $phpvod_content);
		} else { //module
			/* @var module $module_model */
			$module_model = model(module::class);
			$module_model->where(['namespace' => $data['module_namespace']])->update(['version' => $data['updated_version']]);
			\phpvod\library\config\config::get_instance()->update('module');
		}
	}

	/**
	 * run
	 * @param string $update_package
	 * @return void
	 * @throws \Exception
	 */
	public function run(string $update_package): void
	{
		if (!is_file($update_package)) {
			throw new \Exception(lang('admin.package_not_exists'), 0);
		}

		helper('file_system');

		//info
		$data = get_zip_config($update_package);
		if (!$this->check_config($data)) {
			throw new \Exception(lang('admin.config_format_error'), 0);
		}

		//get module
		$module = $this->get_module($data['module_namespace']);

		//check version
		$version = is_array($module) ? $module['version'] : \phpvod::VERSION;
		if ($version != $data['applicable_version']) {
			throw new \Exception(lang('admin.version_not_match'), 0);
		}

		//target_directory
		$target_directory = is_array($module) ? MODULE_PATH . $module['directory'] : ROOT_PATH;

		//check permission
		if (!is_writable($target_directory)) {
			throw new \Exception(lang('admin.directory_is_not_writable'));
		}
		if (!is_array($module) && !is_writable(APP_PATH . 'phpvod.php')) {
			throw new \Exception(lang('admin.phpvod_is_not_writable'));
		}

		//unzip
		if (!unzip($update_package)) {
			throw new \Exception(lang('admin.package_unzip_failed'), 0);
		}

		$path_parts = pathinfo($update_package);
		$unzip_directory = $path_parts['dirname'] . DIRECTORY_SEPARATOR . $path_parts['filename'];
		$upload_directory = $unzip_directory . '/update_package/upload';
		$update_before = $unzip_directory . '/update_package/update_before.php';
		$update_after = $unzip_directory . '/update_package/update_after.php';

		try {
			//close site
			$this->change_site_status(0);

			//update_before
			$this->run_script($update_before);

			//update system
			if (is_dir($upload_directory)) {
				try {
					copy_directory($upload_directory, $target_directory, info: $info);
				} catch (\Exception) {
					throw new \Exception(lang('admin.copy_directory_throw_exception'));
				}
				if ($info['failure_count'] > 0) {
					throw new \Exception(lang('admin.files_or_directories_copy_failed', [$info['failure_count']]));
				}
			}

			//update_after
			$this->run_script($update_after);

			//update version
			$this->update_version($data);

			//event
			event()->trigger(self::EVENT_UPDATE, $data);
		} finally {
			//recovery site
			$this->change_site_status(-1);
			//delete package
			remove_file($update_package);
			//delete unzip directory
			remove_directory($unzip_directory);
		}
	}

}