<?php

namespace phpvod\widget\pagination;

class pagination
{
	public const ALIGN_LEFT = 0;
	public const ALIGN_CENTER = 1;
	public const ALIGN_RIGHT = 2;

	public const NUM_PLACEHOLDER = '(:num)';
	protected int $total_items;
	protected int $items_per_page;
	protected int $current_page;
	protected string $url_pattern;
	protected int $total_pages;
	protected int $page_items = 9;
	protected string $template = 'default';
	protected int $align = 0;

	public function __construct(int $total_items, int $items_per_page, int $current_page, string $url_pattern)
	{
		$this->total_items = $total_items;
		$this->items_per_page = $items_per_page;
		$this->current_page = $current_page;
		$this->url_pattern = $url_pattern;
		$this->update_total_pages();
	}

	protected function update_total_pages(): void
	{
		$this->total_pages = $this->items_per_page == 0 ? 0 : (int)ceil($this->total_items / $this->items_per_page);
	}

	/**
	 * 获取总页数
	 * @return int
	 */
	public function get_total_pages(): int
	{
		return $this->total_pages;
	}

	public function get_total_items(): int
	{
		return $this->total_items;
	}

	public function set_total_items(int $total_items): static
	{
		$this->total_items = $total_items;
		$this->update_total_pages();
		return $this;
	}

	public function get_items_per_page(): int
	{
		return $this->items_per_page;
	}

	public function set_items_per_page(int $items_per_page): static
	{
		$this->items_per_page = $items_per_page;
		$this->update_total_pages();
		return $this;
	}

	public function get_current_page(): int
	{
		return $this->current_page;
	}

	public function set_current_page(int $current_page): static
	{
		$this->current_page = $current_page;
		return $this;
	}

	public function get_url_pattern(): string
	{
		return $this->url_pattern;
	}

	public function set_url_pattern(string $url_pattern): static
	{
		$this->url_pattern = $url_pattern;
		return $this;
	}

	public function get_page_items(): int
	{
		return $this->page_items;
	}

	public function set_page_items(int $page_items): static
	{
		if ($page_items < 3) {
			throw new \InvalidArgumentException('Page items cannot be less than 3.');
		}
		$this->page_items = $page_items;
		return $this;
	}

	public function get_template(): string
	{
		return $this->template;
	}

	public function set_template(string $template): static
	{
		$this->template = $template;
		return $this;
	}

	/**
	 * 获取对齐试工
	 * @return int
	 */
	public function get_align(): int
	{
		return $this->align;
	}

	/**
	 * 设置对齐方式(仅用于bootstrap模板)
	 * @param int $align
	 * @return pagination
	 */
	public function set_align(int $align): static
	{
		$this->align = $align;
		return $this;
	}

	/**
	 * 是否存在上一页
	 * @return bool
	 */
	public function has_prev(): bool
	{
		return $this->current_page > 1;
	}

	/**
	 * 是否存在下一页
	 * @return bool
	 */
	public function has_next(): bool
	{
		return $this->current_page < $this->total_pages;
	}

	/**
	 * 通过页码获取页面url
	 * @param $page_number
	 * @return string
	 */
	public function get_page_url($page_number): string
	{
		return str_replace(self::NUM_PLACEHOLDER, $page_number, $this->url_pattern);
	}

	/**
	 * 获取下一页url
	 * @return string|null
	 */
	public function get_next_url(): ?string
	{
		return $this->has_next() ? $this->get_page_url($this->current_page + 1) : null;
	}

	/**
	 * 获取上一页url
	 * @return string|null
	 */
	public function get_prev_url(): ?string
	{
		return $this->has_prev() ? $this->get_page_url($this->current_page - 1) : null;
	}

	/**
	 * 获取分页页面数据
	 *
	 * Example:
	 * array(
	 *     array ('num' => 1,     'url' => '/example/page/1',  'is_current' => false),
	 *     array ('num' => '...', 'url' => NULL,               'is_current' => false),
	 *     array ('num' => 3,     'url' => '/example/page/3',  'is_current' => false),
	 *     array ('num' => 4,     'url' => '/example/page/4',  'is_current' => true ),
	 *     array ('num' => 5,     'url' => '/example/page/5',  'is_current' => false),
	 *     array ('num' => '...', 'url' => NULL,               'is_current' => false),
	 *     array ('num' => 10,    'url' => '/example/page/10', 'is_current' => false),
	 * )
	 *
	 * @return array
	 */
	public function get_page_data(): array
	{
		$pages = [];

		if ($this->total_pages <= 1) {
			return [];
		}

		if ($this->total_pages <= $this->page_items) {
			for ($i = 1; $i <= $this->total_pages; $i++) {
				$pages[] = [
					'num' => $i,
					'url' => $this->get_page_url($i),
					'is_current' => $i == $this->current_page
				];
			}
		} else {
			// 确定以当前页面为中心的显示范围
			$num_adjacents = (int)floor(($this->page_items - 3) / 2);

			if ($this->current_page + $num_adjacents > $this->total_pages) {
				$sliding_start = $this->total_pages - $this->page_items + 2;
			} else {
				$sliding_start = $this->current_page - $num_adjacents;
			}
			if ($sliding_start < 2) $sliding_start = 2;

			$sliding_end = $sliding_start + $this->page_items - 3;
			if ($sliding_end >= $this->total_pages) $sliding_end = $this->total_pages - 1;

			// 构建页面列表
			$pages[] = ['num' => 1, 'url' => $this->get_page_url(1), 'is_current' => $this->current_page == 1];
			if ($sliding_start > 2) {
				$pages[] = ['num' => '...', 'url' => null, 'is_current' => false];
			}
			for ($i = $sliding_start; $i <= $sliding_end; $i++) {
				$pages[] = ['num' => $i, 'url' => $this->get_page_url($i), 'is_current' => $i == $this->current_page];
			}
			if ($sliding_end < $this->total_pages - 1) {
				$pages[] = ['num' => '...', 'url' => null, 'is_current' => false];
			}
			$pages[] = ['num' => $this->total_pages, 'url' => $this->get_page_url($this->total_pages), 'is_current' => $this->total_pages == $this->current_page];
		}

		return $pages;
	}

	/**
	 * 渲染分页
	 * @return string
	 * @throws \Exception
	 */
	public function render(): string
	{
		$view_file = __DIR__ . '/view/' . $this->template . '.php';

		if (!is_file($view_file)) {
			throw new \Exception('View does not exist (' . $view_file . ')');
		}

		return $this->total_pages > 1 ? (function (string $view_file): string {
			$data = ['pagination' => $this];
			extract($data, EXTR_OVERWRITE);
			ob_start();
			include $view_file;
			return (string)ob_get_clean();
		})($view_file) : '';
	}

	/**
	 * 渲染分页
	 * @return string
	 * @throws \Exception
	 */
	public function __toString(): string
	{
		return $this->render();
	}
}