<?php

namespace venus\application;

use venus\application\exception\controller_exception;
use venus\cache\cache;
use venus\database\database;
use venus\debug\timer;
use venus\di\container;
use venus\event\event;
use venus\filter\filter;
use venus\http\curl_request;
use venus\http\request;
use venus\http\response;
use venus\http\status;
use venus\language\language;
use venus\module\module;
use venus\router\router;
use venus\validation\validation;
use venus\view\view;

/**
 * Class application
 * @package venus\application
 * @property request $request
 * @property response $response
 * @property curl_request $curl_request
 * @property database $database
 * @property router $router
 * @property filter $filter
 * @property validation $validation
 * @property language $language
 * @property module $module
 * @property view $view
 * @property event $event
 * @property cache $cache
 * @property timer $timer
 *
 * PHPVOD
 * @property \phpvod\component\user_auth\user_auth $user_auth
 */
class application
{
	//event
	public const EVENT_BEFORE_CONTROLLER_INSTANCE = 'event_before_controller_instance';
	public const EVENT_BEFORE_ACTION_EXECUTE = 'event_before_action_execute';
	public const EVENT_AFTER_ACTION_EXECUTE = 'event_after_action_execute';
	public const EVENT_BEFORE_RESPONSE_SEND = 'event_before_response_send';

	private array $components = [];

	private \config\application $config;

	public function __construct(\config\application $config)
	{
		$this->config = $config;
	}

	/**
	 * __get
	 * @throws \Exception
	 */
	public function __get(string $name): object
	{
		if (!isset($this->components[$name])) {

			if (!isset($this->config->class_map[$name])) {
				throw new \Exception('Invalid component name (' . $name . ')');
			}

			$class = $this->config->class_map[$name];
			$this->components[$name] = container::get($class);

		}
		return $this->components[$name];
	}

	/**
	 * 获取主模块
	 * @return string
	 */
	public function get_main_module(): string
	{
		return $this->config->main_module;
	}

	/**
	 * 注册组件
	 * @param string $name 组件名称
	 * @param string $class 组件类名
	 * @return void
	 */
	public function register(string $name, string $class): void
	{
		if (!isset($this->config->class_map[$name])) {
			$this->config->class_map[$name] = $class;
		}
	}

	/**
	 * init
	 * @throws \Exception
	 */
	public function init(): application
	{
		// define ENV constants (prod/dev)
		define('ENV', $this->config->env ?? 'prod');

		//timezone
		define('TIMESTAMP', time());
		date_default_timezone_set($this->config->timezone ?? 'UTC');

		return $this;
	}

	/**
	 * run
	 * @throws \Exception
	 */
	public function run(): response
	{
		//application timer
		$this->timer->start('application');

		//ob_start
		ob_start();

		//session
		session()->start();

		//create router -> load settings -> handle
		$data = $this->router->load_settings()->handle();

		//handle request
		return $this->handle_request($data);
	}

	/**
	 * 处理请求
	 * @param array|false $data
	 * @return response
	 * @throws \Exception
	 */
	private function handle_request(array|false $data): response
	{
		$response = \venus::$app->response;
		$event = null;

		if ($data === false) {
			throw new \Exception('No routing rule was matched or the controller does not exist (location:' .
				\venus::$app->request->server('REQUEST_URI') . ')');
		}

		//filter manager
		$filter = \venus::$app->filter;

		//run the filter before method
		$filter->run('before');

		$result = false;
		$count = count($data);
		if ($count == 2 && $data[0] instanceof \Closure) {

			// run closure
			$result = call_user_func_array($data[0], $data[1]);

		} elseif ($count == 3) {

			//check controller
			if (!class_exists($data[0])) {
				throw controller_exception::for_not_exists($data[0]);
			}

			//set current module
			$module = \venus::$app->module;
			$module->set_current_module($module->get_module($data[0]));

			//event
			$event = \venus::$app->event->init(); //创建event组件，并初始化事件订阅。
			//event_before_controller_instance
			$event->trigger(self::EVENT_BEFORE_CONTROLLER_INSTANCE);

			try {
				//create controller
				$controller = new $data[0];

				//run init method
				$controller->init();

				//check method
				if (!is_callable([$controller, $data[1]])) {
					throw controller_exception::for_invalid_method($data[0] . '::' . $data[1]);
				}

				//event_before_action_execute
				$event->trigger(self::EVENT_BEFORE_ACTION_EXECUTE);

				//run controller
				$result = call_user_func_array([$controller, $data[1]], $data[2]);

				//event_after_action_execute
				$event->trigger(self::EVENT_AFTER_ACTION_EXECUTE);
			} catch (controller_exception $controller_exception) {
				if ($controller_exception->getCode() != controller_exception::INTERRUPT) {
					throw $controller_exception;
				}
			}
		}

		//run the filter after method
		$filter->run('after');

		//gather output
		$this->gather_output($result);

		//event_before_response_send
		!is_null($event) && $event->trigger(self::EVENT_BEFORE_RESPONSE_SEND);

		//将当前url保存至会话中，为了更安全、更准确地使用previous_url()辅助函数。
		$this->store_url();

		//total_time
		$total_time = $this->timer->get_elapsed_time('application');
		$response->replace_body('{total_time}', $total_time);

		return $response->send();
	}

	private function gather_output($result): void
	{
		$response = \venus::$app->response;

		$output = ob_get_contents();
		ob_end_clean();

		if (is_string($output)) {
			$response->append_body($output);
		}

		if (is_string($result)) {
			$response->append_body($result);
		}
	}

	private function store_url(): void
	{
		// Ignore CLI requests
		if (is_cli()) {
			return;
		}
		// Ignore AJAX requests
		if (\venus::$app->request->is_ajax()) {
			return;
		}
		// Ignore non-HTML responses
		if (!str_contains(\venus::$app->response->get_header('Content-Type'), 'text/html')) {
			return;
		}
		// Ignore status code is not 200
		if (\venus::$app->response->get_status_code() !== status::HTTP_OK) {
			return;
		}
		// store
		if (isset($_SESSION)) {
			$_SESSION['_venus_previous_url'] = \venus::$app->request->get_url();
		}
	}
}