<?php

namespace venus\asset;

use venus\view\event;

class asset
{
	private static array $base_url_list = [];
	protected static array $loaded_js = [];
	protected static array $loaded_css = [];
	/**
	 * @var string $source_path 指定包包含资源文件的根目录，当根目录不能被 Web 访问时该属性应设置，否则，应设置 base_path 属性和 base_url
	 */
	protected string $source_path = '';
	protected string $base_path = '';
	protected string $base_url = '';
	/**
	 * @var string $asset_dirname 资源发布后的目录名称，如为空，则使用hash方法生成。
	 */
	protected string $asset_dirname = '';
	protected array $js = [];
	protected array $css = [];
	protected array $depends = [];
	/**
	 * 语言标签
	 * @var string
	 */
	protected string $lang_tag = '{language}';
	/**
	 * 语言标签格式，可用标签:
	 * <p>{lang}、{LANG}、{Lang}</p>
	 * <p>{tag}、{TAG}、{Tag}</p>
	 * <p>分别表示小写、大写、首字母大写</p>
	 * @var array
	 */
	protected array $lang_format = [
		'{lang}',
		'{lang}_{TAG}',
		'{lang}-{TAG}',
	];
	protected array $js_options = [
		'event' => event::EVENT_VIEW_IMPORT_JS
	];
	protected array $css_options = [
		'event' => event::EVENT_VIEW_HEAD
	];
	protected array $publish_options = [
		/**
		 * 当 link_assets 为 true 时，会创建一个符号链接到要发布的资源包源路径。
		 * 如果操作系统和 Web 服务器允许可以使用符号链接，建议将该值设置为 true, 这比拷贝文件方式快并能确保发布的资源一直为最新的
		 */
		'link_assets' => true,
		'publish_path' => PUBLIC_PATH . 'assets',
		'publish_url' => 'assets',
	];

	/**
	 * 注册资源
	 * @return asset
	 */
	public static function register(): asset
	{
		$class_name = get_called_class();
		/**
		 * @var asset $obj
		 */
		$obj = new $class_name();
		$obj->init();
		return $obj;
	}

	/**
	 * Asset constructor.
	 */
	private function __construct()
	{
	}

	protected function init(): void
	{
		foreach ($this->depends as $depend) {
			$depend::register();
		}
		$this->publish();

		if (!empty($this->js)) {
			\venus::$app->event->on($this->js_options['event'], [$this, 'register_js_files']);
		}
		if (!empty($this->css)) {
			\venus::$app->event->on($this->css_options['event'], [$this, 'register_css_files']);
		}

		!isset(self::$base_url_list[static::class]) && self::$base_url_list[static::class] = $this->base_url;
	}

	/**
	 * 将js路径中的语言标签按标签格式替换为当前语言，如果文件存在则返回，否则返回null
	 * @param string $js
	 * @return string|null
	 */
	private function replace_lang_tag(string $js): ?string
	{
		if (str_contains($js, $this->lang_tag)) {
			$base_path = !empty($this->source_path) ? $this->source_path : $this->base_path;

			foreach ($this->lang_format as $format) {
				$lang = \venus::$app->language->get_format_language($format);
				if (is_null($lang)) continue;
				$js_file = str_replace($this->lang_tag, $lang, $js);
				if (is_file($base_path . DIRECTORY_SEPARATOR . $js_file)) {
					return $js_file;
				}
			}

			return null;
		}
		return $js;
	}

	public function register_js_files(): void
	{
		$js_tag = '';
		foreach ($this->js as $js) {
			if (!$this->is_absolute_url($js)) {
				$js = $this->replace_lang_tag($js); //替换语言
				if (is_null($js)) continue;
				$js = $this->base_url . $js;
			}
			if (in_array($js, self::$loaded_js)) {
				continue;
			}
			$js_tag .= '<script type="text/javascript" src="' . $js . '"><' . '/script>' . PHP_EOL;
			self::$loaded_js[] = $js;
		}
		echo $js_tag;
	}

	public function register_css_files(): void
	{
		$css_tag = '';
		foreach ($this->css as $css) {
			if (!$this->is_absolute_url($css)) {
				$css = $this->base_url . $css;
			}
			if (in_array($css, self::$loaded_css)) {
				continue;
			}
			$css_tag .= '<link href="' . $css . '" rel="stylesheet" />' . PHP_EOL;
			self::$loaded_css[] = $css;
		}
		echo $css_tag;
	}

	protected function publish(): void
	{
		if (empty($this->source_path) || !empty($this->base_path) || !empty($this->base_url)) {
			return;
		}

		$this->source_path = rtrim($this->source_path, '\\/');
		$dir = !empty($this->asset_dirname) ? $this->asset_dirname : $this->hash($this->source_path);

		$this->publish_options['publish_path'] = rtrim($this->publish_options['publish_path'], '\\/');
		$this->publish_options['publish_url'] = rtrim($this->publish_options['publish_url'], '/');

		$publish_asset_path = $this->publish_options['publish_path'] . DIRECTORY_SEPARATOR . $dir;
		$publish_asset_url = $this->publish_options['publish_url'] . '/' . $dir;

		$this->base_path = $publish_asset_path;
		$this->base_url = $publish_asset_url . '/';

		if (file_exists($publish_asset_path)) {
			return;
		}

		helper('file_system');
		if (!is_dir($this->publish_options['publish_path'])) {
			try {
				if (create_directory($this->publish_options['publish_path']) !== true) {
					return;
				}
			} catch (\Exception) {
				return;
			}
		}

		if ($this->publish_options['link_assets']) { //创建资源链接
			symlink($this->source_path, $publish_asset_path);
		} else {
			try {
				copy_directory($this->source_path, $publish_asset_path);
			} catch (\Exception) {
				return;
			}
		}
	}

	/**
	 * 获取 base_url
	 * @return string|null
	 */
	public static function get_base_url(): ?string
	{
		return self::$base_url_list[static::class] ?? null;
	}

	private function hash(string $path): string
	{
		$path = (is_file($path) ? dirname($path) : $path) . \venus::VERSION;
		return sprintf('%x', crc32($path));
	}

	private function is_absolute_url(string $url): bool
	{
		$url = strtolower($url);
		return str_starts_with($url, 'http://') || str_starts_with($url, 'https://') || str_starts_with($url, '//');
	}
}
