<?php

namespace venus\base;

use venus\database\database;
use venus\database\statement;
use venus\event\event;
use venus\validation\validation;

class model
{
	public const EVENT_MODEL_BEFORE_QUERY = 'event_model_before_query';
	public const EVENT_MODEL_AFTER_QUERY = 'event_model_after_query';
	public const EVENT_MODEL_INSERT = 'event_model_insert';
	public const EVENT_MODEL_UPDATE = 'event_model_update';
	public const EVENT_MODEL_DELETE = 'event_model_delete';

	//占位符序号
	private static int $placeholder_index = 0;

	//占位符实参
	private array $params = [];

	//验证器
	protected validation $validation;

	//数据库访问组件
	protected database $db;

	//事件
	protected event $event;

	/**
	 * @var string 表名
	 */
	protected string $table;

	/**
	 * @var string|null 主键
	 */
	protected ?string $primary_key = null;

	/**
	 * @var array 允许更新的字段
	 */
	protected array $allowed_fields = [];

	/**
	 * @var ?string 用于保存创建时间戳的字段名
	 */
	protected ?string $created_field = null;

	/**
	 * @var ?string 用于保存更新时间戳的字段名
	 */
	protected ?string $updated_field = null;

	//select
	private ?string $select = null;

	//from
	private ?string $from = null;

	//where
	private ?string $where = null;

	//group by
	private ?string $group_by = null;

	//having
	private ?string $having = null;

	//order by
	private ?string $order_by = null;

	//limit
	private ?string $limit = null;

	public function __construct()
	{
		$this->db = \venus::$app->database;
		$this->validation = \venus::$app->validation;
		$this->event = \venus::$app->event;
	}

	/**
	 * 数据库查询
	 * @param bool $reset 重置参数
	 * @return statement
	 */
	public function query(bool $reset = true): statement
	{
		is_null($this->select) && $this->select = '*';
		is_null($this->from) && $this->from = $this->table;

		$where = !is_null($this->where) ? ' WHERE ' . $this->where : '';
		$group_by = !is_null($this->group_by) ? ' GROUP BY ' . $this->group_by : '';
		$having = !is_null($this->having) ? ' HAVING ' . $this->having : '';
		$order_by = !is_null($this->order_by) ? ' ORDER BY ' . $this->order_by : '';
		$limit = !is_null($this->limit) ? ' LIMIT ' . $this->limit : '';

		$query = "SELECT $this->select FROM $this->from{$where}{$group_by}{$having}{$order_by}{$limit}";
		$args = ['query' => $query, 'params' => $this->params];
		$this->event->trigger(self::EVENT_MODEL_BEFORE_QUERY, $args);
		$statement = $this->db->query($query, $this->params);
		$this->event->trigger(self::EVENT_MODEL_AFTER_QUERY, $args);

		$reset && $this->reset();
		return $statement;
	}

	/**
	 * 查找记录
	 * @param int $id
	 * @param bool $reset
	 * @return statement
	 * @throws \Exception
	 */
	public function find(int $id, bool $reset = true): statement
	{
		if (is_null($this->primary_key)) {
			throw new \Exception('The primary key is undefined');
		}
		$this->where = $this->primary_key . '=' . $id;
		return $this->query($reset);
	}

	/**
	 * 插入记录
	 * @param object|array $data 关联数组或对象
	 * @return int 返回受影响的行数
	 */
	public function insert(object|array $data): int
	{
		$affected_rows = 0;

		//created_field
		if (is_string($this->created_field) && !array_key_exists($this->created_field, $data)) {
			$data[$this->created_field] = TIMESTAMP;
		}

		//updated_field
		if (is_string($this->updated_field) && !array_key_exists($this->updated_field, $data)) {
			$data[$this->updated_field] = TIMESTAMP;
		}

		[$field_name, $field_value, $params] = $this->insert_placeholder($data);
		if (!is_null($field_name) && !is_null($field_value)) {
			is_array($params) && $this->append_params($params);
			$query = "INSERT INTO $this->table ($field_name) VALUES ($field_value)";
			$affected_rows = $this->db->update($query, $this->params);
			//event
			$this->event->trigger(self::EVENT_MODEL_INSERT, [
				'query' => $query,
				'params' => $this->params,
				'affected_rows' => $affected_rows,
				'table' => $this->table,
				'field_name' => $field_name,
				'field_value' => $field_value
			]);
		}

		$this->reset();
		return $affected_rows;
	}

	/**
	 * 更新记录
	 * @param object|array $data 关联数组或对象
	 * @param int|null $id id，如为null，可使用where方法设置条件
	 * @param array $safe_field 安全字段，该字段值不会添加转义
	 * @return int 返回受影响的行数
	 */
	public function update(object|array $data, ?int $id = null, array $safe_field = []): int
	{
		$affected_rows = 0;

		if (is_int($id) && !is_null($this->primary_key)) {
			$this->where = $this->primary_key . '=:primary_key';
			$this->append_params(['primary_key' => $id]);
		}

		//updated_field
		if (is_string($this->updated_field) && !array_key_exists($this->updated_field, $data)) {
			$data[$this->updated_field] = TIMESTAMP;
		}

		[$expression, $params] = $this->update_placeholder($data, $safe_field);

		if (!is_null($expression) && !is_null($this->where)) {
			is_array($params) && $this->append_params($params);
			$query = "UPDATE $this->table SET $expression WHERE $this->where";
			$affected_rows = $this->db->update($query, $this->params);
			//event
			$this->event->trigger(self::EVENT_MODEL_UPDATE, [
				'query' => $query,
				'params' => $this->params,
				'affected_rows' => $affected_rows,
				'table' => $this->table,
				'set' => $expression,
				'where' => $this->where,
			]);
		}

		$this->reset();
		return $affected_rows;
	}

	/**
	 * 删除记录
	 * @param array|int|null $ids id或id数组，如为null，可使用where方法设置条件
	 * @return int 返回受影响的行数
	 */
	public function delete(array|int|null $ids = null): int
	{
		$affected_rows = 0;

		if (!is_null($this->primary_key)) {
			if (is_numeric($ids)) {
				$this->where = $this->primary_key . '=:primary_key';
				$this->append_params(['primary_key' => $ids]);
			} elseif (is_array($ids)) {
				$this->where_in($this->primary_key, $ids);
			}
		}

		if (!is_null($this->where)) {
			$query = "DELETE FROM $this->table WHERE $this->where";
			$affected_rows = $this->db->update($query, $this->params);
			//event
			$this->event->trigger(self::EVENT_MODEL_DELETE, [
				'query' => $query,
				'params' => $this->params,
				'affected_rows' => $affected_rows,
				'table' => $this->table,
				'where' => $this->where,
			]);
		}

		$this->reset();
		return $affected_rows;
	}

	/**
	 * 返回最后插入行的ID或序列值
	 * @param string|null $name
	 * @return string
	 */
	public function insert_id(?string $name = null): string
	{
		return $this->db->insert_id($name);
	}

	/**
	 * 删除所有记录
	 * @return int 返回删除的行数
	 */
	public function empty_table(): int
	{
		/** @noinspection SqlWithoutWhere */
		return $this->db->update("DELETE FROM $this->table");
	}

	/**
	 * 清空表
	 * @return void
	 */
	public function truncate(): void
	{
		$this->db->exec("TRUNCATE $this->table");
	}

	/**
	 * 保存数据
	 * @param object|array $data 关联数组或对象
	 * @return int 返回受影响的行数
	 */
	public function save(object|array $data): int
	{
		$affected_rows = 0;

		is_object($data) && $data = get_object_vars($data); //object to array

		if (is_array($data)) {
			if (!is_null($this->primary_key) && array_key_exists($this->primary_key, $data)) {
				//update
				$affected_rows = $this->update($data, $data[$this->primary_key]);
			} else {
				//insert
				$affected_rows = $this->insert($data);
			}
		}

		$this->reset();
		return $affected_rows;
	}

	private function append_where(string $where, string $logical = 'AND'): model
	{
		$this->where .= is_null($this->where) || str_ends_with($this->where, '(') ? $where : " $logical $where";
		return $this;
	}

	private function append_params(array $params): model
	{
		$this->params = array_merge($this->params, $params);
		return $this;
	}

	public function select(?string $select): model
	{
		$this->select = $select;
		return $this;
	}

	public function from(?string $from): model
	{
		$this->from = $from;
		return $this;
	}

	/**
	 * WHERE
	 * @param string|array $where where条件，可为字符串或关联数组。如是关联数组，数组key可带运算符(=、<、<=、>、>=、!=)，不带运算符时默认为等于(=)
	 * @param array|string|null $args
	 * <p>where是字符串时，该参数为占位符参数数组，如为null则表示无参数</p>
	 * <p>where是数组时，该参数为逻辑运算符字符串(AND/OR)，如为null则为AND</p>
	 * @param string $logical 多次调用该方法追加条件时使用的逻辑运算符字符串(AND/OR)
	 * @return $this
	 *
	 * <p>格式一：</p>
	 * <code>public function where(string $where, ?array $args = null, string $logical = 'AND')</code>
	 * <p>格式二：</p>
	 * <code>public function where(array $where, ?string $args = null, string $logical = 'AND')</code>
	 */
	public function where(string|array $where, array|string|null $args = null, string $logical = 'AND'): model
	{
		//string
		if (is_string($where)) {
			$this->append_where($where, $logical);
			is_array($args) && $this->append_params($args);
		} //array
		elseif (is_array($where)) {
			if (!is_string($args) || !in_array(strtoupper($args), ['AND', 'OR'])) {
				$args = 'AND';
			}
			[$expression, $params] = $this->where_placeholder($where, $args);
			!is_null($expression) && $this->append_where($expression, $logical);
			is_array($params) && $this->append_params($params);
		}
		return $this;
	}

	public function or_where(string|array $where, array|string|null $args = null): model
	{
		$this->where($where, $args, 'OR');
		return $this;
	}

	/**
	 * where_in_not_in
	 * @param string $field 字段名称
	 * @param string $operator 操作符(IN/NOT IN)
	 * @param array $array 索引数组
	 * @param string $logical 多次调用该方法追加条件时使用的逻辑运算符字符串(AND/OR)
	 * @return model
	 */
	private function where_in_not_in(string $field, string $operator, array $array, string $logical = 'AND'): model
	{
		[$expression, $params] = $this->where_in_placeholder($array);
		!is_null($expression) && $this->append_where("$field $operator ($expression)", $logical);
		is_array($params) && $this->append_params($params);
		return $this;
	}

	public function where_in(string $field, array $array, string $logical = 'AND'): model
	{
		return $this->where_in_not_in($field, 'IN', $array, $logical);
	}

	public function where_not_in(string $field, array $array, string $logical = 'AND'): model
	{
		return $this->where_in_not_in($field, 'NOT IN', $array, $logical);
	}

	public function group_start(string $logical = 'AND'): model
	{
		$this->append_where('(', $logical);
		return $this;
	}

	public function or_group_start(): model
	{
		return $this->group_start('OR');
	}

	public function group_end(): model
	{
		$this->where .= ')';
		return $this;
	}

	public function group_by(?string $group_by): model
	{
		$this->group_by = $group_by;
		return $this;
	}

	public function having(?string $having): model
	{
		$this->having = $having;
		return $this;
	}

	public function order_by(?string $order_by): model
	{
		$this->order_by = $order_by;
		return $this;
	}

	public function limit(?string $limit): model
	{
		$this->limit = $limit;
		return $this;
	}

	public function reset(): model
	{
		$this->params = [];
		$this->select = null;
		$this->from = null;
		$this->where = null;
		$this->group_by = null;
		$this->having = null;
		$this->order_by = null;
		$this->limit = null;
		return $this;
	}

	/**
	 * 生成 where 的占位符表达式及参数
	 *  <p>key=:placeholder1 AND key>:placeholder2 AND key!=:placeholder3 ...</p>
	 * @param array $array 关联数组，key中可带运算符号(=、<、<=、>、>=、!=)
	 * @param string $logical 逻辑运算符(AND、OR)
	 * @return array
	 */
	private function where_placeholder(array $array, string $logical = 'AND'): array
	{
		$expression = $params = null;
		foreach ($array as $key => $value) {
			if (!is_string($key))
				continue;

			//key contains symbol??
			preg_match('/^.+?(=|<=?|>=?|!=)?$/i', $key, $matches);
			!isset($matches[1]) && $key .= '=';

			//placeholder
			$placeholder = 'placeholder' . self::$placeholder_index++;

			if (is_null($expression)) {
				$expression = $key . ':' . $placeholder;
			} else {
				$expression .= ' ' . $logical . ' ' . $key . ':' . $placeholder;
			}

			$params[$placeholder] = $value;
		}
		return [$expression, $params];
	}

	/**
	 * 生成 where in 的占位符表达式及参数
	 * <p>:placeholder1, :placeholder2, :placeholder3, ...</p>
	 * @param array $array 索引数组
	 * @return array
	 */
	private function where_in_placeholder(array $array): array
	{
		$expression = $params = null;
		foreach ($array as $value) {
			$placeholder = 'placeholder' . self::$placeholder_index++;
			$expression .= is_null($expression) ? ':' . $placeholder : ',:' . $placeholder;
			$params[$placeholder] = $value;
		}
		return [$expression, $params];
	}

	/**
	 * 生成 insert 的占位符表达式及参数
	 * <p>:placeholder1, :placeholder2, :placeholder3, ...</p>
	 * @param object|array $data 关联数组或对象
	 * @return array
	 */
	private function insert_placeholder(object|array $data): array
	{
		$field_name = $field_value = $params = null;
		foreach ($data as $key => $value) {
			if (!is_string($key) || !in_array($key, $this->allowed_fields))
				continue;

			//field_name
			$field_name .= is_null($field_name) ? $key : ',' . $key;

			//field_value
			$placeholder = 'placeholder' . self::$placeholder_index++;
			$field_value .= is_null($field_value) ? ':' . $placeholder : ',:' . $placeholder;

			//params
			$params[$placeholder] = $value;
		}
		return [$field_name, $field_value, $params];
	}

	/**
	 * 生成 update 的占位符表达式及参数
	 * <p>key=:placeholder1, key=:placeholder2, key=:placeholder3, ...</p>
	 * @param object|array $data 关联数组或对象
	 * @param array $safe_field 安全字段，该字段值不会添加转义
	 * @return array
	 */
	private function update_placeholder(object|array $data, array $safe_field): array
	{
		$expression = $params = null;
		foreach ($data as $key => $value) {
			if (!is_string($key) || !in_array($key, $this->allowed_fields))
				continue;

			if (in_array($key, $safe_field)) {
				$expression .= is_null($expression) ? $key . '=' . $value : ',' . $key . '=' . $value;
				continue;
			}

			$placeholder = 'placeholder' . self::$placeholder_index++;
			$expression .= is_null($expression) ? $key . '=:' . $placeholder : ',' . $key . '=:' . $placeholder;
			$params[$placeholder] = $value;
		}
		return [$expression, $params];
	}
}