<?php

namespace venus\cache\handler;

use venus\cache\depend\base_depend;

class redis extends base
{
	protected \config\cache\redis $config;
	protected \Redis $redis;

	public function __construct(\config\cache\redis $config)
	{
		$this->config = $config;
	}

	public function __destruct()
	{
		if (isset($this->redis)) {
			$this->redis->close();
		}
	}

	public function init(): void
	{
		$config = $this->config;
		$this->redis = new \Redis();

		if (!$this->redis->connect($config->host, $config->port, $config->timeout)) {
			throw new \Exception('Cache: Redis connection failed. Check your configuration.');
		}

		if (!is_null($config->password) && !$this->redis->auth($config->password)) {
			throw new \Exception('Cache: Redis authentication failed.');
		}

		if (!$this->redis->select($config->database)) {
			throw new \Exception('Cache: Redis select database failed.');
		}
	}

	public function get(string $key): mixed
	{
		$data = $this->redis->get($this->validate_key($key));

		if($data === false) { //key didn't exist
			return null;
		}

		$data = unserialize((string)$data);
		if (!is_array($data) ||
			!array_key_exists('value', $data) ||
			!array_key_exists('depend', $data)) {
			return null;
		}

		if ($data['depend'] instanceof base_depend && $data['depend']->is_changed($this)) {
			$this->delete($key);
			return null;
		}

		return $data['value'];
	}

	public function set(string $key, mixed $value, int $ttl = 0, ?base_depend $depend = null): bool
	{
		$key = $this->validate_key($key);

		if (!is_null($depend)) {
			$depend->evaluate_depend($this);
		}

		$data = serialize([
			'value' => $value,
			'depend' => $depend
		]);

		if (!$this->redis->set($key, $data)) {
			return false;
		}

		if ($ttl > 0) {
			$this->redis->expireAt($key, TIMESTAMP + $ttl);
		}

		return true;
	}

	public function delete(string $key): bool
	{
		$key = $this->validate_key($key);
		return $this->redis->del($key) === 1;
	}

	public function clean(): bool
	{
		return $this->redis->flushDB();
	}

	public function is_supported(): bool
	{
		return extension_loaded('redis');
	}
}