<?php

use venus\di\container;
use venus\filter\csrf;

if (!function_exists('helper')) {
	/**
	 * 载入助手函数
	 * @param array|string $filenames
	 */
	function helper(array|string $filenames): void
	{
		static $loaded = [];

		if (!is_array($filenames)) {
			$filenames = [$filenames];
		}

		$search_paths = [
			APP_PATH . 'helper',
			SYSTEM_PATH . 'helper'
		];

		// gets the helper directory of the current module
		$module = venus::$app->module->get_current_module();
		if (!is_null($module) && !empty($module['directory'])) {
			$helper_directory = MODULE_PATH . $module['directory'] . '/helper';
			is_dir($helper_directory) && array_unshift($search_paths, $helper_directory);
		}

		$includes = [];
		foreach ($filenames as $filename) {

			if (!str_contains($filename, '_helper')) {
				$filename .= '_helper';
			}

			foreach ($search_paths as $path) {
				$file = $path . DIRECTORY_SEPARATOR . $filename . '.php';
				if (in_array($file, $loaded, true)) {
					continue;
				}
				if (is_file($file)) {
					$includes[] = $file;
					$loaded[] = $file;
				}
			}

		}

		foreach ($includes as $file) {
			include_once $file;
		}
	}
}

if (!function_exists('create_object')) {
	/**
	 * 创建对象
	 * @param string $class
	 * @param array $params
	 * @param bool $get_shared
	 * @return object
	 * @throws Exception
	 */
	function create_object(string $class, array $params = [], bool $get_shared = true): object
	{
		return container::get_instance($class, $params, $get_shared);
	}
}

if (!function_exists('model')) {
	/**
	 * 创建模型
	 * @param string $class
	 * @param bool $get_shared
	 * @return object
	 * @throws Exception
	 */
	function model(string $class, bool $get_shared = true): object
	{
		return container::get_instance($class, [], $get_shared);
	}
}

if (!function_exists('service')) {
	/**
	 * 创建业务逻辑对象（服务）
	 * @param string $class
	 * @param array $params
	 * @param bool $get_shared
	 * @return object
	 * @throws Exception
	 */
	function service(string $class, array $params = [], bool $get_shared = true): object
	{
		return container::get_instance($class, $params, $get_shared);
	}
}

if (!function_exists('is_cli')) {
	function is_cli(): bool
	{
		if (in_array(PHP_SAPI, ['cli', 'phpdbg'], true)) {
			return true;
		}
		return !isset($_SERVER['REMOTE_ADDR']) && !isset($_SERVER['REQUEST_METHOD']);
	}
}

if (!function_exists('csrf_token_name')) {
	function csrf_token_name(): string
	{
		/**
		 * @var csrf $csrf
		 */
		$csrf = venus::$app->filter->get_filter(csrf::class);
		return !is_null($csrf) ? $csrf->get_token_name() : '';
	}
}

if (!function_exists('csrf_header_name')) {
	function csrf_header_name(): string
	{
		/**
		 * @var csrf $csrf
		 */
		$csrf = venus::$app->filter->get_filter(csrf::class);
		return !is_null($csrf) ? $csrf->get_header_name() : '';
	}
}

if (!function_exists('csrf_token_hash')) {
	function csrf_token_hash(): string
	{
		/**
		 * @var csrf $csrf
		 */
		$csrf = venus::$app->filter->get_filter(csrf::class);
		return !is_null($csrf) ? $csrf->get_token_hash() : '';
	}
}

if (!function_exists('csrf_field')) {
	function csrf_field(): string
	{
		/**
		 * @var csrf $csrf
		 */
		$csrf = venus::$app->filter->get_filter(csrf::class);
		return !is_null($csrf) ? '<input type="hidden" name="' . $csrf->get_token_name() . '" value="' . $csrf->get_token_hash() . '">' : '';
	}
}

if (!function_exists('csrf_json')) {
	function csrf_json(bool $header_json = true): string
	{
		/**
		 * @var csrf $csrf
		 */
		$csrf = venus::$app->filter->get_filter(csrf::class);
		return !is_null($csrf) ? '{' . ($header_json ? '"' . $csrf->get_header_name() . '"' : $csrf->get_token_name()) . ':"' . $csrf->get_token_hash() . '"}' : '{}';
	}
}

if (!function_exists('formatter')) {
	/**
	 * 格式化数据
	 * @param mixed $data 格式化数据
	 * @param string $name 格式化器名称 (json/xml)
	 * @return false|string
	 * @throws Exception
	 */
	function formatter(mixed $data, string $name = 'json'): false|string
	{
		return container::get(\venus\format\format::class)->get_formatter($name)->format($data);
	}
}

if (!function_exists('session')) {
	/**
	 * 获取session实例
	 * @throws Exception
	 */
	function session(): \venus\session\session
	{
		/**
		 * @var venus\session\session $session
		 */
		$session = container::get(\venus\session\session::class);
		return $session;
	}
}

if (!function_exists('lang')) {
	function lang(string $key, array $args = []): string
	{
		return venus::$app->language->get($key, $args);
	}
}

if (!function_exists('autoloader')) {
	function autoloader(): ?object
	{
		return container::get_exists_instance(\venus\autoloader\autoloader::class);
	}
}

if (!function_exists('view')) {
	/**
	 * 返回视图
	 * @param string $view 视图文件名。如：home/index
	 * @param array $data 视图数据
	 * @return string
	 * @throws Exception
	 */
	function view(string $view, array $data = []): string
	{
		return venus::$app->view->set_data($data)->render($view);
	}
}

if (!function_exists('event')) {
	function event(): \venus\event\event
	{
		return venus::$app->event;
	}
}

if (!function_exists('old')) {
	/**
	 * 从session中获取请求提交的数据
	 * @param string $key
	 * @param $default
	 * @return mixed
	 */
	function old(string $key, $default = null): mixed
	{
		if (!isset($_SESSION['_venus_old_input']) || !is_array($_SESSION['_venus_old_input'])) {
			return $default;
		}

		if (isset($_SESSION['_venus_old_input']['post'][$key])) {
			return $_SESSION['_venus_old_input']['post'][$key];
		}

		if (isset($_SESSION['_venus_old_input']['get'][$key])) {
			return $_SESSION['_venus_old_input']['get'][$key];
		}

		return $default;
	}
}

if (!function_exists('cache')) {
	/**
	 * 获取缓存处理器
	 * @return \venus\cache\handler
	 * @throws Exception
	 */
	function cache(): \venus\cache\handler
	{
		return phpvod::$app->cache->get_handler();
	}
}

if (!function_exists('curl_request')) {
	/**
	 * 获取cURL请求实例
	 */
	function curl_request(): \venus\http\curl_request
	{
		return phpvod::$app->curl_request;
	}
}