<?php

namespace venus\database;

use PDO;
use venus\base\component;
use venus\event\event;

class database extends component
{
	public const EVENT_DATABASE_EXEC = 'event_database_exec';
	public const EVENT_DATABASE_BEFORE_QUERY = 'event_database_before_query';
	public const EVENT_DATABASE_AFTER_QUERY = 'event_database_after_query';
	public const EVENT_DATABASE_UPDATE = 'event_database_update';

	/**
	 * @var PDO pdo对象
	 */
	public PDO $pdo;

	private event $event;

	/**
	 * @var string 表前缀
	 */
	private string $prefix;

	/**
	 * @var int SQL执行数量
	 */
	private int $count = 0;

	/**
	 * database constructor.
	 * @param \config\database $config
	 * @see https://www.php.net/manual/en/ref.pdo-mysql.connection.php
	 */
	public function __construct(\config\database $config)
	{
		$this->prefix = $config->prefix;
		$options = [
			PDO::ATTR_PERSISTENT => $config->persistent,
			PDO::ATTR_ERRMODE => PDO::ERRMODE_EXCEPTION
		];
		$this->pdo = new PDO($config->dsn, $config->username, $config->password, $options);
		if (!empty($config->charset)) {
			$this->pdo->exec("SET NAMES " . $config->charset);
		}
		$this->event = \venus::$app->event;
	}

	/**
	 * SQL预处理
	 * @param string $query
	 * @param array $options
	 * @return statement
	 */
	public function prepare(string $query, array $options = []): statement
	{
		$pdo_statement = $this->pdo->prepare($query, $options);
		return new statement($pdo_statement);
	}

	/**
	 * 执行SQL语句，并返回受影响的行数
	 * @param string $statement
	 * @return int|false
	 */
	public function exec(string $statement): int|false
	{
		$affected_rows = $this->pdo->exec($statement);
		$affected_rows !== false && $this->count++;
		//event
		$this->event->trigger(self::EVENT_DATABASE_EXEC, ['query' => $statement, 'affected_rows' => $affected_rows]);
		return $affected_rows;
	}

	/**
	 * 查询
	 * @param string $query
	 * @param array|null $params
	 * @return statement
	 */
	public function query(string $query, ?array $params = null): statement
	{
		$this->prefix != 'pv_' && $query = str_replace([' pv_', '`pv_', " 'pv_"], [" $this->prefix", "`$this->prefix", " '$this->prefix"], $query);

		$args = ['query' => $query, 'params' => $params];
		$this->event->trigger(self::EVENT_DATABASE_BEFORE_QUERY, $args);
		$statement = $this->prepare($query);
		$statement->execute($params) && $this->count++;
		$this->event->trigger(self::EVENT_DATABASE_AFTER_QUERY, $args);

		return $statement;
	}

	/**
	 * 更新
	 * @param string $query
	 * @param array|null $params
	 * @return int 返回执行DELETE/INSERT或UPDATE语句受影响的行数
	 */
	public function update(string $query, ?array $params = null): int
	{
		$statement = $this->query($query, $params);
		$value = $statement->affected_rows();
		$statement->close_cursor();
		//event
		$this->event->trigger(self::EVENT_DATABASE_UPDATE, ['query' => $query, 'params' => $params, 'affected_rows' => $value]);
		return $value;
	}

	/**
	 * 返回最后插入行的ID或序列值
	 * @param string|null $name
	 * @return string
	 */
	public function insert_id(?string $name = null): string
	{
		return $this->pdo->lastInsertId($name);
	}

	/**
	 * 启动事务
	 * @return bool
	 */
	public function begin_transaction(): bool
	{
		return $this->pdo->beginTransaction();
	}

	/**
	 * 提交事务
	 * @return bool
	 */
	public function commit(): bool
	{
		return $this->pdo->commit();
	}

	/**
	 * 回滚事务
	 * @return bool
	 */
	public function roll_back(): bool
	{
		return $this->pdo->rollBack();
	}

	/**
	 * 检查是否在事务内
	 * @return bool
	 */
	public function in_transaction(): bool
	{
		return $this->pdo->inTransaction();
	}

	/**
	 * 获取SQL执行数量
	 * @return int
	 */
	public function get_count(): int
	{
		return $this->count;
	}

	/**
	 * 获取单条记录中指定列的值
	 * @param string $query
	 * @param array|null $params
	 * @param int $index
	 * @return mixed
	 */
	public function get_value(string $query, ?array $params = null, int $index = 0)
	{
		$statement = $this->query($query, $params);
		$value = $statement->fetch_column($index);
		$statement->close_cursor();
		return $value;
	}

	/**
	 * 获取单条记录并以数组返回
	 * @param string $query
	 * @param array|null $params
	 * @param int $mode
	 * @return array|false
	 */
	public function get_array(string $query, ?array $params = null, int $mode = PDO::FETCH_ASSOC)
	{
		$statement = $this->query($query, $params);
		$value = $statement->fetch_array($mode);
		$statement->close_cursor();
		return $value;
	}

	/**
	 * 获取单条记录并以对象返回
	 * @param string $query
	 * @param array|null $params
	 * @param string $class
	 * @param array $ctor_args
	 * @return false|object
	 */
	public function get_object(string $query, ?array $params = null, string $class = 'stdClass', array $ctor_args = [])
	{
		$statement = $this->query($query, $params);
		$value = $statement->fetch_object($class, $ctor_args);
		$statement->close_cursor();
		return $value;
	}

	/**
	 * 导入sql
	 * @param string $sql sql语句
	 * @param string $delimiter sql语句定界符
	 * @param bool $start_transaction 是否开启事务
	 * @return bool 成功返回true，出错抛出异常
	 * @throws \PDOException
	 */
	public function import(string $sql, string $delimiter = ';', bool $start_transaction = false): bool
	{
		if (empty($sql)) {
			return true;
		}

		$sql = preg_replace('/^\s*(--\s+|#).*?$/m', '', $sql); //删除单行注释
		$sql = preg_replace('/\/\*.*?\*\//ms', '', $sql); //删除多行注释

		$start_transaction && $this->begin_transaction();

		try {
			$sql_list = explode($delimiter, $sql);
			foreach ($sql_list as $sql_item) {
				$sql_item = trim($sql_item);
				if (empty($sql_item)) {
					continue;
				}
				$this->exec($sql_item);
			}
		} catch (\PDOException $pdo_exception) {
			$start_transaction && $this->roll_back();
			throw $pdo_exception;
		}

		if($start_transaction && !$this->commit()) {
			throw new \PDOException('transaction commit failed');
		}

		return true;
	}

	/**
	 * 导入文件
	 * @param string $sql_file sql文件
	 * @param string $delimiter sql语句定界符
	 * @param bool $start_transaction 是否开启事务
	 * @return bool 成功返回true，失败返回false
	 * @throws \PDOException
	 */
	public function import_file(string $sql_file, string $delimiter = ';', bool $start_transaction = false): bool
	{
		$sql = @file_get_contents($sql_file);
		return $sql !== false && $this->import($sql, $delimiter, $start_transaction);
	}

	/**
	 * 获取服务端版本
	 * @return string|null 成功返回版本号，失败返回null
	 */
	public function get_server_version(): ?string
	{
		try {
			return $this->pdo->getAttribute(PDO::ATTR_SERVER_VERSION);
		} catch (\Exception) {
			return null;
		}
	}
}
