<?php

namespace venus\database;

use PDO;
use PDOStatement;

class statement
{
	public PDOStatement $pdo_statement;

	/**
	 * statement constructor.
	 * @param PDOStatement $pdo_statement
	 */
	public function __construct(PDOStatement $pdo_statement)
	{
		$this->pdo_statement = $pdo_statement;
	}

	public function execute(?array $params = null): bool
	{
		return $this->pdo_statement->execute($params);
	}

	/**
	 * 从结果集中获取下一行并以数组返回
	 * @param int $mode
	 * @return array|false
	 */
	public function fetch_array(int $mode = PDO::FETCH_ASSOC)
	{
		return $this->pdo_statement->fetch($mode);
	}

	/**
	 * 获取结果集中的数据并以数组返回
	 * @param int $mode
	 * @return array
	 */
	public function fetch_all_array(int $mode = PDO::FETCH_ASSOC): array
	{
		return $this->pdo_statement->fetchAll($mode);
	}

	/**
	 * 调用回调函数获取结果集中的数据并以数组返回
	 * @param callable $callback 回调函数
	 * <p>回调格式：fun(array $row)</p>
	 * <p>回调参数: $row - 当前数据行数组。
	 * <p>回调返回:
	 * <ul>
	 * <li>如果返回null，则表示忽略该行，即不将该行添加至结果数组</li>
	 * <li>如果返回数组且只有2个元素，键分别为key与value，则key为结果数组的键，value为值。</li>
	 * <li>其它返回类型添加至结果索引数组</li>
	 * @return array
	 */
	public function fetch_all_by_callback(callable $callback): array
	{
		$array = [];
		while (is_array($row = $this->fetch_array())) {
			$r = $callback($row);
			if (!is_null($r)) {
				if (is_array($r) && count($r) == 2 && array_key_exists('key', $r) && array_key_exists('value', $r)) {
					$array[$r['key']] = $r['value'];
				} else {
					$array[] = $r;
				}
			}
		}
		return $array;
	}

	/**
	 * 从结果集中获取一列并以数组返回
	 * @param int $column_index 列索引
	 * @return array
	 */
	public function fetch_column_array(int $column_index = 0): array
	{
		return $this->pdo_statement->fetchAll(PDO::FETCH_COLUMN, $column_index);
	}

	/**
	 * 从结果集中获取下一行并作为一个对象返回
	 * @param string $class
	 * @param array $ctor_args
	 * @return object|false
	 */
	public function fetch_object(string $class = 'stdClass', array $ctor_args = [])
	{
		return $this->pdo_statement->fetchObject($class, $ctor_args);
	}

	/**
	 * 获取结果集中的数据并以对象数组返回
	 * @param string $class
	 * @param array $ctor_args
	 * @return array
	 */
	public function fetch_all_object(string $class = 'stdClass', array $ctor_args = []): array
	{
		return $this->pdo_statement->fetchAll(PDO::FETCH_CLASS, $class, $ctor_args);
	}

	/**
	 * 从结果集中的下一行返回单独的一列。
	 * @param int $column 列的索引数字
	 * @return mixed 返回结果集的下一行中的单个列，如果没有更多行，则返回FALSE。
	 */
	public function fetch_column(int $column = 0)
	{
		return $this->pdo_statement->fetchColumn($column);
	}

	/**
	 * 返回结果集中的列数
	 * @return int
	 */
	public function column_count(): int
	{
		return $this->pdo_statement->columnCount();
	}

	/**
	 * 返回结果集中指定列的列名
	 * @param int $column
	 * @return mixed|null
	 */
	public function column_name(int $column)
	{
		$meta = $this->pdo_statement->getColumnMeta($column);
		return $meta['name'] ?? null;
	}

	/**
	 * 执行DELETE/INSERT或UPDATE语句受影响的行数
	 * @return int
	 */
	public function affected_rows(): int
	{
		return $this->pdo_statement->rowCount();
	}

	/**
	 * 关闭结果集游标
	 */
	function close_cursor()
	{
		$this->pdo_statement->closeCursor();
	}
}