<?php

namespace venus\di;

use Exception;
use ReflectionClass;

class container
{
	public static array $instances = [];

	/**
	 * 获取实例，如不存在则创建该实例
	 * @param string $class 类名
	 * @param array $params 构造函数参数
	 * @param boolean $get_shared 是否返回已存在的实例
	 * @return object
	 * @throws Exception
	 */
	public static function get_instance(string $class, array $params = [], bool $get_shared = true): object
	{
		if (self::has($class) && $get_shared) {
			return self::$instances[$class];
		}

		$obj = (new ReflectionClass($class))->newInstanceArgs($params);

		if (is_null($obj)) {
			throw new Exception('Object instantiation failed');
		}

		!self::has($class) && self::$instances[$class] = $obj;
		return $obj;
	}

	/**
	 * 获取实例，如不存在则返回null
	 * @param string $class 类名
	 * @return object|null
	 */
	public static function get_exists_instance(string $class): ?object
	{
		return self::has($class) ? self::$instances[$class] : null;
	}

	/**
	 * 获取实例，如不存在则创建该实例(依赖注入)
	 * @param string $class 类名
	 * @param boolean $get_shared 是否返回已存在的实例
	 * @return object
	 * @throws Exception
	 */
	public static function get(string $class, bool $get_shared = true): object
	{
		if (self::has($class) && $get_shared) {
			return self::$instances[$class];
		}

		$clazz = new ReflectionClass($class);
		$args = [];

		$constructor = $clazz->getConstructor();
		if ($constructor) {
			$constructor_params = $constructor->getParameters();

			foreach ($constructor_params as $param) {

				$param_name = $param->getName();
				$param_type = $param->getType();

				if (is_null($param_type)) {
					//Parameter type is not class
					if ($param->isDefaultValueAvailable()) {
						$args[] = $param->getDefaultValue();
						continue;
					}
					throw new Exception($class . '::__construct() No default value was set for parameter ' . $param_name);
				} else {
					//Parameter type is class
					$args[] = self::get($param_type->getName(), $get_shared);
				}

			}
		}

		$obj = $clazz->newInstanceArgs($args);
		if (is_null($obj)) {
			throw new Exception('Object instantiation failed');
		}

		!self::has($class) && self::$instances[$class] = $obj;
		return $obj;
	}

	/**
	 * 检测容器是否包含该类实例
	 * @param string $class 类名
	 * @return bool
	 */
	public static function has(string $class): bool
	{
		return isset(self::$instances[$class]);
	}

	/**
	 * 向容器中添加实例
	 * @param string $class
	 * @param object $instance
	 * @return void
	 */
	public static function add(string $class, object $instance): void {
		self::$instances[$class] = $instance;
	}

	/**
	 * 从容器中移除该类实例
	 * @param string $class 类名
	 * @return void
	 */
	public static function remove(string $class): void
	{
		unset(self::$instances[$class]);
	}

	/**
	 * 清空容器
	 * @return void
	 */
	public static function clear(): void
	{
		self::$instances = [];
	}
}