<?php

namespace venus\filter;

use Exception;
use venus;

class csrf implements before_interface
{
	private array $options;

	private const CSRF_HASH_BYTES = 16;

	private ?string $token_hash;

	private string $token_name = 'venus_csrf_token';

	private string $header_name = 'X-VENUS-CSRF-TOKEN';

	private string $cookie_name = 'venus_csrf_token';

	protected int $cookie_expires = 60 * 60 * 24;

	/**
	 * @throws Exception
	 */
	public function __construct(array $options = [])
	{
		$this->options = $options;

		//init token hash
		$this->token_hash = $_COOKIE[$this->cookie_name] ?? null;
		if (is_null($this->token_hash)) {
			$this->generate_hash(); // Generate the hash and set the cookie
		}
	}

	/**
	 * @throws Exception
	 */
	public function before(): void
	{
		if (!$this->except_uri() && !$this->verify()) {
			throw new Exception("Deny Access (CSRF)");
		}
	}

	/**
	 * 是否排除当前 uri
	 * @return bool
	 */
	private function except_uri(): bool
	{
		if (isset($this->options['except'])) {
			$uri = venus::$app->request->server('REQUEST_URI');
			if (is_null($uri)) {
				return false;
			}

			$except = $this->options['except'];
			is_string($except) && $except = [$except];

			foreach ($except as $pattern) {
				if (preg_match('~^' . $pattern . '$~i', $uri) === 1) {
					return true;
				}
			}
		}
		return false;
	}

	/**
	 * verify token
	 */
	private function verify(): bool
	{
		$request = venus::$app->request;

		$method = $request->get_method();
		if (!in_array($method, ['post', 'put', 'delete', 'patch'])) {
			return true;
		}

		$posted_token = $this->get_posted_token();

		// verify
		if (!isset($posted_token, $this->token_hash) || !hash_equals($posted_token, $this->token_hash)) {
			return false;
		}

		$this->remove_token_in_request();
		return true;
	}

	/**
	 * @throws Exception
	 */
	private function generate_hash(): void
	{
		$this->token_hash = bin2hex(random_bytes(self::CSRF_HASH_BYTES));
		venus::$app->response->set_cookie($this->cookie_name, $this->token_hash, ['prefix' => '', 'expires' => TIMESTAMP + $this->cookie_expires, 'httponly' => true]);
	}

	private function get_posted_token(): ?string
	{
		$result = null;
		$request = venus::$app->request;

		if (!is_null($request->post($this->token_name))) {
			$result = $request->post($this->token_name);
		} elseif ($request->has_header($this->header_name)) {
			$result = $request->get_header($this->header_name);
		}

		return $result;
	}

	private function remove_token_in_request(): void
	{
		$request = venus::$app->request;

		if (isset($_POST[$this->token_name])) {
			unset($_POST[$this->token_name]);
		}

		if ($request->has_header($this->header_name)) {
			$request->remove_header($this->header_name); // Remove only from the request component
		}
	}

	/**
	 * @return string
	 */
	public function get_token_hash(): string
	{
		return $this->token_hash;
	}

	/**
	 * @return string
	 */
	public function get_token_name(): string
	{
		return $this->token_name;
	}

	/**
	 * @return string
	 */
	public function get_header_name(): string
	{
		return $this->header_name;
	}
}