<?php
if (!function_exists('write_file')) {
	/**
	 * 写文件
	 * 将数据写入路径中指定的文件，如果不存在，创建一个新文件。
	 * @param string $path 文件路径
	 * @param string $data 数据
	 * @param string $mode fopen() mode (default: 'wb')
	 * @return bool
	 */
	function write_file(string $path, string $data, string $mode = 'wb'): bool
	{
		try {
			$fp = fopen($path, $mode);

			flock($fp, LOCK_EX);

			for ($result = $written = 0, $length = strlen($data); $written < $length; $written += $result) {
				if (($result = fwrite($fp, substr($data, $written))) === false) {
					break;
				}
			}

			flock($fp, LOCK_UN);
			fclose($fp);

			return is_int($result);
		} catch (Exception) {
			return false;
		}
	}
}

if (!function_exists('create_directory')) {
	/**
	 * 创建目录
	 * 此方法类似于PHP的 mkdir() 函数，但不同之处在于它使用 chmod() 设置创建目录的权限以避免 umask 设置的影响。
	 * @param string $path 待创建目录的路径
	 * @param int $mode 创建的目录要设置的权限
	 * @param bool $recursive 如果父目录不存在，是否创建父目录
	 * @return bool
	 * @throws Exception
	 */
	function create_directory(string $path, int $mode = 0775, bool $recursive = true): bool
	{
		if (is_dir($path)) {
			return true;
		}
		$parent_dir = dirname($path);
		// 父目录不存在则递归创建目录
		if ($recursive && !is_dir($parent_dir) && $parent_dir !== $path) {
			create_directory($parent_dir, $mode);
		}
		try {
			if (!mkdir($path, $mode)) {
				return false;
			}
		} catch (Exception $e) {
			if (!is_dir($path)) {
				throw new Exception("Failed to create directory \"$path\": " . $e->getMessage(), $e->getCode(), $e);
			}
		}
		try {
			return chmod($path, $mode);
		} catch (Exception $e) {
			throw new Exception("Failed to change permissions for directory \"$path\": " . $e->getMessage(), $e->getCode(), $e);
		}
	}
}

if (!function_exists('copy_directory')) {
	/**
	 * 复制目录（文件和子目录也将被复制）
	 * @param string $src 源目录
	 * @param string $dst 目标目录
	 * @param array $options 目录复制选项。有效的选项有:
	 * - dir_mode: 整型，为新复制的目录设置权限。默认为 0775
	 * - file_mode: 整形，为新复制的文件设置权限。默认为当前环境缺省设置
	 * @param array|null $info 用于保存执行结果
	 * @throws Exception
	 */
	function copy_directory(string $src, string $dst, array $options = [], array &$info = null): void
	{
		if (is_null($info)) {
			$info = ['success_count' => 0, 'failure_count' => 0];
		} else {
			!array_key_exists('success_count', $info) && $info['success_count'] = 0;
			!array_key_exists('failure_count', $info) && $info['failure_count'] = 0;
		}

		if ($src === $dst || str_starts_with($dst, $src . DIRECTORY_SEPARATOR)) {
			throw new Exception('Trying to copy a directory to itself or a subdirectory.');
		}
		if (!is_dir($dst)) {
			create_directory($dst, $options['dir_mode'] ?? 0775) ? $info['success_count']++ : $info['failure_count']++;
		}
		$handle = opendir($src);
		if ($handle === false) {
			throw new Exception("Unable to open directory: $src");
		}
		while (($file = readdir($handle)) !== false) {
			if ($file === '.' || $file === '..') {
				continue;
			}
			$from = $src . DIRECTORY_SEPARATOR . $file;
			$to = $dst . DIRECTORY_SEPARATOR . $file;
			if (is_file($from)) {
				copy($from, $to) ? $info['success_count']++ : $info['failure_count']++;
				if (isset($options['file_mode'])) {
					@chmod($to, $options['file_mode']);
				}
			} else {
				// recursive copy
				copy_directory($from, $to, $options, $info);
			}
		}
		closedir($handle);
	}
}

if (!function_exists('remove_directory')) {
	/**
	 * 删除目录(及其所有内容)。
	 * @param string $dir 目录路径
	 * @return bool
	 */
	function remove_directory(string $dir): bool
	{
		if (!is_dir($dir)) {
			return false;
		}
		if (!is_link($dir)) {
			if (!($handle = opendir($dir))) {
				return false;
			}

			while (($file = readdir($handle)) !== false) {
				if ($file === '.' || $file === '..') {
					continue;
				}
				$path = $dir . DIRECTORY_SEPARATOR . $file;
				if (is_dir($path)) {
					remove_directory($path);
				} else {
					remove_file($path);
				}
			}
			closedir($handle);
		}

		if (is_link($dir)) {
			return remove_file($dir);
		} else {
			return rmdir($dir);
		}
	}
}

if (!function_exists('remove_file')) {
	/**
	 * 删除文件或符号链接
	 * @param string $path 文件路径
	 * @return bool
	 */
	function remove_file(string $path): bool
	{
		$is_windows = DIRECTORY_SEPARATOR === '\\';
		if (!$is_windows) {
			return unlink($path);
		}
		if (is_link($path) && is_dir($path)) {
			return rmdir($path);
		}
		try {
			return unlink($path);
		} catch (Exception) {
			// last resort measure for Windows
			if (function_exists('exec') && file_exists($path)) {
				exec('DEL /F/Q ' . escapeshellarg($path));
				return !file_exists($path);
			}
			return false;
		}
	}
}

if (!function_exists('unzip')) {
	/**
	 * 解压zip文件至指定的目录
	 * @param string $zip_file 压缩文件路径
	 * @param string|false $extract_to 解压目录，如果为false（缺省），则解压至压缩文件所在的目录
	 * @param bool $create_same_name_dir 是(true)否(false)创建与zip文件同名的目录，并解压至该目录（此选项只有当$extract_to为false才有效），默认值true
	 * @return bool
	 */
	function unzip(string $zip_file, string|false $extract_to = false, bool $create_same_name_dir = true): bool
	{
		$result = false;
		$zip = new ZipArchive();
		$res = $zip->open($zip_file, ZipArchive::RDONLY);
		if ($res === true) {
			if ($extract_to === false) {
				$path_parts = pathinfo($zip_file);
				$extract_to = $create_same_name_dir ? $path_parts['dirname'] . DIRECTORY_SEPARATOR . $path_parts['filename'] : $path_parts['dirname'];
			}
			$zip->extractTo($extract_to) && $result = true;
			$zip->close();
		}
		return $result;
	}
}