<?php

namespace venus\http;

use config\curl_request\config;
use venus\base\component;

class curl_request extends component
{
	public const METHOD_HEAD = 'HEAD';
	public const METHOD_GET = 'GET';
	public const METHOD_POST = 'POST';
	public const ENCTYPE_FORM_PARAMS = 'application/x-www-form-urlencoded';
	public const ENCTYPE_MULTIPART = 'multipart/form-data';
	private config $config;
	protected array $header = [];

	public function __construct(config $config)
	{
		if (!function_exists('curl_version')) {
			throw new \Exception('CURL must be enabled to use the curl_request class');
		}
		$this->config = $config;
	}

	/**
	 * 设置header
	 * @param array $header
	 * @return curl_request
	 */
	public function set_header(array $header): curl_request
	{
		$this->header = $header;
		return $this;
	}

	/**
	 * 添加header
	 * @param string $key
	 * @param string $value
	 * @return $this
	 */
	public function append_header(string $key, string $value): curl_request
	{
		$this->header[$key] = $value;
		return $this;
	}

	/**
	 * 清除header
	 * @return void
	 */
	public function clear_header(): void
	{
		$this->header = [];
	}

	/**
	 * request
	 * @param string $method
	 * @param string $url
	 * @param array $config
	 * <pre>
	 * string cert 证书路径，如果您要连接到要求您使用证书进行身份验证的服务器，可以通过该参数指定证书路径
	 * string cert_passwd 证书密码，可选
	 * bool|string verify
	 *      true:启用证书验证并使用系统提供的CA包
	 *      false:禁用证书认证
	 *      string:自定义CA包路径并启用证书验证 (https://curl.se/docs/caextract.html)
	 * string proxy HTTP代理通道
	 * bool|array allow_redirects 是否允许重定向
	 * int timeout 允许执行的最长毫秒数
	 * int connect_timeout 尝试连接等待的时间，以毫秒为单位，设置为0，则无限等待。
	 * array form_params 表单提交参数 (application/x-www-form-urlencoded)
	 * array multipart 表单提交参数 (multipart/form-data)
	 * string cookie cookie文件路径
	 * bool fail_on_error 当 HTTP 状态码大于等于 400，true 将触发异常。false(默认)将返回页面。
	 * string version 强制cURL使用的http版本，可选值：1.0/1.1/2.0，缺省时时让cURL自己判断使用哪个版本
	 * </pre>
	 * @return curl_response
	 * @throws \Exception
	 */
	public function request(string $method, string $url, array $config = []): curl_response
	{
		$config = array_merge($this->config->config, $config);
		$config['method'] = $method;
		$config['url'] = $url;

		$options = $this->parse_options($config);
		$response = $this->send_request($options);
		$this->clear_header();
		return new curl_response($response['header'], $response['body'], $response['info']);
	}

	/**
	 * head
	 * @param string $url
	 * @param array $config
	 * @return curl_response
	 * @throws \Exception
	 */
	public function head(string $url, array $config = []): curl_response
	{
		return $this->request(self::METHOD_HEAD, $url, $config);
	}

	/**
	 * get
	 * @param string $url
	 * @param array $config
	 * @return curl_response
	 * @throws \Exception
	 */
	public function get(string $url, array $config = []): curl_response
	{
		return $this->request(self::METHOD_GET, $url, $config);
	}

	/**
	 * post
	 * @param string $url
	 * @param array $fields
	 * @param string $enctype
	 * @param array $config
	 * @return curl_response
	 * @throws \Exception
	 */
	public function post(string $url, array $fields = [], string $enctype = self::ENCTYPE_FORM_PARAMS, array $config = []): curl_response
	{
		if (!empty($fields)) {
			if ($enctype === self::ENCTYPE_FORM_PARAMS) {
				$config['form_params'] = $fields;
			} elseif ($enctype === self::ENCTYPE_MULTIPART) {
				$config['multipart'] = $fields;
			}
		}
		return $this->request(self::METHOD_POST, $url, $config);
	}

	protected function parse_options(array $config): array
	{
		$options = [];
		$options[CURLOPT_URL] = $config['url'];
		$options[CURLOPT_CUSTOMREQUEST] = $config['method'];
		$options[CURLOPT_RETURNTRANSFER] = true;
		$options[CURLOPT_HEADER] = true;
		$options[CURLOPT_FRESH_CONNECT] = true;

		// head method
		if ($config['method'] == self::METHOD_HEAD) {
			$options[CURLOPT_NOBODY] = 1;
		}

		//header
		$http_header = [];
		$header = array_merge($this->config->header, $this->header);
		foreach ($header as $key => $value) {
			$http_header[] = $key . ':' . $value;
		}
		$options[CURLOPT_HTTPHEADER] = $http_header;

		//cert
		if (isset($config['cert'])) {
			if (!is_file($config['cert'])) {
				throw new \Exception('SSL certificate not found at: "' . $config['cert'] . '"');
			}
			$options[CURLOPT_SSLCERT] = $config['cert'];
		}

		//cert_passwd
		if (isset($config['cert_passwd'])) {
			$options[CURLOPT_SSLCERTPASSWD] = $config['cert_passwd'];
		}

		//verify
		if (isset($config['verify'])) {
			if (is_string($config['verify'])) {
				$file = realpath($config['verify']) ?: $config['verify'];
				if (!is_file($file)) {
					throw new \Exception('Cannot set SSL Key. "' . $file . '" is not a valid file.');
				}
				$options[CURLOPT_CAINFO] = $file;
				$options[CURLOPT_SSL_VERIFYPEER] = true;
				$options[CURLOPT_SSL_VERIFYHOST] = 2;
			} elseif (is_bool($config['verify'])) {
				$options[CURLOPT_SSL_VERIFYPEER] = $config['verify'];
				$options[CURLOPT_SSL_VERIFYHOST] = $config['verify'] ? 2 : 0;
			}
		}

		//proxy
		if (isset($config['proxy'])) {
			$options[CURLOPT_HTTPPROXYTUNNEL] = true;
			$options[CURLOPT_PROXY] = $config['proxy'];
		}

		//allow_redirects
		if (isset($config['allow_redirects'])) {
			//default settings
			$settings = [
				'maxredirs' => 10,
				'postredir' => 1 | 2 | 4,
				'redir_protocols' => ['http', 'https']
			];
			if (is_array($config['allow_redirects'])) {
				$settings = array_merge($settings, $config['allow_redirects']);
			}
			if ($config['allow_redirects'] === false) {
				$options[CURLOPT_FOLLOWLOCATION] = false;
			} else {
				$options[CURLOPT_FOLLOWLOCATION] = true;
				$options[CURLOPT_MAXREDIRS] = $settings['maxredirs'];
				$options[CURLOPT_POSTREDIR] = $settings['postredir'];
				//redir_protocols
				$redir_protocols = 0;
				foreach ($settings['redir_protocols'] as $protocols) {
					$redir_protocols += constant('CURLPROTO_' . strtoupper($protocols));
				}
				$options[CURLOPT_REDIR_PROTOCOLS] = $redir_protocols;
			}
		}

		//timeout
		isset($config['timeout']) && $options[CURLOPT_TIMEOUT_MS] = $config['timeout'];
		//connect_timeout
		isset($config['connect_timeout']) && $options[CURLOPT_CONNECTTIMEOUT_MS] = $config['connect_timeout'];

		if ($config['method'] === self::METHOD_POST) {
			// form_params (application/x-www-form-urlencoded)
			if (array_key_exists('form_params', $config) && is_array($config['form_params'])) {
				$post_fields = http_build_query($config['form_params']);
				$options[CURLOPT_POSTFIELDS] = $post_fields;
				$this->append_header('Content-Length', (string)strlen($post_fields));
				$this->append_header('Content-Type', 'application/x-www-form-urlencoded');
			}
			// multipart (multipart/form-data)
			if (array_key_exists('multipart', $config) && is_array($config['multipart'])) {
				$options[CURLOPT_POSTFIELDS] = $config['multipart'];
			}
		}

		//fail_on_error
		if (isset($config['fail_on_error'])) {
			$options[CURLOPT_FAILONERROR] = (bool)$config['fail_on_error'];
		}

		// version
		if (isset($config['version'])) {
			$options[CURLOPT_HTTP_VERSION] = match ($config['version']) {
				'1.0' => CURL_HTTP_VERSION_1_0,
				'1.1' => CURL_HTTP_VERSION_1_1,
				'2.0' => CURL_HTTP_VERSION_2_0,
				default => CURL_HTTP_VERSION_NONE
			};
		}

		//cookie
		if (isset($config['cookie'])) {
			$options[CURLOPT_COOKIEJAR] = $config['cookie'];
			$options[CURLOPT_COOKIEFILE] = $config['cookie'];
		}

		return $options;
	}

	protected function send_request(array $options): array
	{
		$ch = curl_init();
		curl_setopt_array($ch, $options);
		$response = curl_exec($ch);

		if ($response === false) {
			throw new \Exception(curl_errno($ch) . ':' . curl_error($ch));
		}

		$info = curl_getinfo($ch);
		if ($info === false) {
			throw new \Exception('Failed to obtain transfer information');
		}

		$header = substr($response, 0, $info['header_size']);
		$body = substr($response, $info['header_size']);
		curl_close($ch);

		return ['header' => $header, 'body' => $body, 'info' => $info];
	}
}