<?php

namespace venus\http\file;

class uploaded_file
{
	private const UPLOAD_PATH = WRITABLE_PATH . 'upload';

	private string $name;
	private string $type;
	private string $tmp_name;
	private float $size;
	private int $error;

	private bool $has_moved = false;
	private ?string $saved_file = null;


	/**
	 * 构造函数
	 * @param string $name
	 * @param string $type
	 * @param string $tmp_name
	 * @param float $size
	 * @param int $error
	 */
	public function __construct(string $name, string $type, string $tmp_name, float $size, int $error)
	{
		$this->name = $name;
		$this->type = $type;
		$this->tmp_name = $tmp_name;
		$this->size = $size;
		$this->error = $error;
	}

	/**
	 * 移动文件
	 * @param string $target_path 目标目录
	 * @param string|null $name 目标文件名，如为null，则使用上传文件名
	 * @throws \Exception
	 */
	public function move(string $target_path, ?string $name = null): void
	{

		if ($this->has_moved) {
			throw new \Exception('The uploaded file has already been moved');
		}

		if (!$this->is_valid()) {
			throw new \Exception('The original file is not a valid file');
		}

		$target_path = rtrim($target_path, ' \\/');
		$name ??= $this->name;
		$to = $target_path . DIRECTORY_SEPARATOR . $name;
		$this->has_moved = move_uploaded_file($this->tmp_name, $to);

		if ($this->has_moved === false) {
			throw new \Exception('move_uploaded_file() returned false');
		}

		@chmod($to, 0644);
		$this->saved_file = $to;
	}

	/**
	 * 将文件保存至上传目录
	 * @param string|null $folder_name 目录名，如为null，则使用 date('Ymd') 做为目录名称
	 * @param string|null $file_name 文件名，如为null，则使用随机文件名
	 * @return string|null 成功返回保存的文件路径,失败返回null
	 */
	public function store(?string $folder_name = null, ?string $file_name = null): ?string
	{
		try {
			$folder_name = rtrim($folder_name ?? date('Ymd'), ' \\/');
			$file_name ??= $this->get_random_name();
			$this->move(self::UPLOAD_PATH . DIRECTORY_SEPARATOR . $folder_name, $file_name);
			return $this->get_saved_file();
		} catch (\Exception $exception) {
			return null;
		}
	}

	/**
	 * get_name
	 * @return string
	 */
	public function get_name(): string
	{
		return $this->name;
	}

	/**
	 * get_type
	 * @return string
	 */
	public function get_type(): string
	{
		return $this->type;
	}

	/**
	 * get_tmp_name
	 * @return string
	 */
	public function get_tmp_name(): string
	{
		return $this->tmp_name;
	}

	/**
	 * get_size
	 * @return float
	 */
	public function get_size(): float
	{
		return $this->size;
	}

	/**
	 * get_error
	 * @return int
	 */
	public function get_error(): int
	{
		return $this->error;
	}

	/**
	 * get_saved_file
	 * @return string|null
	 */
	public function get_saved_file(): ?string
	{
		return $this->saved_file;
	}

	public function is_valid(): bool
	{
		return is_uploaded_file($this->tmp_name) && $this->error === UPLOAD_ERR_OK;
	}

	public function get_extension(): string
	{
		return pathinfo($this->name, PATHINFO_EXTENSION) ?? '';
	}

	/**
	 * 获取随机文件名
	 * @throws \Exception
	 */
	public function get_random_name(): string
	{
		$extension = $this->get_extension();
		$extension != '' && $extension = '.' . $extension;
		return time() . '_' . bin2hex(random_bytes(10)) . $extension;
	}
}