<?php

namespace venus\http;

use venus\base\component;
use venus\http\file\file_collection;

/**
 * @property $ip read-only
 * @property $method read-only
 * @property $user_agent read-only
 */
class request extends component
{
	private \config\request $config;

	private string $ip;

	private string $url;

	private string $method;

	private string $user_agent;

	private array $headers = [];

	private ?file_collection $file_collection = null;

	/**
	 * @param \config\request $config
	 */
	public function __construct(\config\request $config)
	{
		$this->config = $config;
		$this->ip = $_SERVER['REMOTE_ADDR'];

		//url
		$protocol = $this->is_https() ? 'https://' : 'http://';
		$host = $_SERVER['HTTP_HOST'];
		$request_uri = $_SERVER['REQUEST_URI'];
		$this->url = $protocol . $host . $request_uri;

		$this->method = $_SERVER['REQUEST_METHOD'];
		$this->user_agent = trim($_SERVER['HTTP_USER_AGENT']);
	}

	public function get_ip(): string
	{
		return $this->ip;
	}

	public function get_url(): string
	{
		return $this->url;
	}

	public function get_method(bool $upper = false): string
	{
		return ($upper) ? strtoupper($this->method) : strtolower($this->method);
	}

	public function is_post(): bool
	{
		return $this->get_method() == 'post';
	}

	public function get_user_agent(): string
	{
		return $this->user_agent;
	}

	public function get_query_string(): string
	{
		return $this->server('QUERY_STRING');
	}

	public function is_ajax(): bool
	{
		return $this->has_header('X-Requested-With') && strtolower($this->get_header('X-Requested-With') ?? '') === 'xmlhttprequest';
	}

	public function is_https(): bool
	{
		if (!empty($_SERVER['HTTPS']) && strtolower($_SERVER['HTTPS']) !== 'off') {
			return true;
		} elseif ($this->get_header('X-Forwarded-Proto') === 'https') {
			return true;
		} elseif (strtolower($this->get_header('Front-End-Https') ?? '') === 'on') {
			return true;
		}
		return false;
	}

	public function get_headers(): array
	{
		$this->fill_headers();
		return $this->headers;
	}

	public function get_header(string $name): ?string
	{
		$this->fill_headers();
		$name = strtolower($name);
		if (!isset($this->headers[$name])) {
			return null;
		}
		return $this->headers[$name];
	}

	public function has_header(string $name): bool
	{
		$this->fill_headers();
		$name = strtolower($name);
		return isset($this->headers[$name]);
	}

	public function remove_header(string $name): void
	{
		$this->fill_headers();
		$name = strtolower($name);
		if (isset($this->headers[$name])) {
			unset($this->headers[$name]);
		}
	}

	/**
	 * 获取上传的文件对象
	 * @param string|null $name 上传名称，如为null，则返回所有文件对象
	 * @return array|file\uploaded_file|null
	 */
	public function get_files(?string $name = null)
	{
		if (is_null($this->file_collection)) {
			$this->file_collection = new file_collection();
		}
		return $this->file_collection->get_files($name);
	}

	public function get(?string $var_name = null, $options = null)
	{
		return $this->get_var('get', $var_name, $options);
	}

	public function post(?string $var_name = null, $options = null)
	{
		return $this->get_var('post', $var_name, $options);
	}

	public function cookie(?string $var_name = null, $options = null)
	{
		//add cookie prefix
		if (!is_null($var_name)) {
			$var_name = $this->config->cookie_prefix . $var_name;
		}

		return $this->get_var('cookie', $var_name, $options);
	}

	public function request(?string $var_name = null, $options = null)
	{
		return $this->get_var('request', $var_name, $options);
	}

	public function server(?string $var_name = null, $options = null)
	{
		return $this->get_var('server', $var_name, $options);
	}

	/**
	 * 获取变量
	 * @param string $method get/post/cookie/request/server
	 * @param string|null $var_name 变量名
	 * @param int|array $options 过滤选项
	 * <pre>1.全局过滤器：<code>FILTER_DEFAULT</code>
	 * 2.为标量设置过滤器：
	 * <code>[
	 *      'filter' => FILTER_VALIDATE_INT,
	 *      'flags' => FILTER_REQUIRE_SCALAR,
	 *      'options' => [
	 *          'default' => 0,
	 *          'min_range' => 1,
	 *          'max_range' => 100
	 *      ]
	 * ]</code>
	 * 3.为数组中的每个元素分别设置过滤器：<a href="https://www.php.net/manual/zh/function.filter-var-array.php">https://www.php.net/manual/zh/function.filter-var-array.php</a>
	 * </pre>
	 * @return array|scalar|null
	 */
	private function get_var(string $method, ?string $var_name = null, $options = null)
	{
		$method = '_' . strtoupper($method);

		if (is_null($var_name)) {
			$var = $GLOBALS[$method];
		} else {
			if (!isset($GLOBALS[$method][$var_name]))
				return null;
			$var = $GLOBALS[$method][$var_name];
		}

		if ((!is_array($var) && !is_scalar($var)) || is_null($options))
			return $var;

		if (is_scalar($var)) {
			$var = [$var_name => $var];
			if (is_array($options)) {
				$options = [$var_name => $options];
			}
		}

		//init options
		if (is_array($options)) {
			foreach ($var as $key => $value) {
				if (isset($options[$key])) continue;
				if (is_scalar($value))
					$options[$key] = ['filter' => FILTER_DEFAULT, 'flags' => FILTER_REQUIRE_SCALAR];
				elseif (is_array($value))
					$options[$key] = ['filter' => FILTER_DEFAULT, 'flags' => FILTER_REQUIRE_ARRAY];
			}
		}

		//filter
		$var = filter_var_array($var, $options, false);
		if (is_array($var)) {
			return count($var) == 1 ? array_shift($var) : $var;
		}

		return null; //filter_var_array return false
	}

	private function fill_headers()
	{
		static $filled = false;
		if ($filled) {
			return;
		}

		$content_type = $_SERVER['CONTENT_TYPE'] ?? null;
		if (!empty($content_type)) {
			$this->headers['content-type'] = $content_type;
		}
		unset($content_type);

		foreach ($_SERVER as $key => $val) {
			if (sscanf($key, 'HTTP_%s', $header) === 1) {
				$header = str_replace('_', '-', strtolower($header));
				$this->headers[$header] = $val;
			}
		}

		$filled = true;
	}
}