<?php

namespace venus\http;

use venus\base\component;
use venus\di\container;
use venus\format\format;

class response extends component
{
	private \config\response $config;

	protected float $protocol_version = 1.1;
	protected int $status_code = status::HTTP_OK;

	protected array $headers = [];
	protected array $cookies = [];

	protected string $body = '';

	/**
	 * Construct
	 * @param \config\response $config
	 */
	public function __construct(\config\response $config)
	{
		$this->config = $config;
		$this->set_content_type('text/html');
	}

	public function get_protocol_version(): float
	{
		return $this->protocol_version;
	}

	public function set_protocol_version(float $protocol_version): response
	{
		$this->protocol_version = $protocol_version;
		return $this;
	}

	public function get_status_code(): int
	{
		return $this->status_code;
	}

	public function set_status_code(int $status_code): response
	{
		$this->status_code = $status_code;
		return $this;
	}

	public function set_body(string $body): response
	{
		$this->body = $body;
		return $this;
	}

	public function append_body(string $data): response
	{
		$this->body .= $data;
		return $this;
	}

	public function replace_body(array|string $search, array|string $replace): response
	{
		$this->body = str_replace($search, $replace, $this->body);
		return $this;
	}

	/**
	 * 推送内容
	 * @param string $content
	 * @param string $attachment_filename
	 * @param string $content_type
	 * @return $this
	 */
	public function push_content(string $content, string $attachment_filename, string $content_type = 'application/octet-stream'): response
	{
		$this->no_cache()
			->set_header('content-type', $content_type)
			->set_header('accept-ranges', 'bytes')
			->set_header('content-length', strlen($content))
			->set_header('content-md5', md5($content))
			->set_header('content-disposition', 'attachment; filename="' . $attachment_filename . '"')
			->set_body($content);
		return $this;
	}

	/**
	 * 推送文件
	 * @param string $file
	 * @param string|null $attachment_filename
	 * @param string $content_type
	 * @return $this
	 * @throws \Exception
	 */
	public function push_file(string $file, ?string $attachment_filename = null, string $content_type = 'application/octet-stream'): response
	{
		if (!is_file($file)) {
			throw new \Exception('file does not exist');
		}
		$content = '';
		$f = fopen($file, 'r');
		while (!feof($f)) {
			$content .= fread($f, 8192);
		}
		fclose($f);
		return $this->push_content($content, $attachment_filename ?? basename($file), $content_type);
	}

	/**
	 * @throws \Exception
	 */
	public function set_json($body): response
	{
		$this->body = $this->format_body($body, 'json');
		return $this;
	}

	/**
	 * @throws \Exception
	 */
	public function set_xml($body): response
	{
		$this->body = $this->format_body($body, 'xml');
		return $this;
	}

	/**
	 * @throws \Exception
	 */
	protected function format_body($body, string $format): string
	{
		$this->set_content_type("application/{$format}");

		// 如是字符串，则没什么可做的...
		if (!is_string($body)) {
			$body = container::get(format::class)->get_formatter($format)->format($body);
		}

		return $body;
	}

	/**
	 * 设置日期
	 * @param string $name
	 * @return $this
	 */
	public function set_date_header(string $name): response
	{
		try {
			$date = new \DateTime('now', new \DateTimeZone('UTC'));
			$this->set_header($name, $date->format('D, d M Y H:i:s') . ' GMT');
			return $this;
		} catch (\Exception $e) {
			return $this;
		}
	}

	public function no_cache(): response
	{
		$this->remove_header('cache-control');
		$this->set_header('cache-control', 'no-store, max-age=0, no-cache');
		return $this;
	}

	/**
	 * with
	 * @param string $key
	 * @param $message
	 * @return $this
	 * @throws \Exception
	 */
	public function with(string $key, $message): response
	{
		session()->set_flash_data($key, $message);
		return $this;
	}

	/**
	 * 将请求提交的数据保存在session中
	 * @return $this
	 * @throws \Exception
	 */
	public function with_input(): response
	{
		session()->set_flash_data('_venus_old_input', [
			'get' => $_GET ?? [],
			'post' => $_POST ?? []
		]);
		return $this;
	}

	public function redirect(string $url, int $status_code = status::HTTP_FOUND)
	{
		if (stripos($url, 'http://') !== 0 &&
			stripos($url, 'https://') !== 0 &&
			stripos($url, '/') !== 0
		) {
			$url = '/' . $url;
		}

		$this->set_status_code($status_code)->set_header('Location', $url)->send_headers();
		exit(0);
	}

	public function get_headers(): array
	{
		return $this->headers;
	}

	public function get_header(string $name): string|array|null
	{
		$name = strtolower($name);
		return $this->headers[$name] ?? null;
	}

	public function set_header(string $name, string|array $value): response
	{
		$name = strtolower($name);
		$this->headers[$name] = $value;
		return $this;
	}

	public function remove_header(string $name): response
	{
		$name = strtolower($name);
		unset($this->headers[$name]);
		return $this;
	}

	public function set_content_type(string $mime, string $charset = 'UTF-8'): response
	{
		if ((!str_contains($mime, 'charset=')) && !empty($charset)) {
			$mime .= '; charset=' . $charset;
		}
		$this->set_header('content-type', $mime);
		return $this;
	}

	/**
	 * set cookie
	 * @param string $name
	 * @param string $value
	 * @param array $options 选项值可包含下列值的一个或多个
	 * <p>prefix/expires/path/domain/secure/httponly/samesite</p>
	 * @return $this
	 */
	public function set_cookie(string $name, string $value = '', array $options = []): response
	{
		// Configuration options in the file
		$cookie_config_options = [
			'prefix' => $this->config->cookie_prefix,
			'expires' => $this->config->cookie_expires,
			'path' => $this->config->cookie_path,
			'domain' => $this->config->cookie_domain,
			'secure' => $this->config->cookie_secure,
			'httponly' => $this->config->cookie_httponly,
			'samesite' => $this->config->cookie_samesite
		];

		$options = array_merge($cookie_config_options, $options);
		if (isset($options['prefix'])) {
			$name = $options['prefix'] . $name;
			unset($options['prefix']);
		}

		$cookie = ['name' => $name, 'value' => $value, 'options' => $options];
		$this->cookies[$name] = $cookie;

		return $this;
	}

	public function remove_cookie(string $name): response
	{
		unset($this->cookies[$name]);
		return $this;
	}

	public function send(): response
	{
		$this->send_headers();
		$this->send_cookies();
		$this->send_body();
		return $this;
	}

	public function send_headers(): response
	{
		if (headers_sent()) {
			return $this;
		}

		// Per spec, MUST be sent with each request, if possible.
		// http://www.w3.org/Protocols/rfc2616/rfc2616-sec13.html
		if (!isset($this->headers['date'])) {
			$this->set_date_header('date');
		}

		// HTTP Status
		header(sprintf('HTTP/%s %s %s', $this->protocol_version, $this->status_code, status::get_status($this->status_code)), true, $this->status_code);

		// Send all of our headers
		foreach ($this->headers as $key => $value) {
			is_array($value) && $value = implode(',', $value);
			header($key . ': ' . $value, false, $this->status_code);
		}

		return $this;
	}

	public function send_cookies(): response
	{
		foreach ($this->cookies as $cookie) {
			setcookie($cookie['name'], $cookie['value'], $cookie['options']);
		}
		return $this;
	}

	public function send_body(): response
	{
		echo $this->body;
		return $this;
	}
}