<?php

namespace venus\language;

class language extends \venus\base\component
{
	public const NON_ADDITIONAL_LANGUAGE = 0;
	public const ADDITIONAL_CURRENT_MODULE_LANGUAGE = 1;
	public const ADDITIONAL_ALL_MODULE_LANGUAGE = 2;

	protected const COOKIE_NAME = 'venus_language';

	protected \config\language $config;

	protected array $data = [];

	protected array $loaded_file = [];

	protected string $language;

	/**
	 * __construct
	 * @param \config\language $config
	 * @throws \Exception
	 */
	public function __construct(\config\language $config)
	{
		$this->config = $config;
		$this->language = $this->language_lookup();
	}

	/**
	 * 获取语言数据
	 * @param string $key
	 * @param array $args
	 * @return string
	 */
	public function get(string $key, array $args = []): string
	{
		if (str_contains($key, '.')) {

			[$filename, $key] = explode('.', $key, 2);

			if (!isset($this->data[$filename]) || !is_array($this->data[$filename])) {
				$this->load_file($filename);
			}

			$message = $this->data[$filename][$key] ?? $key;
		} else {
			$message = $key;
		}

		return $this->format_message($message, $args);
	}

	/**
	 * 检测语言数据是否存在
	 * @param string $key
	 * @return bool
	 */
	public function key_is_exists(string $key): bool
	{
		if (!str_contains($key, '.')) {
			return false;
		}
		[$filename, $key] = explode('.', $key, 2);
		if (!isset($this->data[$filename]) || !is_array($this->data[$filename])) {
			$this->load_file($filename);
		}
		return isset($this->data[$filename][$key]);
	}

	public function get_cookie_name(): string
	{
		return self::COOKIE_NAME;
	}

	/**
	 * 更换语言
	 * @param string $language
	 * @return bool 成功返回true，失败返回false
	 */
	public function change_language(string $language): bool
	{
		if (in_array($language, $this->config->supported_language)) {
			\venus::$app->response->set_cookie(self::COOKIE_NAME, $language, [
				'prefix' => '',
				'expires' => TIMESTAMP + 31536000, //Expire after one year
				'httponly' => true
			]);
			return true;
		}
		return false;
	}

	/**
	 * 获取可以使用的语言
	 * @return array
	 */
	public function get_supported_language(): array
	{
		return $this->config->supported_language;
	}

	/**
	 * 获取当前语言
	 * @return string
	 */
	public function get_language(): string
	{
		return $this->language;
	}

	/**
	 * 格式化语言
	 * @param string $format 语言标签格式，可用标签
	 * <p>{lang}、{LANG}、{Lang}</p>
	 * <p>{tag}、{TAG}、{Tag}</p>
	 * <p>分别表示小写、大写、首字母大写</p>
	 * @return ?string 成功返回string，失败返回null
	 */
	public function get_format_language(string $format): ?string
	{
		static $data = null;
		if (is_null($data)) {
			[$language, $tag] = array_pad(explode('-', $this->language, 2), 2, null);
			$data = [
				'lang' => [strtolower($language), strtoupper($language), ucfirst(strtolower($language))],
				'tag' => is_null($tag) ? array_fill(0, 3, '') : [strtolower($tag), strtoupper($tag), ucfirst(strtolower($tag))]
			];
		}

		if (is_array($data)) {
			return strtr($format, [
				'{lang}' => $data['lang'][0], '{LANG}' => $data['lang'][1], '{Lang}' => $data['lang'][2],
				'{tag}' => $data['tag'][0], '{TAG}' => $data['tag'][1], '{Tag}' => $data['tag'][2]]);
		}

		return null;
	}

	/**
	 * 载入语言文件
	 * @param string $filename 语言文件名，不带扩展名
	 * @param int $mode 模块语言文件搜索路径
	 * @param array $paths 其它语言文件搜索路径
	 * @return void
	 */
	public function load_file(string $filename, int $mode = self::ADDITIONAL_CURRENT_MODULE_LANGUAGE, array $paths = []): void
	{
		//越往后，优先级越高
		$search_paths = [
			APP_PATH . 'language/' . $this->language
		];

		//模块语言文件搜索路径
		if ($mode === self::ADDITIONAL_CURRENT_MODULE_LANGUAGE) {
			//获取当前模块的语言目录
			$module_directory = \venus::$app->module->get_current_module('directory');
			if (!empty($module_directory)) {
				$language_directory = MODULE_PATH . $module_directory . '/language/' . $this->language;
				is_dir($language_directory) && $search_paths[] = $language_directory;
			}
		} elseif ($mode === self::ADDITIONAL_ALL_MODULE_LANGUAGE) {
			//获取所有模块的语言目录
			$modules = \venus::$app->module->get_modules();
			foreach ($modules as $module) {
				$language_directory = MODULE_PATH . $module['directory'] . '/language/' . $this->language;
				is_dir($language_directory) && $search_paths[] = $language_directory;
			}
		}

		//其它语言文件搜索路径
		foreach ($paths as $path) {
			$language_directory = rtrim($path, '\\/') . '/' . $this->language;
			is_dir($language_directory) && $search_paths[] = $language_directory;
		}

		foreach ($search_paths as $path) {
			$file = $path . '/' . $filename . '.php';
			if (!is_file($file) || in_array($file, $this->loaded_file)) {
				continue;
			}

			$data = include $file;
			if (empty($this->data[$filename])) {
				$this->data[$filename] = $data;
			} else {
				$this->data[$filename] = array_merge($this->data[$filename], $data);
			}

			$this->loaded_file[] = $file;
		}
	}

	/**
	 * 返回与当前环境相匹配的语言
	 * @throws \Exception
	 */
	protected function language_lookup(): string
	{
		if (empty($this->config->supported_language)) {
			throw new \Exception('No supported languages are set');
		}

		$supported_language = $this->config->supported_language;
		$default_language = $supported_language[0];

		if (count($supported_language) === 1) {
			return $supported_language[0];
		}

		if (isset($_COOKIE[self::COOKIE_NAME]) && in_array($_COOKIE[self::COOKIE_NAME], $supported_language)) {
			return $_COOKIE[self::COOKIE_NAME];
		}

		//内容协商
		$accept_language = \venus::$app->request->get_header('Accept-Language');
		if (!is_null($accept_language)) {
			$accept_language_items = explode(',', $accept_language);
			foreach ($accept_language_items as $accept_language_item) {
				[$language] = explode(';', $accept_language_item);
				if (in_array($language, $supported_language)) {
					return $language;
				}
			}
		}

		return $default_language;
	}

	/**
	 * 格式化语言数据
	 * @param string $message
	 * @param array $args
	 * @return string
	 */
	protected function format_message(string $message, array $args): string
	{
		$replace_pairs = [];
		foreach ($args as $key => $value) {
			$replace_pairs['{' . $key . '}'] = $value;
		}
		return strtr($message, $replace_pairs);
	}
}