<?php

namespace venus\router;

use venus\base\component;

class router extends component
{
	protected routes $config;

	protected ?string $uri = null;

	protected string $controller;

	protected string $method;

	protected array $params = [];

	public function __construct(routes $config)
	{
		$this->config = $config;
	}

	public function get_config(): routes
	{
		return $this->config;
	}

	/**
	 * get_controller
	 * @return string
	 */
	public function get_controller(): string
	{
		return $this->controller;
	}

	/**
	 * get_method
	 * @return string
	 */
	public function get_method(): string
	{
		return $this->method;
	}

	/**
	 * get_params
	 * @return array
	 */
	public function get_params(): array
	{
		return $this->params;
	}

	/**
	 * 载入配置项
	 */
	public function load_settings(): router
	{
		$files = [
			CONFIG_PATH . 'routes.php'
		];

		// get the route files for all modules
		$modules = \venus::$app->module->get_modules();
		foreach ($modules as $module) {
			if (empty($module['directory'])) {
				continue;
			}
			$routes_file = MODULE_PATH . $module['directory'] . '/config/routes.php';
			is_file($routes_file) && $files[] = $routes_file;
		}

		foreach ($files as $file) {
			if (is_file($file)) {
				include $file;
			}
		}

		return $this;
	}

	/**
	 * 根据当前 uri 返回控制器、方法与参数 (或者是匿名函数与参数)，失败返回false
	 * @return array|false
	 */
	public function handle()
	{
		//init uri
		$this->init_uri();
		if (is_null($this->uri)) {
			return false;
		}

		//translate_uri_dashes
		if ($this->config->get_translate_uri_dashes()) {
			$this->uri = str_replace('-', '_', $this->uri);
		}

		//matching route rule
		$result = $this->match_route_rules();

		if ($result === false && $this->config->get_auto_route()) {
			//auto route
			$result = $this->auto_route();
		}

		if (is_array($result) && count($result) == 2) {
			return $this->parse($result[0], $result[1]);
		}

		return false;
	}

	/**
	 * 自动路由
	 * @return array|false
	 */
	private function auto_route()
	{
		if (empty($this->uri)) {
			return [
				$this->config->get_default_controller() . '::' . $this->config->get_default_method(),
				[]
			];
		}

		$result = false;
		$namespace = $this->config->get_default_namespace();
		$directory = $this->config->get_default_directory();

		$path_array = explode('/', $this->uri);
		foreach ($path_array as $key => $value) {
			$if_file = $directory . DIRECTORY_SEPARATOR . $value . '.php';

			if (is_file($if_file)) {

				$to = $namespace . $value;
				//Discards by array_shift in the parse method
				$params = ['unless'];

				$slice = array_slice($path_array, $key + 1);
				$method = array_shift($slice);
				if (!is_null($method)) {
					$to .= '::' . $method;
					foreach ($slice as $index => $param) {
						$to .= '/$' . ($index + 1);
						$params[] = $param;
					}
				}
				$result = [$to, $params];
				break;

			} else {
				$namespace .= $value . '\\';
				$directory .= DIRECTORY_SEPARATOR . $value;
			}
		}

		return $result;
	}

	/**
	 * 匹配路由规则，成功则返回设置的规则及匹配的实参，失败返回false.
	 * @return array|false
	 */
	private function match_route_rules()
	{
		//get routing rules
		$method = \venus::$app->request->get_method();
		$setting_routes = $this->config->get_routes();

		//指定请求方法的路由优先级高于'*'路由
		$routes = isset($setting_routes[$method]) ? $setting_routes[$method] + $setting_routes['*'] : $setting_routes['*'];

		if (empty($routes)) {
			return false;
		}

		//按优先级排序
		$priority = array_column($routes, 'priority');
		array_multisort($priority, SORT_DESC, SORT_NUMERIC, $routes);

		//try matching
		$result = false;
		foreach ($routes as $setting) {
			$pattern = "~^{$setting['from']}$~iu";
			$match_result = preg_match($pattern, $this->uri, $matches);
			if ($match_result) {
				$result = [
					$setting['to'],
					$matches
				];
				break;
			}
		}

		return $result;
	}

	/**
	 * 解析控制器、方法、参数信息
	 * @param string|\Closure $to
	 * @param array $matches
	 * @return array
	 */
	private function parse($to, array $matches): array
	{
		//args
		$args = $matches;
		array_shift($args);

		//$to is anonymous function
		if ($to instanceof \Closure) {
			return [$to, $args];
		}

		//replace params
		$replace_pairs = [];
		foreach ($args as $key => $arg) {
			$replace_pairs['$' . ($key + 1)] = $arg;
		}
		$to = strtr($to, $replace_pairs);

		//controller
		[$controller, $method_and_params] = array_pad(explode('::', $to, 2), 2, null);
		empty($controller) && $controller = $this->config->get_default_controller();

		//$method_and_params
		if (empty($method_and_params)) {
			$method = $this->config->get_default_method();
			$params = [];
		} else {
			[$method, $params_string] = array_pad(explode('/', $method_and_params, 2), 2, null);
			empty($method) && $method = $this->config->get_default_method();
			//params
			$params = is_null($params_string) || trim($params_string) == '' ? [] : explode('/', $params_string);
		}

		//If the controller does not contain a namespace, the default namespace is added
		strpos($controller, '\\') === false && $controller = $this->config->get_default_namespace() . $controller;

		$this->controller = $controller;
		$this->method = $method;
		$this->params = $params;

		return [$controller, $method, $params];
	}

	/**
	 * 初始化uri
	 * @return void
	 */
	private function init_uri(): void
	{
		if (isset($_SERVER['PATH_INFO'])) {
			$this->uri = trim($_SERVER['PATH_INFO'], '/');
		} elseif (isset($_SERVER['REQUEST_URI'])) {
			$path = parse_url($_SERVER['REQUEST_URI'], PHP_URL_PATH);
			if ($path !== false) {
				if (strpos($path, $_SERVER['SCRIPT_NAME']) === 0) {
					$path = substr($path, strlen($_SERVER['SCRIPT_NAME']));
				}
				$this->uri = trim($path, '/');
			}
		}

		//trim url_suffix
		$url_suffix = $this->config->get_url_suffix();
		if (!is_null($this->uri) &&
			$url_suffix !== '' &&
			strtolower(substr($this->uri, 0 - strlen($url_suffix))) === strtolower($url_suffix)
		) {
			$this->uri = substr($this->uri, 0, 0 - strlen($url_suffix));
		}
	}

}