<?php

namespace venus\router;

class routes extends \venus\base\config
{
	public const LOW_PRIORITY = 0;
	public const MIDDLE_PRIORITY = 100;
	public const HIGH_PRIORITY = 200;

	protected array $routes = [
		'*' => [],
		'get' => [],
		'post' => [],
	];

	protected array $place_holders = [
		'any' => '.*',
		'segment' => '[^/]+',
		'alphanum' => '[a-zA-Z0-9]+',
		'num' => '[0-9]+',
		'alpha' => '[a-zA-Z]+',
	];

	/**
	 * @var bool 是否开启自动路由
	 */
	protected bool $auto_route = true;

	/**
	 * @var bool 是否将 uri 中的连字符'-'转换成'_'
	 */
	protected bool $translate_uri_dashes = false;

	/**
	 * @var string 默认的控制器目录，末尾不含'/'
	 */
	protected string $default_directory = APP_PATH . 'controller';

	/**
	 * @var string 默认的控制器名字空间，末尾含'\'
	 */
	protected string $default_namespace = '\\app\\controller\\';

	/**
	 * @var string 默认控制器
	 */
	protected string $default_controller = 'index';

	/**
	 * @var string 默认方法
	 */
	protected string $default_method = 'index';

	/**
	 * @var string url后缀，如：.html
	 */
	protected string $url_suffix = '';

	/**
	 * @var string 组前缀
	 */
	private string $group_prefix = '';

	/**
	 * @var array 组选项
	 */
	private array $group_options = [];

	public function get(string $from, string|\Closure $to, array $options = []): void
	{
		$this->add($from, $to, $options, 'get');
	}

	public function post(string $from, string|\Closure $to, array $options = []): void
	{
		$this->add($from, $to, $options, 'post');
	}

	/**
	 * 添加路由
	 * @param string $from URI String
	 * @param string|\Closure $to class or closure
	 * @param array $options options
	 * @param string|array $methods request method
	 * @return void
	 * @example $routes->add('product/(:any)', 'Catalog::productLookup');
	 */
	public function add(string $from, string|\Closure $to, array $options = [], string|array $methods = '*'): void
	{
		//group_prefix
		$from = trim($from, " \t\n\r\0\x0B/");
		$from = trim(strip_tags($this->group_prefix . '/' . $from), " \t\n\r\0\x0B/");
		//place_holders
		foreach ($this->place_holders as $tag => $pattern) {
			$from = str_ireplace(':' . $tag, $pattern, $from);
		}

		//options
		$default_options = ['priority' => self::LOW_PRIORITY];
		$options = array_merge($default_options, $this->group_options, $options);

		//namespace
		if (isset($options['namespace']) && is_string($to)) {
			$to = $options['namespace'] . '\\' . $to;
		}

		//domain
		if (isset($options['domain'])) {
			$list = is_array($options['domain']) ? $options['domain'] : [$options['domain']];
			if(!$this->host_in_list($list)) {
				return;
			}
		}

		//exclude_domain
		if(isset($options['exclude_domain'])) {
			$list = is_array($options['exclude_domain']) ? $options['exclude_domain'] : [$options['exclude_domain']];
			if($this->host_in_list($list)) {
				return;
			}
		}

		if (is_string($methods)) {
			$methods = [$methods];
		}

		foreach ($methods as $method) {
			$this->routes[$method][$from] = [
				'from' => $from,
				'to' => $to,
				'priority' => $options['priority']
			];
		}
	}

	/**
	 * 新增路由组
	 * @param string $name
	 * @param array|callable ...$params
	 * @return void
	 */
	public function group(string $name, ...$params): void
	{
		$old_group_prefix = $this->group_prefix;
		$old_group_options = $this->group_options;

		$this->group_prefix = $name != '' ? trim($old_group_prefix . '/' . $name, '/') : $old_group_prefix;

		$callable = array_pop($params);

		if(isset($params[0]) && is_array($params[0])) {
			$this->group_options = array_shift($params);
		}

		if (is_callable($callable)) {
			$callable($this);
		}

		$this->group_prefix = $old_group_prefix;
		$this->group_options = $old_group_options;
	}

	private function host_in_list(array $list): bool
	{
		$http_host = \venus::$app->request->server('HTTP_HOST');
		if (is_null($http_host))
			return false;

		return in_array(strtolower($http_host), array_map('strtolower', $list));
	}

	/**
	 * @return array
	 */
	public function get_routes(): array
	{
		return $this->routes;
	}

	/**
	 * @return bool
	 */
	public function get_auto_route(): bool
	{
		return $this->auto_route;
	}

	/**
	 * @param bool $auto_route
	 */
	public function set_auto_route(bool $auto_route): void
	{
		$this->auto_route = $auto_route;
	}

	/**
	 * @return string
	 */
	public function get_default_directory(): string
	{
		return $this->default_directory;
	}

	/**
	 * @param string $default_directory
	 */
	public function set_default_directory(string $default_directory): void
	{
		$this->default_directory = $default_directory;
	}

	/**
	 * @return string
	 */
	public function get_default_namespace(): string
	{
		return $this->default_namespace;
	}

	/**
	 * @param string $default_namespace
	 */
	public function set_default_namespace(string $default_namespace): void
	{
		$this->default_namespace = $default_namespace;
	}

	/**
	 * @return string
	 */
	public function get_default_controller(): string
	{
		return $this->default_controller;
	}

	/**
	 * @param string $default_controller
	 */
	public function set_default_controller(string $default_controller): void
	{
		$this->default_controller = $default_controller;
	}

	/**
	 * @return string
	 */
	public function get_default_method(): string
	{
		return $this->default_method;
	}

	/**
	 * @param string $default_method
	 */
	public function set_default_method(string $default_method): void
	{
		$this->default_method = $default_method;
	}

	/**
	 * @return string
	 */
	public function get_url_suffix(): string
	{
		return trim($this->url_suffix);
	}

	/**
	 * @param string $url_suffix
	 */
	public function set_url_suffix(string $url_suffix): void
	{
		$this->url_suffix = trim($url_suffix);
	}

	/**
	 * @return bool
	 */
	public function get_translate_uri_dashes(): bool
	{
		return $this->translate_uri_dashes;
	}

	/**
	 * @param bool $translate_uri_dashes
	 */
	public function set_translate_uri_dashes(bool $translate_uri_dashes): void
	{
		$this->translate_uri_dashes = $translate_uri_dashes;
	}

}