<?php

namespace venus\session;

class session
{
	protected const VENUS_VARS = '__venus_vars__';

	private \config\session $config;

	/**
	 * @param \config\session $config
	 */
	public function __construct(\config\session $config)
	{
		$this->config = $config;
	}

	public function __set(string $key, $value): void
	{
		$_SESSION[$key] = $value;
	}

	public function __get(string $key)
	{
		return $_SESSION[$key] ?? null;
	}

	/**
	 * 获取 session
	 * @param string|null $key
	 * @return array|mixed|null
	 */
	public function get(?string $key = null)
	{
		$session = !empty($_SESSION) ? $_SESSION : [];
		unset($session[self::VENUS_VARS]);
		return is_null($key) ? $session : ($session[$key] ?? null);
	}

	/**
	 * 设置 session
	 * @param string|array $data
	 * @param mixed|null $value
	 * @return session
	 */
	public function set($data, $value = null): session
	{
		if (is_array($data)) {
			foreach ($data as $key => $value) {
				if (is_int($key)) {
					$_SESSION[$value] = null;
				} else {
					$_SESSION[$key] = $value;
				}
			}
		} elseif (is_string($data)) {
			$_SESSION[$data] = $value;
		}
		return $this;
	}

	/**
	 * 删除 session
	 * @param string|array $key
	 * @return session
	 */
	public function remove($key): session
	{
		if (is_array($key)) {
			foreach ($key as $k) {
				unset($_SESSION[$k]);
			}
		} elseif (is_string($key)) {
			unset($_SESSION[$key]);
		}
		return $this;
	}

	/**
	 * 检测 session 是否存在
	 * @param string $key
	 * @return bool
	 */
	public function has(string $key): bool
	{
		return isset($_SESSION[$key]);
	}

	/**
	 * 设置 flash session
	 * @param string|array $data
	 * @param mixed|null $value
	 * @return session
	 */
	public function set_flash_data($data, $value = null): session
	{
		$this->set($data, $value);
		$this->add_flash_mark(is_array($data) ? array_keys($data) : $data);
		return $this;
	}

	/**
	 * 获取 flash session
	 * @param string|null $key
	 * @return array|mixed|null
	 */
	public function get_flash_data(?string $key = null)
	{
		if (!is_null($key)) {
			return isset($_SESSION[self::VENUS_VARS][$key], $_SESSION[$key]) ? $_SESSION[$key] : null;
		}

		$flash_data = [];

		if (is_array($_SESSION[self::VENUS_VARS])) {
			foreach ($_SESSION[self::VENUS_VARS] as $key => $value) {
				$flash_data[$key] = $_SESSION[$key];
			}
		}

		return $flash_data;
	}

	/**
	 * 添加 flash 标记
	 * @param string|array $key
	 * @return session
	 */
	public function add_flash_mark($key): session
	{
		if (is_array($key)) {
			foreach ($key as $session_key) {
				if (isset($_SESSION[$session_key])) {
					$_SESSION[self::VENUS_VARS][$session_key] = 0;
				}
			}
		}

		if (is_string($key) && isset($_SESSION[$key])) {
			$_SESSION[self::VENUS_VARS][$key] = 0;
		}

		return $this;
	}

	/**
	 * 移除 flash 标记
	 * @param string|array $key
	 * @return session
	 */
	public function remove_flash_mark($key): session
	{
		if (isset($_SESSION[self::VENUS_VARS]) && is_array($_SESSION[self::VENUS_VARS])) {
			if (!is_array($key)) {
				$key = [$key];
			}
			foreach ($key as $k) {
				if (isset($_SESSION[self::VENUS_VARS][$k])) {
					unset($_SESSION[self::VENUS_VARS][$k]);
				}
			}
			if (empty($_SESSION[self::VENUS_VARS])) {
				unset($_SESSION[self::VENUS_VARS]);
			}
		}

		return $this;
	}

	/**
	 * 保留 flash session 至下一次请求
	 * @param string|array $key
	 * @return session
	 */
	public function keep_flash_data($key): session
	{
		$this->add_flash_mark($key);
		return $this;
	}

	/**
	 * 启用 session
	 * @param bool $read_and_close 如果将此选项的值设置为 true， 那么会话文件会在读取完毕之后马上关闭， 因此，可以在会话数据没有变动的时候，避免不必要的文件锁。
	 * @return session
	 */
	public function start(bool $read_and_close = false): session
	{

		if (ini_get('session.auto_start')) {
			return $this;
		}

		if (session_status() != PHP_SESSION_NONE) {
			return $this;
		}

		// session_name
		if (isset($this->config->session_name)) {
			session_name($this->config->session_name);
		}

		// session_save_path
		if (isset($this->config->session_save_path)) {
			session_save_path($this->config->session_save_path);
		}

		// session_cookie_params
		$cookie_params = array_merge($this->config->session_cookie_params, ['lifetime' => $this->config->session_lifetime]);
		session_set_cookie_params($cookie_params);

		// session_gc_max_lifetime
		$gc_max_lifetime = $this->config->session_lifetime > 0 ? $this->config->session_lifetime : 1440;
		ini_set('session.gc_max_lifetime', $gc_max_lifetime);

		// ini_set
		ini_set('session.use_trans_sid', '0');
		ini_set('session.use_strict_mode', '1');
		ini_set('session.use_cookies', '1');
		ini_set('session.use_only_cookies', '1');

		// session_start
		session_start(['read_and_close' => $read_and_close]);

		// update_cookie
		$this->update_cookie($this->config->session_lifetime);

		// handle_venus_vars
		$this->handle_venus_vars();

		return $this;
	}

	/**
	 * 销毁会话
	 * @return void
	 */
	public function destroy(): void
	{
		$_SESSION = [];
		$this->update_cookie(-3600);
		session_destroy();
	}

	/**
	 * handle_venus_vars
	 * @return void
	 */
	private function handle_venus_vars(): void
	{
		if (!isset($_SESSION[self::VENUS_VARS]) || !is_array($_SESSION[self::VENUS_VARS])) {
			return;
		}

		foreach ($_SESSION[self::VENUS_VARS] as $key => $value) {
			$_SESSION[self::VENUS_VARS][$key]++;
			if ($_SESSION[self::VENUS_VARS][$key] > 1) {
				unset($_SESSION[$key], $_SESSION[self::VENUS_VARS][$key]);
			}
		}

		if (empty($_SESSION[self::VENUS_VARS])) {
			unset($_SESSION[self::VENUS_VARS]);
		}
	}

	/**
	 * 更新 cookie
	 * @param int $lifetime
	 * @return void
	 */
	private function update_cookie(int $lifetime): void
	{
		$session_name = $this->config->session_name;
		if (isset($_COOKIE[$session_name]) && $_COOKIE[$session_name] == session_id()) {
			\venus::$app->response->set_cookie($session_name, $_COOKIE[$session_name], [
				'prefix' => '',
				'expires' => TIMESTAMP + $lifetime,
				'path' => $this->config->session_cookie_params['path'],
				'domain' => $this->config->session_cookie_params['domain'],
				'secure' => $this->config->session_cookie_params['secure'],
				'httponly' => $this->config->session_cookie_params['httponly'],
				'samesite' => $this->config->session_cookie_params['samesite'],
			]);
		}
	}

}