<?php

namespace venus\validation;

use Exception;
use venus\base\component;
use venus\di\container;
use venus\validation\validator\general;

class validation extends component
{
	/**
	 * @var array $validator_list 验证器实例
	 */
	protected array $validator_list = [];

	/**
	 * @var array $rules 规则集合
	 */
	protected array $rules = [];

	/**
	 * @var array $data 需要验证的数据
	 */
	protected array $data = [];

	/**
	 * @var array $errors 出错信息
	 */
	protected array $errors = [];


	/**
	 * 载入配置文件
	 * @param string $config_class 配置文件类名
	 * @param ?string $scene 场景
	 * @throws Exception
	 */
	public function load_config(string $config_class, ?string $scene = null): validation
	{
		/**
		 * @var config $config
		 */
		$config = container::get_instance($config_class);

		// validator class
		$this->load_validator($config->get_validator_class());

		// rules
		$this->rules = $config->get_rules($scene);

		return $this;
	}

	/**
	 * 载入验证器
	 * @param array|string $class_list 验证器类名
	 * @throws Exception
	 */
	public function load_validator(array|string $class_list): validation
	{
		if (empty($class_list)) {
			throw new Exception('Validator class names cannot is empty');
		}

		if (is_string($class_list)) {
			$class_list = [$class_list];
		}

		foreach ($class_list as $class) {
			if (!array_key_exists($class, $this->validator_list)) {
				$this->validator_list[$class] = container::get_instance($class);
			}
		}

		return $this;
	}

	/**
	 * 将请求提交的数据设置为待验证数据
	 * @return $this
	 */
	public function with_request(): validation
	{
		$this->data = \venus::$app->request->request();
		return $this;
	}

	/**
	 * 将get数据设置为待验证数据
	 * @return $this
	 */
	public function with_get(): validation
	{
		$this->data = \venus::$app->request->get();
		return $this;
	}

	/**
	 * 将post数据设置为待验证数据
	 * @return $this
	 */
	public function with_post(): validation
	{
		$this->data = \venus::$app->request->post();
		return $this;
	}

	/**
	 * 开始验证
	 * @throws Exception
	 */
	public function run(): bool
	{
		helper('array');

		// If no rules exist, we return false to ensure
		// the developer didn't forget to set the rules.
		if (empty($this->rules)) {
			return false;
		}

		//flatten data
		$flatten_data = array_flatten_with_dots($this->data);

		//fill placeholders
		$this->rules = $this->fill_placeholders($this->rules, $flatten_data);

		$run_result = true;
		foreach ($this->rules as $field => $setup) {
			$process_result = $this->process_rules($field, $setup, $flatten_data);
			!$process_result && $run_result = false;
		}
		return $run_result;
	}

	/**
	 * 验证单个值
	 * @param mixed $value 需要验证的值
	 * @param array|string $rules 验证规则
	 * @param string $class 验证器类名
	 * @return bool
	 * @throws Exception
	 */
	public function check(mixed $value, array|string $rules, string $class = general::class): bool
	{
		$this->reset();
		$rules = ['check' => ['rules' => $rules]];
		$data = ['check' => $value];

		return $this->load_validator($class)->set_rules($rules)->set_data($data)->run();
	}

	public function reset(): validation
	{
		$this->data = [];
		$this->rules = [];
		$this->errors = [];
		return $this;
	}

	/**
	 * 验证规则
	 * @param string $field 扁平化字段名 (flatten)
	 * @param array $setup 设置的验证规则及出错信息
	 * @param array $data 经过扁平化处理的数据
	 * @return bool
	 * @throws Exception
	 */
	protected function process_rules(string $field, array $setup, array $data): bool
	{
		//value
		try{
			$value = get_value_by_flatten_key($field, $this->data);
		}catch (Exception) {
			$value = null;
		}

		//field rules
		$rules = $this->get_field_rules($setup['rules']);
		if (empty($rules)) {
			throw new Exception('Invalid validation rule');
		}

		foreach ($rules as $rule) {

			if (preg_match('/(.*?)\[(.*)\]/', $rule, $match)) {
				$method = $match[1];
				$params = $match[2];
			} else {
				$method = $rule;
				$params = null;
			}

			$found = $passed = false;
			foreach ($this->validator_list as $validator) {
				if (!method_exists($validator, $method)) {
					continue;
				}
				$found = true;
				$passed = is_null($params) ? $validator->{$method}($value, $data, $this->data)
					: $validator->{$method}($value, $params, $data, $this->data);
				break;
			}

			if (!$found) {
				throw new Exception('Validator method not found (' . $method . ')');
			}

			if (!$passed) {
				$this->errors[$field] = $setup['errors'][$method] ?? "$field validation failed ($method)";
				return false;
			}
		}

		return true;
	}

	/**
	 * 获取字段验证规则
	 * @param array|string $rules
	 * @return array
	 */
	protected function get_field_rules(array|string $rules): array
	{
		$result = [];

		if (is_string($rules)) {
			$rules = explode('|', $rules);
		}

		if (is_array($rules)) {
			foreach ($rules as $rule) {
				if (!is_string($rule)) {
					continue;
				}
				$rule = trim($rule);
				if ($rule != '') {
					$result[] = $rule;
				}
			}
		}

		return array_unique($result);
	}


	/**
	 * 替换占位符
	 * @param array $rules 规则集合
	 * @param array $data 经过扁平化处理的数据
	 * @return array
	 */
	protected function fill_placeholders(array $rules, array $data): array
	{
		$replacements = [];

		foreach ($data as $key => $value) {
			/** @noinspection PhpUnnecessaryCurlyVarSyntaxInspection */
			$replacements["{{$key}}"] = $value;
		}

		if (!empty($replacements)) {
			foreach ($rules as &$rule) {
				$rule_set = $rule['rules'];
				//array
				if (is_array($rule_set)) {
					foreach ($rule_set as &$row) {
						if (is_string($row)) {
							$row = strtr($row, $replacements);
						}
					}
				}
				//string
				if (is_string($rule_set)) {
					$rule_set = strtr($rule_set, $replacements);
				}

				$rule['rules'] = $rule_set;
			}
		}

		return $rules;
	}

	/**
	 * 返回规则集
	 * @return array
	 */
	public function get_rules(): array
	{
		return $this->rules;
	}

	/**
	 * 设置规则集
	 * @param array $rules
	 * @return $this
	 */
	public function set_rules(array $rules): validation
	{
		$this->rules = $rules;
		return $this;
	}

	/**
	 * @return array
	 */
	public function get_data(): array
	{
		return $this->data;
	}

	/**
	 * @param array $data
	 * @return validation
	 */
	public function set_data(array $data): validation
	{
		$this->data = $data;
		return $this;
	}

	/**
	 * @return array
	 */
	public function get_errors(): array
	{
		return $this->errors;
	}

	/**
	 * @param array $errors
	 * @return validation
	 */
	public function set_errors(array $errors): validation
	{
		$this->errors = $errors;
		return $this;
	}

	public function has_errors(): bool
	{
		return !empty($this->errors);
	}
}