<?php

namespace venus\validation\validator;

class general
{
	/**
	 * 匹配只包含字母的字符串
	 * @param $value
	 * @return bool
	 */
	public function alpha($value): bool
	{
		return ctype_alpha($value);
	}

	/**
	 * 匹配只包含数字(0-9)的字符串
	 * @param $value
	 * @return bool
	 */
	public function digit($value): bool
	{
		if ($value === null) {
			return false;
		}
		return ctype_digit($value);
	}

	/**
	 * 匹配只包含字母和数字的字符串
	 * @param $value
	 * @return bool
	 */
	public function alpha_numeric($value): bool
	{
		return ctype_alnum($value);
	}

	/**
	 * 匹配只包含字母、数字、下划线、破折号的字符串
	 * @param $value
	 * @return bool
	 */
	public function alpha_dash($value): bool
	{
		if ($value === null) {
			return false;
		}

		return preg_match('/\A[a-z0-9_-]+\z/i', $value) === 1;
	}

	/**
	 * 匹配自然数，即非负整数(首位不能为0，0除外)
	 * @param $value
	 * @return bool
	 */
	public function natural_number($value): bool
	{
		if($value === null) {
			return false;
		}
		return (bool)preg_match('/\A0\z|\A[1-9][0-9]*\z/', $value);
	}

	/**
	 * 匹配正整数(首位不能为0)
	 * @param $value
	 * @return bool
	 */
	public function positive_integer($value): bool
	{
		if($value === null) {
			return false;
		}
		return (bool)preg_match('/\A[1-9][0-9]*\z/', $value);
	}

	/**
	 * 匹配整数，即正整数、零、负整数的集合(首位不能为0，0除外)
	 * @param $value
	 * @return bool
	 */
	public function integer($value): bool
	{
		if ($value === null) {
			return false;
		}
		return (bool)preg_match('/\A0\z|\A\-?[1-9][0-9]*\z/', $value);
	}

	/**
	 * 如果字段是有效的电子邮件地址，则成功。
	 * @param $value
	 * @return bool
	 */
	public function valid_email($value): bool
	{
		return (bool)filter_var($value, FILTER_VALIDATE_EMAIL);
	}

	/**
	 * 如果字段是有效的url，则成功。
	 * @param $value
	 * @return bool
	 */
	public function valid_url($value): bool
	{
		if (empty($value)) {
			return false;
		}

		if (!preg_match('/^https?\:\/\/.+$/i', $value, $matches)) {
			return false;
		}

		return filter_var($value, FILTER_VALIDATE_URL) !== false;
	}

	/**
	 * 如果字段是有效的ip，则成功。参数可指定用于检测的ip类型，如：ipv4 或 ipv6
	 * @param $value
	 * @param $param
	 * @return bool
	 */
	public function valid_ip($value, $param): bool
	{
		if (empty($value)) {
			return false;
		}

		$option = match ($param) {
			'ipv4' => FILTER_FLAG_IPV4,
			'ipv6' => FILTER_FLAG_IPV6,
			default => 0,
		};

		return filter_var($value, FILTER_VALIDATE_IP, $option) !== false;
	}

	/**
	 * 如果字段与正则表达式匹配，则成功。
	 * @param $value
	 * @param $param
	 * @return bool
	 */
	public function regex_match($value, $param): bool
	{
		if ($value === null) {
			return false;
		}

		if (!str_starts_with($param, '/')) {
			$param = "/{$param}/";
		}

		return (bool)preg_match($param, $value);
	}

	/**
	 * 如果字段与参数指定的字段不相同，则成功
	 * @param $value
	 * @param $param
	 * @param $data
	 * @return bool
	 */
	public function differs($value, $param, $data): bool
	{
		return array_key_exists($param, $data) && $value !== $data[$param];
	}

	/**
	 * 如果字段与参数指定的字段相同，则成功
	 * @param $value
	 * @param $param
	 * @param $data
	 * @return bool
	 */
	public function matches($value, $param, $data): bool
	{
		return array_key_exists($param, $data) && $value === $data[$param];
	}

	/**
	 * 如果字段与参数值相同，则成功
	 * @param $value
	 * @param $param
	 * @return bool
	 */
	public function equals($value, $param): bool
	{
		return $value === $param;
	}

	/**
	 * 如果字段与参数值相同(忽略大小写)，则成功
	 * @param $value
	 * @param $param
	 * @return bool
	 */
	public function equals_i($value, $param): bool
	{
		return strtolower($value) === strtolower($param);
	}

	/**
	 * 如果字段与参数值不相同，则成功
	 * @param $value
	 * @param $param
	 * @return bool
	 */
	public function not_equals($value, $param): bool
	{
		return $value !== $param;
	}

	/**
	 * 如果字段与参数值不相同(忽略大小写)，则成功
	 * @param $value
	 * @param $param
	 * @return bool
	 */
	public function not_equals_i($value, $param): bool
	{
		return strtolower($value) !== strtolower($param);
	}

	/**
	 * 如果字段为空数组、空字符串、null或false，则失败
	 * @param $value
	 * @return bool
	 */
	public function required($value): bool
	{
		if ($value === null) {
			return false;
		}

		if (is_object($value)) {
			return true;
		}

		if (is_array($value)) {
			return !empty($value);
		}

		return trim((string)$value) !== '';
	}

	/**
	 * 如果字段是数值，且大于参数值，则成功。
	 * @param $value
	 * @param $param
	 * @return bool
	 */
	public function greater_than($value, $param): bool
	{
		return is_numeric($value) && $value > $param;
	}

	/**
	 * 如果字段是数值，且大于或等于参数值，则成功。
	 * @param $value
	 * @param $param
	 * @return bool
	 */
	public function greater_than_equal_to($value, $param): bool
	{
		return is_numeric($value) && $value >= $param;
	}

	/**
	 * 如果字段是数值，且小于参数值，则成功
	 * @param $value
	 * @param $param
	 * @return bool
	 */
	public function less_than($value, $param): bool
	{
		return is_numeric($value) && $value < $param;
	}

	/**
	 * 如果字段是数值，且小于或等于参数值，则成功
	 * @param $value
	 * @param $param
	 * @return bool
	 */
	public function less_than_equal_to($value, $param): bool
	{
		return is_numeric($value) && $value <= $param;
	}

	/**
	 * 如果字段是整型，且位于min与max之间（含），则成功
	 * @param $value
	 * @param $params
	 * @return bool
	 * @throws \Exception
	 */
	public function int_between($value, $params): bool
	{
		if(!$this->integer($value)) {
			return false;
		}

		if ($params === null) {
			throw new \Exception('You must supply the parameters: params');
		}

		[$min, $max] = explode(',', $params);

		if (!is_numeric($min) || !is_numeric($max)) {
			throw new \Exception('params error');
		}

		return (int)$value >= (int)$min && (int)$value <= (int)$max;
	}

	/**
	 * 如果字段在参数列表中，则成功。
	 * @param $value
	 * @param $params
	 * @return bool
	 */
	public function in_list($value, $params): bool
	{
		$params = array_map('trim', explode(',', $params));
		return in_array($value, $params, true);
	}

	/**
	 * 如果字段在参数列表中，则成功（忽略大小写）
	 * @param $value
	 * @param $params
	 * @return bool
	 */
	public function in_list_i($value, $params): bool
	{
		$params = array_map(function ($v) {
			return strtolower(trim($v));
		}, explode(',', $params));

		return in_array(strtolower($value), $params, true);
	}

	/**
	 * 如果字段不在参数列表中，则成功
	 * @param $value
	 * @param $params
	 * @return bool
	 */
	public function not_in_list($value, $params): bool
	{
		return !$this->in_list($value, $params);
	}

	/**
	 * 如果字段不在参数列表中，则成功（忽略大小写）
	 * @param $value
	 * @param $params
	 * @return bool
	 */
	public function not_in_list_i($value, $params): bool
	{
		return !$this->in_list_i($value, $params);
	}

	/**
	 * 检查字段值是否存在于数据库中。可设置忽略条件，这在更新记录以忽略自身时非常有用。
	 * @param $value
	 * @param $params
	 * @return bool
	 */
	public function is_unique($value, $params): bool
	{
		list($field, $ignore_field, $ignore_value) = array_pad(explode(',', $params), 3, null);
		sscanf($field, '%[^.].%[^.]', $table, $field);

		$query = "SELECT count(*) FROM {$table} WHERE {$field} = :value";
		$query_params = [':value' => $value];
		if (!empty($ignore_field) && !empty($ignore_value) && !preg_match('/^\{(\w+)\}$/', $ignore_value)) {
			$query .= " AND {$ignore_field} != :ignore_value";
			$query_params[':ignore_value'] = $ignore_value;
		}
		$query .= ' LIMIT 1';

		$count = \venus::$app->database->query($query, $query_params)->fetch_column();
		return (int)$count === 0;
	}

	/**
	 * 如果参数值大于或等于字段的长度，则成功。
	 * @param $value
	 * @param $param
	 * @return bool
	 */
	public function max_length($value, $param): bool
	{
		is_null($value) && $value = '';
		return is_numeric($param) && $param >= mb_strlen($value);
	}

	/**
	 * 如果参数值小于或等于字段的长度，则成功
	 * @param $value
	 * @param $param
	 * @return bool
	 */
	public function min_length($value, $param): bool
	{
		return is_numeric($param) && $param <= mb_strlen($value);
	}

	/**
	 * 当参数指定的字段有一个或多个存在且不为空时，则该字段是必填字段
	 * 当存在任何其他字段时，则该字段是必填字段。
	 *
	 * Example (field is required when the password field is present):
	 *     required_with[password]
	 *
	 * @param $value
	 * @param $params
	 * @param $data
	 * @return bool
	 * @throws \Exception
	 */
	public function required_with($value, $params, $data): bool
	{
		if ($params === null || empty($data)) {
			throw new \Exception('You must supply the parameters: params, data.');
		}

		$present = $this->required($value);
		if ($present) {
			return true;
		}

		$params = explode(',', $params);
		$required_fields = [];
		foreach ($params as $param) {
			if ((array_key_exists($param, $data) && !empty($data[$param]))) {
				$required_fields[] = $param;
			}
		}

		return empty($required_fields);
	}

	/**
	 * 当参数指定的字段有一个或多个不存在或为空时，则该字段是必填字段。
	 * @param $value
	 * @param $params
	 * @param $data
	 * @return bool
	 * @throws \Exception
	 */
	public function required_without($value, $params, $data): bool
	{
		if ($params === null || empty($data)) {
			throw new \Exception('You must supply the parameters: params, data.');
		}

		$present = $this->required($value);
		if ($present) {
			return true;
		}

		$params = explode(',', $params);
		foreach ($params as $param) {
			if (!array_key_exists($param, $data) || empty($data[$param])) {
				return false;
			}
		}

		return true;
	}
}