<?php

namespace venus\view;

class view extends \venus\base\component
{
	protected array $data = [];

	protected array $sections = [];

	protected array $section_stack = [];

	protected ?string $layout = null;

	protected ?string $view_path = null;

	/**
	 * 设置视图路径
	 * @param string|null $view_path 视图路径，末尾不带目录分隔符，如：__DIR__ . '/view';
	 * @return view
	 */
	public function set_view_path(?string $view_path): static
	{
		$this->view_path = $view_path;
		return $this;
	}

	/**
	 * 渲染
	 * @param string $view
	 * @return string
	 * @throws \Exception
	 */
	public function render(string $view): string
	{
		//reader view
		$output = $this->render_view($view);

		//render layout
		if (!is_null($this->layout)) {
			$output = $this->render_view($this->layout);
		}

		$this->data = [];
		$this->layout = null;
		$this->view_path = null;
		return $output;
	}

	/**
	 * 渲染视图
	 * @param string $view
	 * @return string
	 * @throws \Exception
	 */
	protected function render_view(string $view): string
	{
		$view_file = $this->get_view_file($view);
		if (is_null($view_file)) {
			throw new \Exception('View does not exist (' . $view . ')');
		}

		return (function (string $view_file): string {
			$data = $this->data;
			extract($data, EXTR_OVERWRITE);
			ob_start();
			include $view_file;
			return (string)ob_get_clean();
		})($view_file);
	}

	/**
	 * 获取视图文件
	 * @param string $view
	 * @return string|null
	 */
	protected function get_view_file(string $view): ?string
	{
		if(is_null($this->view_path)) {
			$view_path = APP_PATH . 'view';
			$module = \venus::$app->module->get_current_module();
			if (is_array($module) && isset($module['directory'])) {
				$view_path = MODULE_PATH . $module['directory'] . '/view';
			}
		} else {
			$view_path = $this->view_path;
		}
		$view_file = $view_path . '/' . $view . '.php';
		return is_file($view_file) ? $view_file : null;
	}

	/**
	 * 引入视图
	 * @param string $view
	 * @return void
	 * @throws \Exception
	 */
	public function include(string $view): void
	{
		$content = $this->render_view($view);
		echo $content;
	}

	/**
	 * 获取模板变量
	 * @return array
	 */
	public function get_data(): array
	{
		return $this->data;
	}

	/**
	 * 设置模板变量
	 * @param array $data
	 * @return static
	 */
	public function set_data(array $data): static
	{
		$this->data = $data;
		return $this;
	}

	/**
	 * 添加模板变量
	 * @param array $vars
	 * @return static
	 */
	public function set_vars(array $vars): static
	{
		$this->data = array_merge($this->data, $vars);
		return $this;
	}

	/**
	 * extend 设置布局文件
	 * @param string $layout
	 * @return void
	 */
	public function extend(string $layout): void
	{
		$this->layout = $layout;
	}

	/**
	 * section 开始片段
	 * @param string $name
	 * @return void
	 */
	public function section(string $name): void
	{
		$this->section_stack[] = $name;
		ob_start();
	}

	/**
	 * end_section 片段结束
	 * @throws \Exception
	 */
	public function end_section(): void
	{
		$contents = ob_get_clean();

		if ($this->section_stack === []) {
			throw new \Exception('View themes, no current section.');
		}

		$section = array_pop($this->section_stack);
		if (isset($this->sections[$section]) && is_array($this->sections[$section])) {
			$this->sections[$section][] = $contents;
		} else {
			$this->sections[$section] = [$contents];
		}
	}

	/**
	 * render_section 渲染片段
	 * @param string $section_name
	 * @return void
	 */
	public function render_section(string $section_name): void
	{
		if (!isset($this->sections[$section_name])) {
			return;
		}

		foreach ($this->sections[$section_name] as $contents) {
			echo $contents;
		}

		unset($this->sections[$section_name]);
	}
}