<?php

namespace video\config\validation;

use config\validation\validator;
use venus\validation\validator\general;
use venus\validation\validator\upload;
use video\library\config;

class video implements \venus\validation\config
{
	private array $validator_class = [
		general::class,
		upload::class,
		validator::class,
		self::class,
	];

	private string $image_format;
	private float $image_max_size;

	public function get_validator_class(): array
	{
		return $this->validator_class;
	}

	public function get_rules(?string $scene): array
	{
		$image_config = config::get_instance()->get('image');
		$this->image_format = $image_config['format'];
		$this->image_max_size = size2bytes($image_config['max_size'] . $image_config['max_size_unit']);
		$rules = $scene == 'add' ? $this->add() : $this->edit();
		return array_merge($rules, $this->video_ext());
	}

	private function add(): array
	{
		return [
			'video.category_id' => [
				'rules' => 'category_id',
				'errors' => [
					'category_id' => lang('video.invalid_category_id')
				]
			],
			'video.price' => [
				'rules' => 'price',
				'errors' => [
					'price' => lang('video.invalid_price')
				]
			],
			'video.subject' => [
				'rules' => ['required'],
				'errors' => [
					'required' => lang('video.subject_cannot_be_empty'),
				]
			],
			'video.duration' => [
				'rules' => 'duration',
				'errors' => [
					'duration' => lang('video.invalid_duration'),
				]
			],
			'poster' => [
				'rules' => ["ext_in[poster,{$this->image_format}]", "max_size[poster,{$this->image_max_size}]"],
				'errors' => [
					'ext_in' => lang('app.invalid_file_type'),
					'max_size' => lang('app.file_size_error')
				]
			],
			'video_url.player' => [
				'rules' => 'video_player',
				'errors' => [
					'video_player' => lang('video.player_cannot_be_empty')
				]
			],
			'video_url.url' => [
				'rules' => 'video_url',
				'errors' => [
					'video_url' => lang('video.url_cannot_be_empty')
				]
			]
		];
	}

	private function edit(): array
	{
		return array_merge([
			'id' => [
				'rules' => 'id',
				'errors' => [
					'id' => lang('app.invalid_id'),
				]
			]
		], $this->add());
	}

	/**
	 * 自定义字段验证规则
	 * @return array
	 */
	private function video_ext(): array
	{
		$rules = [];
		$video = \phpvod::$app->request->post('video');
		$category_id = $video['category_id'];
		$fields = config::get_instance()->get('field');

		foreach ($fields as $field) {
			if (in_array($category_id, $field['category_id']) && $field['required'] == '1') {
				$label = \phpvod::$app->language->key_is_exists('pv_video_ext.' . $field['field']) ? lang('pv_video_ext.' . $field['field']) : $field['label'];
				$rules['video_ext.' . $field['field']] = [
					'rules' => 'required',
					'errors' => [
						'required' => lang('video.field_cannot_be_empty', [$label])
					]
				];
			}
		}

		return $rules;
	}

	/**
	 * 栏目ID验证规则
	 * @param $value
	 * @return bool
	 */
	public function category_id($value): bool
	{
		$category = config::get_instance()->get('category');
		return isset($category[$value]);
	}

	/**
	 * 价格验证规则
	 * @param $value
	 * @param $data
	 * @return bool
	 */
	public function price($value, $data): bool
	{
		if (array_key_exists('video.kind', $data) && $data['video.kind'] === '2') {
			return is_numeric($value) && floatval($value) >= 0.01;
		}
		return true;
	}

	/**
	 * 时长验证规则
	 * @param $value
	 * @return bool
	 */
	public function duration($value): bool
	{
		if (trim($value) == '')
			return true;

		return (bool)preg_match('/^\d+:\d+:\d+$/', $value);
	}

	/**
	 * 播放器验证规则
	 * @param $value
	 * @param $data
	 * @param $original_data
	 * @return bool
	 */
	public function video_player($value, $data, $original_data): bool
	{
		return is_array($value) && count($value) === count($original_data['video_url']['url']);
	}

	/**
	 * 视频地址验证规则
	 * @param $value
	 * @return bool
	 */
	public function video_url($value): bool
	{
		if (is_array($value)) {
			$array = array_filter($value, fn($v) => !is_blank($v));
			return count($array) > 0;
		}
		return false;
	}
}