<?php

namespace video\controller\admin;

use config\validation\validator;
use phpvod\base\backend_controller;
use phpvod\event\event;
use phpvod\widget\form\admin_form;
use video\extension\upload\mars_transcoder\mars_transcoder;
use video\library\config;
use video\service\mars;

class video extends backend_controller
{
	protected array $helper = ['form'];
	private \video\model\video $video_model;
	private \video\model\video_file $video_file_model;
	private \video\service\video $video_service;
	private \video\service\file $file_service;

	/** @noinspection PhpFieldAssignmentTypeMismatchInspection */
	public function init(): void
	{
		parent::init();
		$this->video_model = model(\video\model\video::class);
		$this->video_file_model = model(\video\model\video_file::class);
		$this->video_service = service(\video\service\video::class);
		$this->file_service = service(\video\service\file::class);
	}

	public function index(): string
	{
		return $this->view('video/index');
	}

	/**
	 * 获取视频列表数据
	 * @return void
	 * @throws \Exception
	 */
	public function get_data(): void
	{
		$params = $this->get_bootstrap_table_query_params();
		if ($params !== false) {
			$data = $this->video_model->get_data($params);
			$this->response->set_json($data);
		}
	}

	/**
	 * 更改状态(上架与下架)(ajax)
	 * @return void
	 * @throws \Exception
	 */
	public function change_status(): void
	{
		$id = $this->request->post('id');
		parent::change_status_by_id([$this->video_model, 'change_status'], $id, function () use ($id) {
			//event
			event()->trigger(event::EVENT_UPDATE_FROM_MODULE, 'video', $id);
		});
	}

	/**
	 * 更改推荐(ajax)
	 * @return void
	 * @throws \Exception
	 */
	public function change_recommend(): void
	{
		$id = $this->request->post('id');
		parent::change_status_by_id([$this->video_model, 'change_recommend'], $id, function () use ($id) {
			//event
			event()->trigger(event::EVENT_UPDATE_FROM_MODULE, 'video', $id);
		});
	}

	/**
	 * 导入采集数据
	 * @return void
	 * @throws \Exception
	 */
	public function collect(): void
	{
		if (!$this->request->is_post()) {
			$this->response->set_json(['id' => 0, 'msg' => 'invalid_request']);
			return;
		}

		$this->response->set_json($this->video_service->collect($this->request->post()));
	}

	/**
	 * 新增视频
	 * @return string
	 * @throws \Exception
	 */
	public function add(): string
	{
		if ($this->request->is_post()) {
			//cleanup
			$this->file_service->cleanup();
			if ($this->validation->load_config(\video\config\validation\video::class, 'add')->with_post()->run()) {
				if ($this->video_service->add($this->request->post()) !== false) {
					$this->module_redirect('video/index', false, true);
				} else {
					$this->module_redirect('video/add', true, false);
				}
			} else {
				$this->set_message_from_validator();
				$this->module_redirect('video/add', true);
			}
		}
		return $this->view('video/add');
	}

	/**
	 * 编辑视频
	 * @return string
	 * @throws \Exception
	 */
	public function edit(): string
	{
		if ($this->request->is_post()) {
			//cleanup
			$this->file_service->cleanup();
			$data = $this->request->post();
			if ($this->validation->load_config(\video\config\validation\video::class, 'edit')->with_post()->run()) {
				if ($this->video_service->edit($data)) {
					$this->module_redirect('video/index', false, true);
				} else {
					$this->redirect(admin_module_uri('video/edit') . '?id=' . $data['id'], true);
				}
			} else {
				$this->set_message_from_validator();
				$this->redirect(admin_module_uri('video/edit') . '?id=' . $data['id'], true);
			}
		}

		$id = $this->request->get('id');
		$data = $this->video_service->get_edit_data($id);
		if (is_null($data)) {
			$this->module_redirect('video/index', false, lang('app.invalid_id'));
		}

		return $this->view('video/edit', $data);
	}

	/**
	 * 删除视频(ajax)
	 * @return void
	 * @throws \Exception
	 */
	public function delete(): void
	{
		$id = $this->request->post('id');
		if ($this->validation->check($id, 'id', validator::class)) {
			$this->video_service->delete($id);
			$this->response->set_json(['id' => 1, 'msg' => lang('app.operation_succeeded')]);
		} else {
			$this->response->set_json(['id' => 0, 'msg' => lang('app.invalid_id')]);
		}
	}

	/**
	 * 根据栏目ID返回自定义字段表单(ajax)
	 * @return void
	 */
	public function get_field_input(): void
	{
		$category_id = $this->request->get('category_id');
		$video_ext = $this->request->get('video_ext'); //default value (array)
		$fields = config::get_instance()->get('field');
		$admin_form = admin_form::get_instance();
		$template = '
		<div class="row mb-4">
			<label class="col-md-2 col-form-label text-md-end"{for}>{label}</label>
			<div class="col-md-6">{input}</div>
		</div>
		';

		if (is_array($fields)) {
			$html = '';
			foreach ($fields as $field) {
				if (!in_array($category_id, $field['category_id'])) {
					continue;
				}

				$for = '';
				if ($field['input_type'] == 'text') {
					$input = $admin_form->input('video_ext[' . $field['field'] . ']', 'text', default_value: $video_ext[$field['field']] ?? '', options: ['id' => $field['field'], 'class' => 'form-control']);
					$for = ' for="' . $field['field'] . '"';
				} elseif ($field['input_type'] == 'textlist') {
					$input = $admin_form->input('video_ext[' . $field['field'] . ']', 'text', default_value: $video_ext[$field['field']] ?? '', options: ['id' => $field['field'], 'class' => 'form-control tags_input']);
					$for = ' for="' . $field['field'] . '"';
				} elseif ($field['input_type'] == 'textarea') {
					$input = $admin_form->textarea('video_ext[' . $field['field'] . ']', default_value: $video_ext[$field['field']] ?? '', options: ['id' => $field['field'], 'class' => 'form-control', 'rows' => '3']);
					$for = ' for="' . $field['field'] . '"';
				} elseif ($field['input_type'] == 'radio') {
					$values = array_column($field['items'], 'label', 'value');
					$input = $admin_form->radio_group('video_ext[' . $field['field'] . ']', $values, checked_value: $video_ext[$field['field']] ?? array_key_first($values), options: ['class' => 'form-check-input']);
				} elseif ($field['input_type'] == 'checkbox') {

					if (isset($video_ext[$field['field']]) && is_string($video_ext[$field['field']]) && $video_ext[$field['field']] != '') {
						$checked_values = explode(',', $video_ext[$field['field']]);
						$default_value = $video_ext[$field['field']];
					} else {
						$checked_values = null;
						$default_value = '';
					}

					$values = array_column($field['items'], 'label', 'value');
					$input = $admin_form->checkbox_group($field['field'] . '[]', $values, checked_values: $checked_values, options: ['class' => 'form-check-input', 'data-associated-id' => $field['field']]);
					$input .= $admin_form->input('video_ext[' . $field['field'] . ']', 'hidden', $default_value, ['class' => 'form-control', 'id' => $field['field']]);

				} elseif ($field['input_type'] == 'select') {
					$values = array_column($field['items'], 'label', 'value');
					$input = $admin_form->select('video_ext[' . $field['field'] . ']', $values, selected_value: $video_ext[$field['field']] ?? null, options: ['class' => 'form-select']);
				}

				if (!empty($input)) {
					$label = \phpvod::$app->language->key_is_exists('pv_video_ext.' . $field['field']) ? lang('pv_video_ext.' . $field['field']) : $field['label'];
					$html .= strtr($template, ['{label}' => $label, '{input}' => $input, '{for}' => $for]);
				}
			}

			$this->response->set_body($html);
		}

	}

	/**
	 * 获取已上传的文件信息(ajax)
	 * @return void
	 * @throws \Exception
	 */
	public function get_uploaded_info(): void
	{
		$post = $this->request->post();
		$data = $this->video_file_model->get_uploaded_info($post['vid'] ?? null, $post['hash_set'] ?? null);
		$this->response->set_json($data);
	}

	/**
	 * 删除文件(ajax)
	 * @return void
	 * @throws \Exception
	 */
	public function delete_file(): void
	{
		$hash = $this->request->post('hash');
		$data = $this->file_service->delete_by_hash($hash);
		$this->response->set_json($data);
	}

	/**
	 * 添加转码任务(ajax)
	 * @return void
	 * @throws \Exception
	 */
	public function add_trans_task(): void
	{
		$post_name = 'uploaded_file';
		$server_id = $this->request->post('server_id');
		$mars_service = mars::get_instance_by_server_id($server_id);
		if (!is_null($mars_service)) {
			//passback
			$params = ['passback' => "server_id=$server_id"];
			//tasks
			$tasks = $this->request->post('tasks');
			if (is_string($tasks) && trim($tasks) != '') {
				$params['tasks'] = $tasks;
			}
			//priority
			$priority = $this->request->post('priority');
			if (is_numeric($priority)) {
				$params['priority'] = (int)$priority;
			}
			//add_task
			$added_task = $mars_service->add_task($post_name, $params);
			$data = mars_transcoder::success_callback($server_id, $added_task);
			$this->response->set_json(['id' => 1, 'msg' => lang('app.operation_succeeded'), 'data' => $data]);
		} else {
			$this->response->set_json(['id' => 0, 'msg' => lang('app.invalid_id')]);
		}
	}
}