<?php

namespace mars\sdk;
class mars_client
{
	private string $ip;
	private int $port;
	private string $key;
	private string $notify_url;

	public function __construct(string $ip, int $port, string $key, string $notify_url = '')
	{
		$this->ip = $ip;
		$this->port = $port;
		$this->key = $key;
		$this->notify_url = $notify_url;
		return $this;
	}

	/**
	 * 获取网关
	 * @return string
	 */
	public function get_notify_url(): string
	{
		return $this->notify_url;
	}

	/**
	 * 设置网关
	 * @param string $notify_url
	 * @return mars_client
	 */
	public function set_notify_url(string $notify_url): static
	{
		$this->notify_url = $notify_url;
		return $this;
	}

	/**
	 * 测试client->server连接
	 * @return false|string
	 * @throws \Exception
	 */
	public function test_client_conn(): false|string
	{
		$message = $this->build_message(array('command' => 'testClientConn'));
		return $this->send_message($message);
	}

	/**
	 * 测试server->client连接
	 * @param string $notify_url
	 * @param string $passback
	 * @return false|string
	 * @throws \Exception
	 */
	public function test_server_conn(string $notify_url, string $passback = ''): false|string
	{
		$message = $this->build_message(array(
			'command' => 'testServerConn',
			'params' => array(
				'notify_url' => $notify_url,
				'passback' => $passback
			)
		));
		return $this->send_message($message);
	}

	/**
	 * 获取版本信息
	 * @return false|string
	 * @throws \Exception
	 */
	public function get_version(): false|string
	{
		$message = $this->build_message(array('command' => 'getVersion'));
		return $this->send_message($message);
	}

	/**
	 * 获取转码任务标签
	 * @return false|string
	 * @throws \Exception
	 */
	public function get_tasks(): false|string
	{
		$message = $this->build_message(array('command' => 'getTasks'));
		return $this->send_message($message);
	}

	/**
	 * 重新载入配置
	 * @return false|string
	 * @throws \Exception
	 */
	public function reload(): false|string
	{
		$message = $this->build_message(array('command' => 'reload'));
		return $this->send_message($message);
	}

	/**
	 * 获取线程信息
	 * @return false|string
	 * @throws \Exception
	 */
	public function get_thread_info(): false|string
	{
		$message = $this->build_message(array('command' => 'getThreadInfo'));
		return $this->send_message($message);
	}

	/**
	 * 获取硬件信息
	 * @return false|string
	 * @throws \Exception
	 */
	public function get_hardware_info(): false|string
	{
		$message = $this->build_message(array('command' => 'getHardwareInfo'));
		return $this->send_message($message);
	}

	/**
	 * 获取所有信息（线程信息+硬件信息）
	 * @return false|string
	 * @throws \Exception
	 */
	public function get_all_info(): false|string
	{
		$message = $this->build_message(array('command' => 'getAllInfo'));
		return $this->send_message($message);
	}

	/**
	 * 提交转码任务
	 * @param string $file 视频文件地址
	 * @param string $subtitle 字幕文件地址
	 * @param string $tasks 转码执行的任务标签，多个标签之间用半角逗号分开
	 * @param int $priority 优先级(0-9)
	 * @param string $passback 回传参数，如:a=1&b=2
	 * @return false|string
	 * @throws \Exception
	 */
	public function add_task(string $file, string $subtitle = '', string $tasks = '', int $priority = 0, string $passback = ''): false|string
	{
		$message = $this->build_message(array(
			'command' => 'addTransTask',
			'params' => array(
				'file' => $file,
				'subtitle' => $subtitle,
				'tasks' => $tasks,
				'notify_url' => $this->notify_url,
				'passback' => $passback,
				'priority' => $priority
			)
		));
		return $this->send_message($message);
	}

	/**
	 * 提交转码任务
	 * @param string $video_token 视频文件token
	 * @param string $subtitle_token 字幕文件token
	 * @param string $tasks 转码执行的任务标签，多个标签之间用半角逗号分开
	 * @param int $priority 优先级(0-9)
	 * @param string $passback 回传参数，如:a=1&b=2
	 * @return false|string
	 * @throws \Exception
	 */
	public function add_task_by_token(string $video_token, string $subtitle_token = '', string $tasks = '', int $priority = 0, string $passback = ''): false|string
	{
		$file = aes::decrypt($video_token);
		$subtitle = !empty($subtitle_token) ? aes::decrypt($subtitle_token) : '';
		return $this->add_task($file, $subtitle, $tasks, $priority, $passback);
	}

	/**
	 * 激活转码任务
	 * @param int $tid 转码任务ID
	 * @return false|string
	 * @throws \Exception
	 */
	public function active_task(int $tid): false|string
	{
		$message = $this->build_message(array(
			'command' => 'activeTransTask',
			'params' => array(
				'tid' => $tid
			)
		));
		return $this->send_message($message);
	}

	/**
	 * 删除转码任务
	 * @param int $tid 转码任务ID
	 * @return false|string
	 * @throws \Exception
	 */
	public function del_task(int $tid): false|string
	{
		$message = $this->build_message(array(
			'command' => 'delTransTask',
			'params' => array(
				'tid' => $tid
			)
		));
		return $this->send_message($message);
	}

	/**
	 * 构建消息
	 * @param array $arr
	 * @return string
	 * @throws \Exception
	 */
	private function build_message(array $arr): string
	{
		if (!isset($arr['params']['key'])) {
			$arr['params']['key'] = $this->key;
		}
		return $this->json_encode($arr) . "\n";
	}

	/**
	 * 发送信息
	 * @param string $message 消息字符串
	 * @return false|string false表示发送任务失败，其它由SocketServer返回
	 */
	private function send_message(string $message): false|string
	{
		$r = false;
		$socket = false;
		try {
			$socket = socket_create(AF_INET, SOCK_STREAM, SOL_TCP);
			if ($socket) {
				$conn = @socket_connect($socket, $this->ip, $this->port);
				if ($conn) {
					socket_write($socket, $message, strlen($message));
					$response = socket_read($socket, 1024);
					$r = is_string($response) ? trim($response) : false;
				}
			}
		} catch (\Exception) {
		}
		if ($socket) {
			socket_close($socket);
		}
		return $r;
	}

	/**
	 * json编码
	 * @param mixed $data
	 * @return string
	 * @throws \Exception
	 */
	private function json_encode(mixed $data): string
	{
		$result = json_encode($data, JSON_UNESCAPED_UNICODE | JSON_UNESCAPED_SLASHES | JSON_PARTIAL_OUTPUT_ON_ERROR);

		if (!in_array(json_last_error(), [JSON_ERROR_NONE, JSON_ERROR_RECURSION], true)) {
			throw new \Exception(json_last_error_msg(), json_last_error());
		}

		return $result;
	}
}
