<?php

namespace video\library;

use phpvod\library\config\abstract_config;
use video\model\category;
use video\model\field;
use video\model\mars_map;
use video\model\mars_server;
use video\model\player;
use video\model\slide;

class config extends abstract_config
{
	public static ?config $instance = null;

	protected array $method_map = [
		'category' => ['category_cache', 'category'],
		'field' => ['field_cache', 'field'],
		'player' => ['player_cache', 'player'],
		'slide' => ['slide_cache', 'slide'],
		'config' => ['config_cache', 'config', 'video.config'],
		'image' => ['config_cache', 'image', 'video.image'],
		'upload' => ['config_cache', 'upload', 'video.upload'],
		'cache' => ['config_cache', 'cache', 'video.cache'],
		'mars_server' => ['mars_server', 'mars_server'],
		'mars_map' => ['mars_map', 'mars_map'],
		'mars_tag' => ['config_cache', 'mars_tag', 'video.mars.tag']
	];

	protected string $config_path = WRITABLE_PATH . 'cache/config/video';

	private function __construct()
	{
	}

	private function __clone()
	{
	}

	public static function get_instance(): ?config
	{
		if (is_null(self::$instance)) {
			self::$instance = new config();
		}
		return self::$instance;
	}

	/**
	 * 创建配置缓存
	 * @param string $file
	 * @param string $type
	 * @return void
	 * @throws \Exception
	 */
	public function config_cache(string $file, string $type): void
	{
		/**
		 * @var \phpvod\model\config $model
		 */
		$model = model(\phpvod\model\config::class);
		$data = $model->get_config($type);
		write_file($this->get_file_path($file), "<?php\nreturn " . var_export($data, true) . ";");
	}

	/**
	 * 栏目缓存
	 * @param string $file
	 * @return void
	 * @throws \Exception
	 */
	public function category_cache(string $file): void
	{
		/**
		 * @var category $model
		 */
		$model = model(category::class);
		$data = $model->order_by('sort DESC, id ASC')->query()->fetch_all_by_callback(function (array $row) {
			return ['key' => $row['id'], 'value' => $row];
		});
		write_file($this->get_file_path($file), "<?php\nreturn " . var_export($data, true) . ";");
	}

	/**
	 * 自定义字段缓存
	 * @param string $file
	 * @return void
	 * @throws \Exception
	 */
	public function field_cache(string $file): void
	{
		/**
		 * @var field $model
		 */
		$model = model(field::class);
		$data = $model->order_by('id ASC')->query()->fetch_all_by_callback(function (array $row) {
			$row['items'] = is_null($row['items']) ? [] : json_decode($row['items'], true);
			$row['category_id'] = is_null($row['category_id']) ? [] : explode(',', $row['category_id']);
			return ['key' => $row['field'], 'value' => $row];
		});
		write_file($this->get_file_path($file), "<?php\nreturn " . var_export($data, true) . ";");
	}

	/**
	 * 播放器缓存
	 * @param string $file
	 * @return void
	 * @throws \Exception
	 */
	public function player_cache(string $file): void
	{
		/**
		 * @var player $model
		 */
		$model = model(player::class);
		$data = $model->order_by('id ASC')->query()->fetch_all_by_callback(function (array $row) {
			if ($row['status'] == '0') {
				return null;
			}
			unset($row['status']);
			return ['key' => $row['name'], 'value' => $row];
		});
		write_file($this->get_file_path($file), "<?php\nreturn " . var_export($data, true) . ";");
	}

	/**
	 * 幻灯缓存
	 * @param string $file
	 * @return void
	 * @throws \Exception
	 */
	public function slide_cache(string $file): void
	{
		/**
		 * @var slide $model
		 */
		$model = model(slide::class);
		$data = $model->order_by('sort DESC, id ASC')->query()->fetch_all_by_callback(function (array $row) {
			return ['key' => $row['id'], 'value' => $row];
		});
		write_file($this->get_file_path($file), "<?php\nreturn " . var_export($data, true) . ";");
	}

	/**
	 * 转码服务器缓存
	 * @param string $file
	 * @return void
	 * @throws \Exception
	 */
	public function mars_server(string $file): void
	{
		/**
		 * @var mars_server $model ;
		 */
		$model = model(mars_server::class);
		$data = $model->order_by('id ASC')->query()->fetch_all_by_callback(function (array $row) {
			return ['key' => $row['id'], 'value' => $row];
		});
		write_file($this->get_file_path($file), "<?php\nreturn " . var_export($data, true) . ";");
	}

	/**
	 * 域名映射缓存
	 * @param string $file
	 * @return void
	 * @throws \Exception
	 */
	public function mars_map(string $file): void
	{
		/**
		 * @var mars_map $model ;
		 */
		$model = model(mars_map::class);
		$data = $model->order_by('id ASC')->query()->fetch_all_by_callback(function (array $row) {
			if (!empty($row['server_id'])) {
				return ['key' => $row['domain'], 'value' => $row['server_id']];
			}
			return null;
		});
		write_file($this->get_file_path($file), "<?php\nreturn " . var_export($data, true) . ";");
	}
}
