<?php

namespace video\model;

use phpvod\widget\form\admin_form;

class player extends \phpvod\base\model
{
	protected string $table = 'pv_video_player';
	protected ?string $primary_key = 'id';
	protected array $allowed_fields = ['name', 'directory', 'class', 'description', 'status'];

	/**
	 * 获取播放器数据
	 * @param array $params 查询参数，包含search/order_by/offset_limit元素(非null即可用)
	 * @return array
	 */
	public function get_data(array $params): array
	{
		//where
		if (!is_null($params['search'])) {
			$this->where('name LIKE :name OR directory LIKE :directory', [
				'name' => '%' . $params['search'] . '%',
				'directory' => '%' . $params['search'] . '%'
			]);
		}

		$total = (int)$this->select('count(id)')->query(false)->fetch_column();

		//order by
		if (!is_null($params['order_by'])) {
			$this->order_by($params['order_by']);
		}

		//offset_limit
		if (!is_null($params['offset_limit'])) {
			$this->limit($params['offset_limit']);
		}

		$rows = $this->select('*')->query()->fetch_all_by_callback(function (array $row) {
			$row['status'] = admin_form::get_instance()->switch((boolean)$row['status'], ['class' => 'form-check-input', 'role' => 'switch']);
			return $row;
		});
		return ['total' => $total, 'rows' => $rows];
	}

	/**
	 * 安装播放器
	 * @param string $zip_file
	 * @return void
	 * @throws \Exception
	 */
	public function install(string $zip_file): void
	{
		// check data
		$data = get_zip_config($zip_file);
		if (!$this->check_config($data)) {
			throw new \Exception(lang('admin.config_format_error'));
		}

		$module_directory = \phpvod::$app->module->get_module('video', 'directory');
		if (is_null($module_directory)) {
			throw new \Exception(lang('admin.invalid_module_directory'));
		}

		$install_directory = MODULE_PATH . $module_directory . '/extension/player';

		// is_dir
		if (!is_dir($install_directory)) {
			throw new \Exception(lang('admin.player_path_not_exists'));
		}

		// is_writable
		if (!is_writable($install_directory)) {
			throw new \Exception(lang('admin.player_path_not_writable'));
		}

		// check if player exists
		$count = (int)$this->select('count(name)')->where([
			'name' => $data['name'],
			'directory' => $data['directory'],
			'class' => $data['class']
		], 'OR')->query()->fetch_column();
		if ($count > 0) {
			throw new \Exception(lang('admin.player_exists'));
		}

		// unzip
		if (!unzip($zip_file, $install_directory)) {
			throw new \Exception(lang('admin.player_unzip_failed'));
		}

		// install
		if ($this->insert($data) <= 0) {
			throw new \Exception(lang('admin.player_install_failed'));
		}
	}

	/**
	 * 卸载播放器
	 * @param int $id
	 * @return void
	 * @throws \Exception
	 */
	public function uninstall(int $id): void
	{
		$player = $this->select('*')->find($id)->fetch_array();
		if (!is_array($player)) {
			throw new \Exception(lang('app.invalid_id'));
		}

		//delete
		$this->delete($id);

		//delete files
		$module_directory = \phpvod::$app->module->get_module('video', 'directory');
		if (is_null($module_directory)) {
			throw new \Exception(lang('admin.invalid_module_directory'));
		}

		$player_directory = MODULE_PATH . $module_directory . '/extension/player/' . $player['directory'];
		if (!is_blank($player['directory']) &&
			is_dir($player_directory) &&
			@remove_directory($player_directory) !== true
		) {
			throw new \Exception(lang('admin.player_files_delete_failed', [$player_directory]), 2); //code:2
		}
	}

	/**
	 * 检测配置数据格式是否正确
	 * @param array|null $data
	 * @return bool
	 */
	private function check_config(?array $data): bool
	{
		if (is_array($data) &&
			isset($data['name'], $data['directory'], $data['class'], $data['description'], $data['status']) &&
			!is_blank($data['name']) && !is_blank($data['directory']) && !is_blank($data['class']) && in_array($data['status'], ['0', '1'])
		) {
			return true;
		}
		return false;
	}

	/**
	 * 更新播放器状态
	 * @param int $id 播放器ID
	 * @return int|false 成功返回更新后的状态，失败返回false
	 * @throws \Exception
	 */
	public function change_status(int $id): int|false
	{
		$this->update(['status' => 'CASE status WHEN 0 THEN 1 ELSE 0 END'], $id, ['status']);
		$row = $this->select("status")->find($id)->fetch_array();
		return is_array($row) ? (int)$row['status'] : false;
	}
}