<?php

namespace video\service;

use mars\sdk\mars_client;
use mars\sdk\mars_helper;
use venus\autoloader\autoloader;
use venus\di\container;
use video\library\config;

class mars
{
	public static array $config = [];
	private static array $instance = [];
	private mars_client $mars_client;
	private mars_helper $mars_helper;

	private function __construct(string $ip, int $port, string $key, string $notify_url)
	{
		//SDK namespace
		/**
		 * @var autoloader $autoloader
		 */
		$autoloader = container::get_exists_instance(autoloader::class);
		$autoloader->add_psr4('mars', dirname(__DIR__, 1) . '/extension/upload/mars_transcoder/mars/');

		$this->mars_client = new mars_client($ip, $port, $key, $notify_url);
		$this->mars_helper = new mars_helper($this->mars_client);
	}

	/**
	 * 通过服务器ID获取实例
	 * @param int $server_id
	 * @return mars|null
	 */
	public static function get_instance_by_server_id(int $server_id): ?mars
	{
		if (!isset(self::$instance[$server_id])) {
			$config = config::get_instance()->get('mars_server', $server_id);
			if (is_array($config) && isset($config['ip'], $config['port'], $config['secret_key'], $config['notify_url'])) {
				self::$instance[$server_id] = new mars($config['ip'], $config['port'], $config['secret_key'], $config['notify_url']);
				self::$config = $config;
			}
		}
		return self::$instance[$server_id] ?? null;
	}

	/**
	 * 连接测试
	 * @param string $passback
	 * @return array 数组包含c2s与s2c二个元素，分别表示客户端至服务器端连接与服务器端至客户端连接。
	 * 元素值如果为null，表示未进行连接测试；值为false，表示连接失败；值为array表示返回的测试结果
	 * @throws \Exception
	 */
	public function test_conn(string $passback): array
	{
		/** @noinspection PhpArrayIndexImmediatelyRewrittenInspection */
		$data = [
			'c2s' => null,
			's2c' => null
		];

		//Test the client-to-server connection
		$c2s_response = $this->mars_client->test_client_conn();
		if (is_string($c2s_response)) {
			$data['c2s'] = json_decode($c2s_response, true);
			if (isset($data['c2s']['code']) && $data['c2s']['code'] == '1') {
				//Test the server-to-client connection
				$notify_url = $this->mars_client->get_notify_url();
				if (!empty($notify_url)) {
					$s2c_response = $this->mars_client->test_server_conn($notify_url, $passback);
					$data['s2c'] = is_string($s2c_response) ? json_decode($s2c_response, true) : false;
				}
			}
		} else {
			$data['c2s'] = false;
		}

		return $data;
	}

	/**
	 * 添加转码任务
	 * @param string $post_name 保存文件上传信息的$_POST元素名称
	 * @param array $params 添加转码任务时的额外参数，['tasks' => '', 'priority' => 0]
	 * @return array 已成功添加的转码任务（一个或多个），包含 tid/truename/duration 元素
	 * @throws \Exception
	 */
	public function add_task(string $post_name, array $params = []): array
	{
		$file_data = $this->mars_helper->get_uploaded_file_data($post_name);
		return $this->mars_helper->add_task($file_data, $params);
	}

	/**
	 * 激活转码任务
	 * @param int $tid 转码任务ID
	 * @return bool
	 * @throws \Exception
	 */
	public function active_task(int $tid): bool
	{
		$response = $this->mars_client->active_task($tid); //active task
		if (is_string($response)) {
			$data = json_decode($response, true);
			if ($data['code'] == '1') {
				return true;
			}
		}
		return false;
	}

	/**
	 * 删除转码任务
	 * @param int $tid 转码任务ID
	 * @return false|string
	 * @throws \Exception
	 */
	public function del_task(int $tid): false|string
	{
		return $this->mars_client->del_task($tid);
	}

	/**
	 * 获取转码任务标签
	 * @return false|string
	 * @throws \Exception
	 */
	public function get_tasks(): false|string
	{
		return $this->mars_client->get_tasks();
	}

	/**
	 * 根据权重获取服务器ID (该方法按实际需求进行算法调整)
	 * @return ?int 成功返回服务器id，失败返回null
	 */
	public static function get_server_id(): ?int
	{
		//mars_server
		$mars_server = config::get_instance()->get('mars_server');

		//mars_map
		$mars_map = config::get_instance()->get('mars_map');
		if (!empty($mars_map)) {
			$http_host = \phpvod::$app->request->server('HTTP_HOST');
			if (is_string($http_host) && !empty($mars_map[$http_host])) {
				$id = explode(',', $mars_map[$http_host]);
				$mars_server = array_intersect_key($mars_server, array_flip($id));
				// 计算总权重
				$total_weight = array_sum(array_column($mars_server, 'weight'));
				if ($total_weight === 0) {
					return array_rand($mars_server);
				}
			}
		}

		$server_id = null;

		// 计算总权重
		$total_weight = array_sum(array_column($mars_server, 'weight'));
		if ($total_weight >= 1) {
			// 根据权重选择服务器
			$random_value = mt_rand(1, $total_weight);
			foreach ($mars_server as $key => $value) {
				$random_value -= $value['weight'];
				if ($random_value <= 0) {
					$server_id = $key;
					break;
				}
			}
		}

		return $server_id;
	}

	/**
	 * 根据名称获取数据
	 * @param string|array $names 名称（play、preview、clips、poster、player_poster、sprite、dynamic、static）
	 * @param array $remote_data 转码服务器返回的数据
	 * @return array|null
	 */
	public static function get_url_by_name(string|array $names, array $remote_data): array|null
	{
		$url = [];
		$mars_tag = config::get_instance()->get('mars_tag');

		is_string($names) && $names = [$names];
		foreach ($names as $name) {
			if (isset($mars_tag[$name]) && !empty($mars_tag[$name])) {
				$tags = explode(',', $mars_tag[$name]);
				foreach ($tags as $tag) {
					if(!str_contains($tag, '.')) { //tag format like: hls
						if(!isset($remote_data[$tag]) || !is_array($remote_data[$tag])) {
							continue;
						}
						$url[$name][] = array_map(function ($value) {
							$urls = array_column($value, 'url');
							return count($urls) === 1 ? $urls[0] : $urls;
						}, $remote_data[$tag]);
					} else {
						[$k1, $k2] = explode('.', $tag, 2); //tag format like: image.sprite
						if(!isset($remote_data[$k1][$k2]) || !is_array($remote_data[$k1][$k2])) {
							continue;
						}
						$urls = array_column($remote_data[$k1][$k2], 'url');
						$url[$name] = isset($url[$name]) ? array_merge($url[$name], $urls) : $urls;
					}
				}
				if(isset($url[$name]) && count($url[$name]) === 1) {
					$url[$name] = $url[$name][0];
				}
			}
		}

		return count($url) > 0 ? $url : null;
	}
}