<?php

namespace video\service;

use phpvod\event\event;
use venus\database\database;

class video
{
	private database $db;
	private \video\model\video $video_model;
	private \video\model\video_ext $video_ext_model;
	private \video\model\video_data $video_data_model;
	private \video\model\video_url $video_url_model;
	private \video\model\video_file $video_file_model;
	private \video\model\video_collect $video_collect_model;
	private poster $poster_service;
	private file $file_service;

	/** @noinspection PhpFieldAssignmentTypeMismatchInspection */
	public function __construct()
	{
		$this->db = \phpvod::$app->database;
		$this->video_model = model(\video\model\video::class);
		$this->video_ext_model = model(\video\model\video_ext::class);
		$this->video_data_model = model(\video\model\video_data::class);
		$this->video_url_model = model(\video\model\video_url::class);
		$this->video_file_model = model(\video\model\video_file::class);
		$this->video_collect_model = model(\video\model\video_collect::class);
		//service
		$this->poster_service = service(poster::class);
		$this->file_service = service(file::class);
	}

	/**
	 * 导入采集数据
	 * @param array $data
	 * @return array
	 * @throws \Exception
	 */
	public function collect(array $data): array
	{
		if (!isset($data['video_collect']['source_flag']) || is_blank($data['video_collect']['source_flag']) ||
			!isset($data['video_collect']['update_flag']) || is_blank($data['video_collect']['update_flag'])
		) {
			return ['id' => 0, 'msg' => 'invalid_params'];
		}

		$row = $this->video_collect_model->where(['source_flag' => $data['video_collect']['source_flag']])->query()->fetch_array();
		if ($row === false) { //未曾导入过
			$id = $this->add($data);
			if (is_int($id)) { //导入成功
				$this->video_collect_model->insert([
					'id' => $id,
					'source_flag' => $data['video_collect']['source_flag'],
					'update_flag' => $data['video_collect']['update_flag']
				]);
				return ['id' => 1, 'msg' => 'add_succeeded'];
			} else { //导入失败
				return ['id' => 0, 'msg' => 'add_failed'];
			}
		} else { //导入过
			if ($row['update_flag'] == $data['video_collect']['update_flag']) { //数据是否更新
				return ['id' => 1, 'msg' => 'no_need_to_update'];
			}
			$data['id'] = $row['id'];
			if ($this->edit($data)) { //更新成功
				$this->video_collect_model->update(['update_flag' => $data['video_collect']['update_flag']], $row['id']);
				return ['id' => 1, 'msg' => 'update_succeeded'];
			} else { //更新失败
				return ['id' => 0, 'msg' => 'update_failed'];
			}
		}
	}

	/**
	 * 新增视频
	 * @param array $data post数据
	 * @return int|false 成功返回视频ID,失败返回false
	 * @throws \Exception
	 */
	public function add(array $data): int|false
	{
		$this->db->begin_transaction();

		try {
			//author_id/author/author_identity
			$data['video']['author_id'] = \phpvod::$app->user_auth->get_backend_identity('id');
			$data['video']['author'] = \phpvod::$app->user_auth->get_backend_identity('username');
			$data['video']['author_identity'] = 1;

			//price
			$data['video']['price'] = $data['video']['kind'] === '2' ? floor(floatval($data['video']['price']) * 100) : null;

			//duration
			$duration = !empty($data['video']['duration']) ? duration2seconds($data['video']['duration']) : 0;
			$data['video']['duration'] = $duration > 0 ? $duration : null;

			//recommend
			$data['video']['recommend'] = $data['video']['recommend'] ?? 0;

			//status
			$data['video']['status'] = $data['video']['status'] ?? 0;

			$vid = 0;
			$affected_rows = $this->video_model->insert($data['video']);
			if ($affected_rows == 1) {
				$vid = (int)$this->video_model->insert_id();

				//video_ext
				$data['video_ext']['id'] = $vid;
				$this->video_ext_model->insert($data['video_ext']);

				//video_data
				!isset($data['video_data']['content']) && $data['video_data']['content'] = '';
				$data['video_data']['id'] = $vid;
				$this->video_data_model->insert($data['video_data']);

				//video_url
				if (isset($data['video_url']['url']) && is_array($data['video_url']['url'])) {
					foreach ($data['video_url']['url'] as $key => $value) {
						$value = trim($value);
						if ($value != '') {
							$this->video_url_model->insert([
								'vid' => $vid,
								'player' => $data['video_url']['player'][$key],
								'url' => $value,
								'group_name' => $data['video_url']['group_name'][$key]
							]);
						}
					}
				}

				//video_file
				if (isset($data['hash_set'])) {
					$this->video_file_model->bind_vid($data['hash_set'], $vid);
					$this->file_service->active_task_by_hash_set($data['hash_set']);
				}

				//update poster
				if ($data['poster']['from'] == '1') {
					$this->poster_service->update_file($vid, \phpvod::$app->request->get_files('poster')['file']);
				} else {
					$this->poster_service->update_url($vid, $data['poster']['url']);
				}
			}
		} catch (\Exception) {
			$this->db->roll_back();
			return false;
		}

		if ($this->db->commit()) {
			//event
			$vid > 0 && event()->trigger(event::EVENT_INSERT_FROM_MODULE, 'video', $vid);
			return $vid;
		} else {
			return false;
		}
	}

	/**
	 * 获取视频数据(用于初始化编辑页面)
	 * @param int $id 视频ID
	 * @return array|null
	 * @throws \Exception
	 */
	public function get_edit_data(int $id): ?array
	{
		$video = $this->video_model->find($id)->fetch_array();
		if (is_array($video)) {
			//price
			$video['price'] = $video['kind'] == 2 && is_numeric($video['price']) ? sprintf('%.2f', floatval($video['price']) / 100) : '';

			//duration
			$video['duration'] = is_numeric($video['duration']) ? seconds2duration($video['duration']) : '';

			$video_ext = $this->video_ext_model->find($id)->fetch_array();
			$video_data = $this->video_data_model->find($id)->fetch_array();

			$data = $this->video_url_model->where('vid=:vid', ['vid' => $id])->query()->fetch_all_array();
			$video_url = [
				'url' => array_column($data, 'url'),
				'player' => array_column($data, 'player'),
				'group_name' => array_column($data, 'group_name')
			];

			return [
				'id' => $id,
				'video' => $video,
				'video_ext' => is_array($video_ext) ? $video_ext : [],
				'video_data' => is_array($video_data) ? $video_data : [],
				'video_url' => $video_url
			];
		}
		return null;
	}

	/**
	 * 编辑视频
	 * @param array $data post数据
	 * @return bool
	 */
	public function edit(array $data): bool
	{
		$id = $data['id'];
		$this->db->begin_transaction();

		try {
			//price
			$data['video']['price'] = $data['video']['kind'] === '2' ? floor(floatval($data['video']['price']) * 100) : null;

			//duration
			$duration = !empty($data['video']['duration']) ? duration2seconds($data['video']['duration']) : 0;
			$data['video']['duration'] = $duration > 0 ? $duration : null;

			//recommend
			$data['video']['recommend'] = $data['video']['recommend'] ?? 0;

			//status
			$data['video']['status'] = $data['video']['status'] ?? 0;

			//video
			$this->video_model->update($data['video'], $id);

			//video_ext
			if (isset($data['video_ext']) && is_array($data['video_ext'])) {
				$this->video_ext_model->update($data['video_ext'], $id);
			}

			//video_data
			if (isset($data['video_data']) && is_array($data['video_data'])) {
				$this->video_data_model->update($data['video_data'], $id);
			}

			//video_url
			if (isset($data['video_url']['url']) && is_array($data['video_url']['url'])) {

				$rows = $this->video_url_model->select('id')->where(['vid' => $id])->query()->fetch_all_array();
				$url_id_set = array_column($rows, 'id');

				foreach ($data['video_url']['url'] as $key => $value) {
					$value = trim($value);
					if ($value != '') {
						//insert or update record
						$url_id = array_shift($url_id_set);
						if (!is_null($url_id)) {
							$this->video_url_model->update([
								'player' => $data['video_url']['player'][$key],
								'url' => $value,
								'group_name' => $data['video_url']['group_name'][$key]
							], $url_id);
						} else {
							$this->video_url_model->insert([
								'vid' => $id,
								'player' => $data['video_url']['player'][$key],
								'url' => $value,
								'group_name' => $data['video_url']['group_name'][$key]
							]);
						}
					}
				}

				//删除多余的记录
				if (!empty($url_id_set)) {
					$this->video_url_model->where_in('id', $url_id_set)->delete();
				}
			}

			//video_file
			if (isset($data['hash_set'])) {
				$this->video_file_model->bind_vid($data['hash_set'], $id);
				$this->file_service->active_task_by_hash_set($data['hash_set']);
			}

			//update poster
			if ($data['poster']['from'] == '1') {
				$this->poster_service->update_file($id, \phpvod::$app->request->get_files('poster')['file']);
			} else {
				$this->poster_service->update_url($id, $data['poster']['url']);
			}

		} catch (\Exception) {
			$this->db->roll_back();
			return false;
		}

		if ($this->db->commit()) {
			//event
			event()->trigger(event::EVENT_UPDATE_FROM_MODULE, 'video', $id);
			return true;
		} else {
			return false;
		}
	}

	/**
	 * 删除视频
	 * @param int|array $id
	 * @return bool
	 * @throws \Exception
	 */
	public function delete(int|array $id): bool
	{
		is_int($id) && $id = [$id];

		//poster
		$poster_array = $this->video_model->select('poster')->where_in('id', $id)->query()->fetch_all_array();
		//platform & file_info
		$file_array = $this->video_file_model->select('platform,file_info,remote_key,remote_data')->where_in('vid', $id)->query()->fetch_all_array();

		$this->db->begin_transaction();
		try {
			//video
			$this->video_model->delete($id);
			//video_ext
			$this->video_ext_model->delete($id);
			//video_data
			$this->video_data_model->delete($id);
			//video_url
			$this->video_url_model->where_in('vid', $id)->delete();
			//video_file
			$this->video_file_model->where_in('vid', $id)->delete();
			//video_collect
			$this->video_collect_model->delete($id);
		} catch (\Exception) {
			$this->db->roll_back();
			return false;
		}

		if ($this->db->commit()) {
			//delete poster
			foreach ($poster_array as $row) {
				$this->poster_service->delete($row['poster']);
			}
			//delete file
			foreach ($file_array as $row) {
				$this->file_service->delete($row);
			}
			//event
			event()->trigger(event::EVENT_DELETE_FROM_MODULE, 'video', $id);
			return true;
		} else {
			return false;
		}
	}
}