<?php

namespace video\tag;

use venus\cache\handler;
use venus\database\database;
use video\library\config;
use video\model\purchase;
use video\service\mars;

class video_details
{
	private database $db;
	private handler $cache_handler;
	private array $mars_tag = [];

	/**
	 * 构造函数
	 * @throws \Exception
	 */
	public function __construct()
	{
		$this->db = \phpvod::$app->database;
		$this->cache_handler = \phpvod::$app->cache->get_handler();
		//mars_tag
		$mars_tag = config::get_instance()->get('mars_tag');
		is_array($mars_tag) && $this->mars_tag = $mars_tag;
		//helper
		helper('array');
	}

	/**
	 * 获取视频详情
	 * @param int $id 视频ID
	 * @param bool|null $use_cache 是否使用缓存，为null时则根据后台设置项（视频模块/设置/缓存设置/缓存视频数据）来决定是否使用缓存
	 * @throws \Exception
	 */
	public function fetch(int $id, ?bool $use_cache = null): array|false
	{
		is_null($use_cache) && $use_cache = config::get_instance()->get('cache', 'use_cache') == '1';

		$key = strtr(\video\event\handler::KEY_VIDEO_DETAILS, ['(:id)' => $id]);
		if ($use_cache === true) {
			$cache_data = $this->cache_handler->get($key);
			if (!is_null($cache_data)) {
				//hits
				$hits_key = strtr(\video\event\handler::KEY_VIDEO_HITS, ['(:id)' => $id]);
				$hits_value = (int)($this->cache_handler->get($hits_key) ?? 0);
				$cache_data['hits'] += $hits_value;
				return $cache_data;
			}
		}

		$query = "SELECT pv_video.*, pv_video_data.* 
			FROM pv_video LEFT JOIN pv_video_data ON pv_video.id=pv_video_data.id
			WHERE pv_video.id=:id AND pv_video.status=:status";
		$details = $this->db->query($query, ['id' => $id, 'status' => 1])->fetch_array();
		if (is_array($details)) {
			//video_ext
			$details['video_ext'] = $this->get_video_ext($details['category_id'], $id);
			//video_url
			$details['video_url'] = $this->get_video_url($id);

			if ($use_cache) {
				$this->cache_handler->set($key, $details);
			}
		}

		return $details;
	}

	/**
	 * 热度递增
	 * @param int $id 视频ID
	 * @param int $increment 递增值
	 * @param bool|null $use_cache 是否使用缓存，为null时则根据后台设置项（视频模块/设置/缓存设置/缓存视频数据）来决定是否使用缓存
	 * @param int $threshold 数据库及视频详情缓存更新阈值，即当缓存中的值大于或等于该值时即更新数据库
	 * @return void
	 * @throws \Exception
	 */
	public function hits_increment(int $id, int $increment = 1, ?bool $use_cache = null, int $threshold = 10): void
	{
		is_null($use_cache) && $use_cache = config::get_instance()->get('cache', 'use_cache') == '1';

		$hits_increment = $increment;

		if ($use_cache === true) {
			$key = strtr(\video\event\handler::KEY_VIDEO_HITS, ['(:id)' => $id]);
			$cache_data = (int)($this->cache_handler->get($key) ?? 0);
			$cache_data += $increment;
			if ($cache_data >= $threshold) {
				$hits_increment = $cache_data;
				$cache_data = 0;
			} else {
				$hits_increment = 0;
			}
			$this->cache_handler->set($key, $cache_data);
		}

		if ($hits_increment > 0) {
			//update database
			/***
			 * @var \video\model\video $video_model
			 */
			$video_model = model(\video\model\video::class);
			$video_model->update(['hits' => 'hits+' . $hits_increment], $id, ['hits']);

			//update details cache
			$this->update_details_cache($id, [
				['key' => 'hits', 'value' => $hits_increment, 'operator' => '+=']
			]);
		}
	}

	/**
	 * 更新视频详情缓存
	 * @param int $id 视频ID
	 * @param array $items 更新数据，格式：
	 * <code>[['key'=>'hits', 'value'=>1, 'operator'=>'+='],...]</code>
	 * @return void
	 */
	public function update_details_cache(int $id, array $items): void
	{
		$key = strtr(\video\event\handler::KEY_VIDEO_DETAILS, ['(:id)' => $id]);
		$cache_data = $this->cache_handler->get($key);
		if (!is_null($cache_data)) {
			foreach ($items as $item) {
				$cache_data[$item['key']] = match ($item['operator']) {
					'+=' => $cache_data[$item['key']] + $item['value'],
					'-=' => $cache_data[$item['key']] - $item['value'],
					default => $item['value']
				};
			}
			$this->cache_handler->set($key, $cache_data);
		}
	}

	/**
	 * 获取播放地址
	 * @param array $details
	 * @param int $group
	 * @param int $index
	 * @return string|array|null
	 */
	public function get_play_url(array $details, int $group, int $index): string|array|null
	{
		if (!isset($details['video_url'][$group]['url'][$index]['value']))
			return null;

		$value = $details['video_url'][$group]['url'][$index]['value'];
		if (is_string($value)) {
			return $value;
		} elseif (is_array($value)) {
			if ($value['platform'] == 'server') { //server
				return $value['file_info']['url'] ?? null;
			} elseif ($value['platform'] == 'mars_transcoder') { //mars_transcoder
				$remote_data = $this->get_remote_data($details, $group, $index, 'play');
				if (isset($remote_data['play'])) {
					return $remote_data['play']; //return string or array
				}
			}
		}

		return null;
	}

	/**
	 * 视频是否已购买
	 * @param int $video_id
	 * @return bool
	 * @throws \Exception
	 */
	public function has_purchased(int $video_id): bool
	{
		$identity = \phpvod::$app->user_auth->get_frontend_identity();
		if (is_array($identity)) {
			/* @var purchase $purchase_model */
			$purchase_model = model(purchase::class);
			$count = (int)$purchase_model->select('count(id)')->where(['member_id' => $identity['id'], 'video_id' => $video_id])->query()->fetch_column();
			if ($count > 0) {
				return true;
			}
		}
		return false;
	}

	/**
	 * 获取转码数据
	 * @param array $details
	 * @param int $group
	 * @param int $index
	 * @param string|array $names
	 * @return array|null
	 */
	public function get_remote_data(array $details, int $group, int $index, string|array $names): array|null
	{
		if (!isset($details['video_url'][$group]['url'][$index]['value']['remote_data']) ||
			!is_array($details['video_url'][$group]['url'][$index]['value']['remote_data'])
		) {
			return null;
		}

		$remote_data = $details['video_url'][$group]['url'][$index]['value']['remote_data'];
		return mars::get_url_by_name($names, $remote_data);
	}

	/**
	 * 获取自定义字段数据
	 * @param int $category_id 栏目ID
	 * @param int $id 视频ID
	 * @return array
	 */
	private function get_video_ext(int $category_id, int $id): array
	{
		//category_field
		$category_field = [];
		$field = config::get_instance()->get('field');
		foreach ($field as $value) {
			if (!in_array($category_id, $value['category_id']))
				continue;
			$category_field[] = $value['field'];
		}
		if (empty($category_field))
			return [];

		//query
		$select = implode(',', $category_field);
		$row = $this->db->query("SELECT $select FROM pv_video_ext WHERE id=:id", ['id' => $id])->fetch_array();
		if (!is_array($row))
			return [];

		//get
		$video_ext = [];
		foreach ($row as $key => $value) {

			if (in_array($field[$key]['input_type'], ['checkbox', 'textlist']) && str_contains($value, ',')) {
				$list = explode(',', $value);
			} else {
				$list = [$value];
			}

			foreach ($list as $item) {
				$value_label = $item;
				if (in_array($field[$key]['input_type'], ['select', 'radio', 'checkbox'])) {
					$index = array_search($item, array_column($field[$key]['items'], 'value'));
					$index !== false && $value_label = $field[$key]['items'][$index]['label'];
				}
				$video_ext[] = [
					'field' => $key,
					'field_label' => \phpvod::$app->language->key_is_exists('pv_video_ext.' . $key) ? lang('pv_video_ext.' . $key) : $field[$key]['label'],
					'value' => $item,
					'value_label' => $value_label
				];
			}
		}

		return $video_ext;
	}

	/**
	 * 获取视频数据(URL)
	 * @param int $id 视频ID
	 * @return array
	 * @throws \Exception
	 */
	private function get_video_url(int $id): array
	{
		$video_file = $this->get_video_file($id);

		return $this->db->query("SELECT player, url, group_name FROM pv_video_url WHERE vid=$id")->fetch_all_by_callback(function (array $row) use ($video_file) {
			$url = [];
			$list = explode("\n", $row['url']);
			foreach ($list as $item) {
				$item = trim($item);
				if (empty($item)) continue;
				if (str_contains($item, '|')) {
					[$label, $value] = explode('|', $item, 2);
				} else {
					$label = null;
					$value = $item;
				}

				if (array_key_exists($value, $video_file)) {
					$value = $video_file[$value];
				}

				$url[] = ['label' => $label, 'value' => $value];
			}
			$row['url'] = $url;
			return $row;
		});
	}

	/**
	 * 获取视频数据(FILE)
	 * @param int $id
	 * @return array
	 * @throws \Exception
	 */
	private function get_video_file(int $id): array
	{
		return $this->db->query("SELECT id, platform, file_info, remote_data FROM pv_video_file WHERE vid=$id")->fetch_all_by_callback(function (array $row) {
			!empty($row['file_info']) && $row['file_info'] = json_decode($row['file_info'], true);
			!empty($row['remote_data']) && $row['remote_data'] = json_decode($row['remote_data'], true);
			return ['key' => strtr('[file:(:id)]', ['(:id)' => $row['id']]), 'value' => $row];
		});
	}
}