<?php

namespace video\tag;

use venus\cache\depend\cache_depend;
use venus\cache\handler;
use venus\database\database;
use video\library\config;

class video_list
{
	private database $db;
	private handler $cache_handler;

	private array $pv_video_fields = [];
	private array $pv_video_data_fields = [];

	//params
	private array $params = [
		'select' => 'pv_video.*',
		'category_id' => null,
		'author_id' => null,
		'author_identity' => 1,
		'subject' => null,
		'kind' => null,
		'recommend' => null,
		'status' => 1,
		'ext' => [],
		'order' => null,
		'limit' => null,
		'current_page' => null,
		'url_pattern' => null,
	];

	//cache params
	private array $cache_params = [
		'use_cache' => null,
		'key_name' => null,
		'ttl' => 0,
		'depend_key' => null,
	];

	//sql
	private string $sql_select = 'pv_video.*';
	private string $sql_where = '';
	private array $sql_params = [];
	private string $sql_order = '';
	private string $sql_limit = '';
	private bool $left_join_data = false;
	private bool $left_join_ext = false;

	//result
	private array $data = [];
	private ?array $pagination_params = null;

	/**
	 * 构造函数
	 * @throws \Exception
	 */
	public function __construct()
	{
		$this->db = \phpvod::$app->database;
		$this->cache_handler = \phpvod::$app->cache->get_handler();

		//init fields
		foreach (['pv_video', 'pv_video_data'] as $table) {
			$key = $table . '_fields';
			$value = $this->cache_handler->get($key);
			if (is_null($value)) {
				$value = $this->db->query('SHOW COLUMNS FROM ' . $table . ' WHERE Field<>"id"')->fetch_column_array();
				$table == 'pv_video' && array_unshift($value, 'id');
				$this->cache_handler->set($key, $value);
			}
			$this->$key = $value; //init
		}
	}

	/**
	 * 获取视频列表数据
	 * @param string $select 查询字段
	 * @param int|null $category_id 栏目ID
	 * @param int|null $author_id 发布人ID
	 * @param int $author_identity 发布人身份：0会员，1管理员
	 * @param string|null $subject 标题
	 * @param int|null $kind 视频类别：0普通视频，1会员视频，2收费视频
	 * @param int|null $recommend 推荐：0未推荐，1推荐
	 * @param int $status 状态：0下架，1上架
	 * @param array $ext 自定义字段查询参数，格式：
	 * <p>[字段名=>字段值]，使用精确匹配（=）</p>
	 * <p>[字段名*=>字段值]，字段名以*结尾，则表示查找该字段包含字段值的记录（FIND_IN_SET）</p>
	 * <p>[字段名%=>字段值]，字段名以%结尾，则使用模糊查询（LIKE，字段值首尾加%）</p>
	 * @param array|null $order 排序，格式：[字段1=>ASC, 字段2=>DESC, ...]
	 * @param int|null $limit (每页)显示数量
	 * @param int|null $current_page 当前页码
	 * @param string|null $url_pattern 分页地址规则，页码参数用(:num)代替，如：page=(:num)
	 * @param bool|null $use_cache 是否使用缓存，为null时则根据后台设置项（视频模块/设置/缓存设置/缓存视频数据）来决定是否使用缓存
	 * @param string|null $key_name 缓存key名称，该名称必须具有唯一性，为null时由系统设定。
	 * @param int|string $ttl 缓存生存期，单位秒，如为字符串，可使用的单位有y/m/d/h/i，如：1d表示1天
	 * @param string|null $depend_key 缓存依赖key
	 * @return array
	 * @throws \Exception
	 */
	public function fetch(
		///////////params///////////
		//select
		string     $select = 'pv_video.*',
		//where
		?int       $category_id = null,
		?int       $author_id = null,
		int        $author_identity = 1,
		?string    $subject = null,
		?int       $kind = null,
		?int       $recommend = null,
		int        $status = 1,
		array      $ext = [],
		//order
		?array     $order = null,
		//limit
		?int       $limit = null,
		?int       $current_page = null,
		?string    $url_pattern = null,
		///////////cache params///////////
		?bool      $use_cache = null,
		?string    $key_name = null,
		int|string $ttl = 0,
		?string    $depend_key = null
	): array
	{
		//reset
		$this->reset();

		//params
		$this->params['select'] = $select;
		$this->params['category_id'] = $category_id;
		$this->params['author_id'] = $author_id;
		$this->params['author_identity'] = $author_identity;
		$this->params['subject'] = $subject;
		$this->params['kind'] = $kind;
		$this->params['recommend'] = $recommend;
		$this->params['status'] = $status;
		$this->params['ext'] = $ext;
		$this->params['order'] = $order;
		$this->params['limit'] = $limit;
		$this->params['current_page'] = $current_page;
		$this->params['url_pattern'] = $url_pattern;

		//cache params
		$this->cache_params['use_cache'] = is_null($use_cache) ? config::get_instance()->get('cache', 'use_cache') == '1' : $use_cache;
		$this->cache_params['key_name'] = $key_name;
		$this->cache_params['ttl'] = $ttl;
		$this->cache_params['depend_key'] = $depend_key;

		//get_cache_data
		$key = $this->get_cache_key();
		if ($this->cache_params['use_cache']) {
			$cache_data = $this->get_cache_data($key);
			if (!is_null($cache_data)) {
				return $cache_data;
			}
		}

		//select
		$this->select();

		//where
		$this->where();

		//order
		$this->order();

		//limit
		$this->limit();

		//query
		$this->query();

		$value = ['data' => $this->data, 'pagination_params' => $this->pagination_params];

		//set_cache_data
		if ($this->cache_params['use_cache']) {
			$this->set_cache_data($key, $value, $this->cache_params['ttl'], $this->cache_params['depend_key']);
		}

		return $value;
	}

	/**
	 * 获取缓存键
	 * @return string
	 */
	private function get_cache_key(): string
	{
		return is_null($this->cache_params['key_name']) || is_blank($this->cache_params['key_name']) ? 'video_list_' . md5(serialize($this->params)) : $this->cache_params['key_name'];
	}

	/**
	 * 读缓存
	 * @param string $key
	 * @return array|null
	 */
	private function get_cache_data(string $key): array|null
	{
		return $this->cache_handler->get($key);
	}

	/**
	 * 写缓存
	 * @param string $key
	 * @param array $value
	 * @param int|string $ttl
	 * @param string|null $depend_key
	 * @return void
	 * @throws \Exception
	 */
	private function set_cache_data(string $key, array $value, int|string $ttl, ?string $depend_key): void
	{
		//ttl
		$ttl = $this->int_ttl($ttl);
		if ($ttl === false) {
			throw new \Exception(lang('app.invalid_param', ['ttl']));
		}
		//depend
		$depend = !is_null($depend_key) ? new cache_depend($depend_key) : null;
		$this->cache_handler->set($key, $value, $ttl, $depend);
	}

	/**
	 * 将缓存生存期字符串转换为整型
	 * @param int|string $ttl
	 * @return int|false
	 */
	private function int_ttl(int|string $ttl): int|false
	{
		if (is_int($ttl)) {
			return $ttl;
		}

		$matches = [];
		if (preg_match('/^(\d+)([ymdhis]?)$/i', $ttl, $matches) === 1) {
			if (!is_numeric($matches[1]) || $matches[1] < 0)
				return false;

			$value = intval($matches[1]);

			if (empty($matches[2])) { //无单位
				return $value;
			}

			$unit = strtolower($matches[2]);

			//有单位
			return match ($unit) {
				'y' => $value * 31536000,
				'm' => $value * 2592000,
				'd' => $value * 86400,
				'h' => $value * 3600,
				'i' => $value * 60,
				default => $value
			};

		} else {
			return false;
		}
	}

	/**
	 * 获取字段全名(带表前缀)
	 * @param string $field 字段
	 * @return string|null
	 */
	private function get_field_name(string $field): string|null
	{
		$field_name = null;
		if (!is_blank($field)) {
			if (str_contains($field, '.')) {
				[$table] = explode('.', $field, 2);
				$table = strtolower(trim($table));
				$field_name = $field;
			} else {
				$table = in_array($field, $this->pv_video_fields) ? 'pv_video' : (in_array($field, $this->pv_video_data_fields) ? 'pv_video_data' : 'pv_video_ext');
				$field_name = $table . '.' . $field;
			}
			//join table
			if ($table == 'pv_video_data' && $this->left_join_data === false)
				$this->left_join_data = true;
			elseif ($table == 'pv_video_ext' && $this->left_join_ext === false)
				$this->left_join_ext = true;
		}
		return $field_name;
	}

	/**
	 * select
	 * @return void
	 */
	private function select(): void
	{
		if ($this->params['select'] == '*') {
			$this->sql_select = '*';
			$this->left_join_data = true;
			$this->left_join_ext = true;
			return;
		}
		$fields = explode(',', $this->params['select']);
		foreach ($fields as $key => $value) {
			$field_name = $this->get_field_name($value);
			if ($value != $field_name) {
				$fields[$key] = $field_name;
			}
		}
		$this->sql_select = implode(',', $fields);
	}

	/**
	 * where
	 * @return void
	 */
	private function where(): void
	{
		//status
		$this->sql_where = 'pv_video.status=' . $this->params['status'];


		//category_id
		if (!is_null($this->params['category_id'])) {
			$this->sql_where .= ' AND pv_video.category_id=:category_id';
			$this->sql_params['category_id'] = $this->params['category_id'];
		}

		//author_id & author_identity
		if (!is_null($this->params['author_id'])) {
			$this->sql_where .= ' AND pv_video.author_id=:author_id AND pv_video.author_identity=:author_identity';
			$this->sql_params['author_id'] = $this->params['author_id'];
			$this->sql_params['author_identity'] = $this->params['author_identity'];
		}

		//subject
		if (!is_null($this->params['subject'])) {
			$this->sql_where .= ' AND pv_video.subject LIKE :subject';
			$this->sql_params['subject'] = '%' . $this->params['subject'] . '%';
		}

		//kind
		if (!is_null($this->params['kind'])) {
			$this->sql_where .= ' AND pv_video.kind=:kind';
			$this->sql_params['kind'] = $this->params['kind'];
		}

		//recommend
		if (!is_null($this->params['recommend'])) {
			$this->sql_where .= ' AND pv_video.recommend=:recommend';
			$this->sql_params['recommend'] = $this->params['recommend'];
		}

		//ext
		if (!empty($this->params['ext'])) {
			$this->left_join_ext = true;
			foreach ($this->params['ext'] as $key => $value) {
				$key = trim($key);
				if (str_ends_with($key, '*')) { //FIND_IN_SET
					$key = substr($key, 0, -1);
					if (is_blank($key)) continue;
					$this->sql_where .= " AND FIND_IN_SET(:$key, pv_video_ext.$key)";
					$this->sql_params[$key] = $value;
				} elseif (str_ends_with($key, '%')) { //LIKE
					$key = substr($key, 0, -1);
					if (is_blank($key)) continue;
					$this->sql_where .= " AND pv_video_ext.$key LIKE :$key";
					$this->sql_params[$key] = '%' . $value . '%';
				} else { //EQUALS
					if (is_blank($key)) continue;
					$this->sql_where .= " AND pv_video_ext.$key=:$key";
					$this->sql_params[$key] = $value;
				}
			}
		}
	}

	/**
	 * order
	 * @return void
	 */
	private function order(): void
	{
		$this->sql_order = '';
		if (is_array($this->params['order'])) {
			$order = '';
			foreach ($this->params['order'] as $field => $sort) {
				$field_name = $this->get_field_name($field);
				$order .= empty($order) ? $field_name . ' ' . $sort : ',' . $field_name . ' ' . $sort;
			}
			!empty($order) && $this->sql_order = ' ORDER BY ' . $order;
		}
	}

	/**
	 * limit
	 * @return void
	 */
	private function limit(): void
	{
		//limit
		if (is_null($this->params['limit'])) {
			$this->sql_limit = '';
		} else {
			if (!is_null($this->params['current_page']) && !is_null($this->params['url_pattern'])) { //分页
				$total_items = (int)$this->db->query('SELECT count(pv_video.id) FROM pv_video LEFT JOIN pv_video_ext ON pv_video.id=pv_video_ext.id WHERE ' . $this->sql_where, $this->sql_params)->fetch_column();

				//check current_page
				$max_page = $total_items > 0 ? ceil($total_items / $this->params['limit']) : 1;
				$this->params['current_page'] < 1 && $this->params['current_page'] = 1;
				$this->params['current_page'] > $max_page && $this->params['current_page'] = $max_page;

				$this->pagination_params = [
					'total_items' => $total_items,
					'items_per_page' => $this->params['limit'],
					'current_page' => $this->params['current_page'],
					'url_pattern' => $this->params['url_pattern']
				];
				$offset = ($this->params['current_page'] - 1) * $this->params['limit'];
			} else {
				$offset = 0;
			}
			$this->sql_limit = ' LIMIT ' . $offset . ',' . $this->params['limit'];
		}
	}

	/**
	 * query
	 * @return void
	 */
	private function query(): void
	{
		$form = 'pv_video';
		$this->left_join_data === true && $form .= ' LEFT JOIN pv_video_data ON pv_video.id=pv_video_data.id';
		$this->left_join_ext === true && $form .= ' LEFT JOIN pv_video_ext ON pv_video.id=pv_video_ext.id';

		$sql = 'SELECT ' . $this->sql_select . ' FROM ' . $form . ' WHERE ' . $this->sql_where . $this->sql_order . $this->sql_limit;
		$this->data = $this->db->query($sql, $this->sql_params)->fetch_all_array();
	}

	/**
	 * reset
	 * @return void
	 */
	private function reset(): void
	{
		//params
		$this->params = [
			'select' => 'pv_video.*',
			'category_id' => null,
			'author_id' => null,
			'author_identity' => 1,
			'subject' => null,
			'kind' => null,
			'recommend' => null,
			'status' => 1,
			'ext' => [],
			'order' => null,
			'limit' => null,
			'current_page' => null,
			'url_pattern' => null,
		];

		//cache params
		$this->cache_params = [
			'use_cache' => null,
			'key_name' => null,
			'ttl' => 0,
			'depend_key' => null,
		];

		//sql
		$this->sql_select = 'pv_video.*';
		$this->sql_where = '';
		$this->sql_params = [];
		$this->sql_order = '';
		$this->sql_limit = '';
		$this->left_join_data = false;
		$this->left_join_ext = false;

		//result
		$this->data = [];
		$this->pagination_params = null;
	}
}